/*
   AmziCGI.C -- Standard CGI Script Start-Up Program

   Link with AmziSub.C

   Copyright (c) 1996 Amzi! inc.  All Rights Reserved.
   See license.htm for Amzi! License Agreement

   This program can be compiled to run in standard CGI
   environments, or Windows-CGI environments.  Change
   the #define for the appropriate environment.

   Under Win95 this is built as a Windows application.
   Under NT/standard CGI this is built as a console
   application. Under Unix/standard CGI this is built
   as a regular program.

   Relies on _WIN32 and __unix__ defines
*/

/*#define STDCGI*/			/* define either STDCGI or WINCGI */
/*#define WINCGI*/			/* here or in your make file */

/*#define DEBUG*/

#ifdef _WIN32
#include <windows.h>
#else
#define	_MAX_PATH	512
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* The header file required for linking with Prolog.  It contains
   definitions and common variables. */

#include "amzi.h"

extern RC InitPreds(ENGid, void*);
extern void slashslash1(char *s);
extern void slashslash2(char* sout, const char* sin);

TF EXPFUNC p_cgiPurgeFactFiles(ENGid);

ENGid CurEng;

/* We use the Win-CGI standard names for environment variables.
   These are asserted as Prolog facts either named 'cgi' or 
   'system'.  For example, HTTP_REFERER is asserted as
   cgi('Referer', <value>). */

char *envlist[][3] = 
{
	"AUTH_NAME", "'Authentication Realm", "cgi",
	"AUTH_USER", "'Authentication Username'", "cgi",
	"AUTH_TYPE", "'Authentication Method'", "cgi",
	"CGI_VERSION", "'CGI Version'", "cgi",
	"CONTENT_FILE", "'Content File'", "cgi",
	"CONTENT_LENGTH", "'Content Length'", "cgi",
	"CONTENT_TYPE", "'Content Type'", "cgi",
   	"GATEWAY_INTERFACE", "'Gateway Interface'",	"cgi",
	"HTTP_FROM", "'From'", "cgi",
	"HTTP_REFERER", "'Referer'", "cgi",
	"PATH_INFO", "'Logical Path'", "cgi",
	"PATH_TRANSLATED", "'Physical Path'", "cgi",
	"QUERY_STRING", "'Query String'", "cgi",
 	"REMOTE_ADDR", "'Remote Address'",	"cgi",
	"REMOTE_HOST", "'Remote Host'",	"cgi",
	"REQUEST_METHOD", "'Request Method'", "cgi",
	"REQUEST_RANGE", "'Request Range'", "cgi",
	"SCRIPT_NAME", "'Executable Path'", "cgi",
	"SERVER_ADMIN", "'Server Admin'",	"cgi",
	"SERVER_NAME", "'Server Name'",	"cgi",
	"SERVER_PORT", "'Server Port'",	"cgi",
	"SERVER_PROTOCOL", "'Server Protocol'", "cgi",
	"SERVER_SOFTWARE", "'Server Software'",	"cgi",
	"USER_AGENT", "'User Agent'",	"cgi",

	"DEBUG_MODE", "'Debug Mode'", "system",
	"GMT_OFFSET", "'GMT Offset'", "system",

	NULL, NULL
};

/* Converts nn to character */

char x2c(char *what) 
{
    register char digit;

    digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A')+10 : (what[0] - '0'));
    digit *= 16;
    digit += (what[1] >= 'A' ? ((what[1] & 0xdf) - 'A')+10 : (what[1] - '0'));
    return(digit);
}

/* Converts &nn's to characters and + signs to spaces in 
   url encoded strings */

void decode_url(char *url) 
{
    register int x,y;

    for(x=0,y=0;url[y];++x,++y) 
	{
        if((url[x] = url[y]) == '%') 
		{
            url[x] = x2c(&url[y+1]);
            y+=2;
        }
		if(url[x] == '+')
			url[x] = ' ';
    }
    url[x] = '\0';
}

/* Extracts all the facts from the URL encoded input stream and asserts
   them to the Prolog dynamic database in the form 
   "fact(<attribute>, <value>). */

void assertFacts()
{
	char	*p, *q, *u, *tp, *val, *buf, factbuf[4096];
	long	buflen;
	TERM	term;
	TF		tf;


	/* Get the length of the encoded URL */
	val = getenv("CONTENT_LENGTH");
	if (val == NULL) return;

	/* Add two for two nuls and malloc space */
	buflen = atol(val)+2;
	buf = malloc(buflen);
	fgets(buf, buflen, stdin);
	buf[buflen-1] = '\0';			/* Add an extra nul */

	p = buf;
	while (*p != '\0')
	{
		/* Create a query of the form "cgiAddNewFact(fact, <attribute>, <value>)" */
		strncpy(factbuf, "cgiAddNewFact(fact, '", 4096);

		/* Grab the attribute (up to the equal = sign) */
		q = strchr(p, '=');
		*q = '\0';
		tp = q + 1;
		decode_url(p);

		/* If its a multi-valued field hack off the _<num> part */
		u = strrchr(p, '_');
		if (u != NULL)
			if (isdigit(*(u+1))) *u = '\0';

		/* Now append the attribute name */
		strncat(factbuf, p, 4096);
		strncat(factbuf, "', '", 4096);
 		
		/* Grab the value (up to the & or nul) */
		p = tp;
		q = strchr(p, '&');
		if (q == NULL)		/* End of string */
		{
			q = &buf[buflen-1];
			tp = q;
		}
		else				/* More values left to process */
		{
			*q = '\0';
			tp = q + 1;
		}
		decode_url(p);

		/* Now append the value */
		strncat(factbuf, p, 4096);
		strncat(factbuf, "')", 4096);

		/* Assert it in the Prolog logic-base */
		if (strlen(p) > 0) tf = lsExecStr(CurEng, &term, factbuf);

		p = tp;
	}

	/* Free our url buffer */
	free(buf);
}


#ifdef	STDCGI
int main(int argc, char *argv[])
{
	char	*p, *val;
	int		i;
#endif
#ifdef	WINCGI
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdParam, int nCmdShow)
{
	char	ininame[_MAX_PATH];
	char	buf[250];
	char	*sFirstArg;
#endif

	char	exename[_MAX_PATH];
	char	xplname[_MAX_PATH];
	char	strbuf[4096];
	char	*sExeName, *sColon, *sSlash, *sDot;
	int		rc, exitrc;
	TF		tf;
#ifdef	_WIN32
	MSG		msg;
#endif

#ifdef __unix__
	umask(0);
#endif

	exitrc = 0;

	/* Get the name of the program to run */
	/* ---------------------------------- */

	/* First check the name of this .EXE file; run the .XPL file of the
	   same name. */

#ifdef STDCGI
	strcpy(exename, argv[0]);
#else
	GetModuleFileName(NULL, exename, _MAX_PATH);
#endif

	sColon = strrchr(exename,':');           /* remove path from file name */
	sSlash = strrchr(exename,'\\');
	if (sSlash == NULL) sSlash = strrchr(exename,'/');       /* Unix path? */
   
	if (sColon == NULL && sSlash == NULL) sExeName = exename;
	else if (sSlash == NULL) sExeName = sColon + 1;
	else sExeName = sSlash + 1;

	sDot = strrchr(sExeName,'.');
	if (sDot) *sDot = '\0';

	strcpy(xplname, sExeName);


	/* Initialize the Prolog environment */
	/* --------------------------------- */

#ifdef DEBUG
	printf("lsInit %s\n", xplname);
#endif
	rc = lsInit(&CurEng, xplname);
	if (rc != 0)
	{
#ifdef STDCGI
		printf("Error #%d initializing Amzi! Logic Server", rc);
#else
		wsprintf(buf, "Error #%d initializing Amzi! Logic Server", rc);
		MessageBox(NULL, buf, "AMZICGI Error", MB_OK);
#endif
		lsClose(CurEng);
		exit(rc);
	}

	/* Initialize the extended predicates in amzisub.c */
	rc = InitPreds(CurEng, NULL);
	if (rc != 0)
	{
#ifdef STDCGI
		printf("Error #%d loading Amzi! Logic Server extended predicates", rc);
#else
		wsprintf(buf, "Error #%d loading Amzi! Logic Server extended predicates", rc);
		MessageBox(NULL, buf, "AMZICGI Error", MB_OK);
#endif
		lsClose(CurEng);
		exit(rc);
	}


	/* Load the .xpl file */
	/* ------------------ */

#ifdef DEBUG
	printf("lsLoad %s\n", xplname);
#endif
	rc = lsLoad(CurEng, xplname);
	if (rc != 0)
	{
#ifdef STDCGI
		printf("Error #%d loading Amzi! Logic Server XPL file", rc);
#else
		wsprintf(buf, "Error #%d loading Amzi! Logic Server XPL file", rc);
		MessageBox(NULL, buf, "AMZICGI Error", MB_OK);
#endif
		lsClose(CurEng);
        exit(rc);
	}


	/* Set Up the Dynamic Database */
	/* --------------------------- */

#ifdef DEBUG
	printf("Asserting...\n");
#endif

#ifdef STDCGI

	/* Assert all the environment variables on our list */
	i = 0;
	p = envlist[i][0];
	while (p != NULL)
	{
		val = getenv(p);

		/* Skip the ones that have no value */
		if (val != NULL)
		{
			strcpy(strbuf, envlist[i][2]);
			strcat(strbuf, "(");
			strcat(strbuf, envlist[i][1]);
			strcat(strbuf, ", '");
			strcat(strbuf, val);
			strcat(strbuf, "')");

			rc = lsAssertaStr(CurEng, strbuf);
		}

		i++;
		p = envlist[i][0];
	}

	/* Assert all the facts decoded from stdin */
	assertFacts();

#else

	/* Assert the INI file name as system($Input File$, <INIFilePath>) */
	sFirstArg = strtok(lpszCmdParam, " ");
	strcpy(ininame, sFirstArg);
	slashslash1(ininame);
	strcpy(strbuf, "system($Input File$, $");
	strcat(strbuf, ininame);
	strcat(strbuf, "$)");
	rc = lsAssertaStr(CurEng, strbuf);

#endif


	/* Finally call main/0 of the loaded program */
	/* ----------------------------------------- */

#ifdef DEBUG
	printf("lsMain\n");
#endif
	tf = lsMain(CurEng);	/* Note use lsExecStr if making multiple calls */

	if (tf == FALSE) 
	  {
#ifdef DEBUG
	    printf("FAILED\n");
#endif
	    exitrc = 1;
	  }

	/* Close down the Prolog environment, freeing memory */
	/* ------------------------------------------------- */

	lsClose(CurEng);

#ifdef	_WIN32
	/* Let Windows run so all ends well */
	SetTimer(NULL,1,100,NULL);
	while ( PeekMessage((MSG FAR*)&msg, NULL, WM_TIMER, WM_TIMER, PM_REMOVE) );
#endif

	return exitrc;
}
