% ------------------------------------------------------------------- %
%                 The Prolog Inference Engine                         %
% ------------------------------------------------------------------- %

:- op(900, fy, [?, bug]).

:- op(790, fx, if).     	% prefix operator
:- op(780, xfx, then).  	% infix operator
:- op(775, xfy, or).    	% infix that can be linked
:- op(770, xfy, and).   	% infix that can be linked
:- op(700, xfx, <=).		% infix operator
:- op(700, xfx, include).	% infix operator


%
% cgiStart
%
% This is called by main before anything else is done, so you can
% setup a log file (logging is enabled only if you have defined a
% log file).  Also, when debugging in the listener, you must assert
% the input file (this is normally done by amzicgi.c).  Its handy to
% turn on debug mode in WebSite and just reuse one of their files.
%
cgiStart :-
   % This is a convenient way to test the engine in the Amzi! IDE
%   asserta(system('Input File', 'C:\\WebSite\\cgi-temp\\444ws.ini')),

   % These must be asserted when the logic-base is compiled
%   asserta(system('Log File', '/usr/web/amzi/WWW/trace.htm')),
%   asserta(system('Log File URL', 'http://amzi.harvard.net/trace.htm')),
%   asserta(system('Results File', $/usr/web/amzi/logs/results.pro$)),
%   asserta(system('Initial Form', $/usr/web/amzi/WWW/rptprob.htm$)),
%   asserta(system('Temp Directory', $/usr/web/amzi/temp/$)),

%   asserta(system('Log File', 'C:\\WebLS\\logs\\trace.htm')),
%   asserta(system('Log File URL', 'file:///C:\\WebLS\\logs\\trace.htm')),
%   asserta(system('Results File', $C:\\WebLS\\logs\\results.pro$)),
%   asserta(system('Initial Form', $C:\\WebLS\\html\\rptprob.htm$)),
%   asserta(system('Temp Directory', $C:\\WebLS\\temp\\$)),
%   asserta(system('Body Args', $bgcolor=#CEFAFF text=#000000$)),
%   asserta(system('Title', $Amzi! Problem Resolver$)),
%   asserta(system('Footer', $<FONT SIZE=-1>Copyright &#169;1996 Amzi! inc. All Rights Reserved.$)),
%   asserta(system('Form Action', 'Executable Path')),
%   asserta(system('Goal', 'problem')),

   true.

%
% cgiMain
%
% The start of the Prolog Engine.
%
cgiMain :-
   % Comment out the consult when the logic-base is compiled
   % and linked in
   consult('logicbse.pro'),
   cgiErrorCheck,

   % Get the form method and act accordingly
   cgi('Request Method', RM),
   processMethod(RM).

%
% processMethod(Type)
%
% This is the heart of the system.  The first time through we use a
% GET request, to which the standard problem report form is sent.
% On subsequent times through we use POST requests.
%
processMethod('GET') :-
   cgiLog($Processing GET method (first time through)\n$),
   sendProblemReport.

processMethod('POST') :-
   %
   % The current state is hidden in the name of the submit button, which always
   % has the value 'Submit'.  All the fact(Key, Value) predicates are written
   % out to the state file.  openFact creates the file first time through and
   % consults it on subsequent times through
   %
   ( fact(State, 'Submit') ; fact(State, 'Submit Query') ),
   openFactFile(State),
   !,

   %
   % Start the log file here after all the facts have been loaded because
   % opening the log file outputs all the system, cgi and fact predicates
   %
   (system('Input File', INIFile) ->
      cgiLog([$Processing input file $, INIFile, $\n$])
   ;
      cgiLog($Processing input file from stdin\n$)
   ),

   cgiLog([$Opened fact file: $, State, $\n$, 
           $Processing POST method (subsequent times through)\n$]),

   system('Goal', Goal),
   processPost(Goal, RuleName).

processMethod(X) :-
   cgiError([$\nERROR: GET/POST method processing failed --or-- Unrecognized method $, X]).

%
% First time through we send out the problem report for the user to fill in
% to try and gather a bit of data in the first place.
%
sendProblemReport :-
   cgiSend($Content-type: text/html$),
   cgiSend($$),
   system('Initial Form', FormFile),
   cgiLog([$Sending HTML file: $, FormFile]),
   cgiSendLocalHTMLFile(FormFile).

%
% Process a post request.  First we run the rule-base to see if an answer 
% can be reached without asking the user further questions
%
processPost(Goal, Name) :-
   cgiLog($\n\n---> Calling logic-base for the first time\n\n$),
   getAV(Goal, Name, [], [], 0),
   !,
   cgiLog([$\n\n---> Logic-base succeeded in finding an answer: $, Name, $\n$]),
   writeFactFile,
   writeResultsLog(Name),
   deleteFactFile,
   outputAnswer(Name).
%
% If no rules suceeded we try again, this time we collect all the
% questions together for a particular hypothesis to ask the user.
%
processPost(Goal, Name) :-
   cgiLog($\n\n--->Calling logic-base for the second time\n\n$),
   getAV(Goal, Name, [], NeedsList, 0),
   !,
   cgiLog([$\n\n---> Logic-base needs more facts, current hypothesis is: $, Name, $\n$,
           $     The hypothesis needs these facts: $]),
   needsLog(NeedsList), cgiLog($\n$),
   needsAskAll(NeedsList),
   writeFactFile.
%
% If there is nothing to ask and no answer, we send out a default answer
% and (someday) e-mail the problem to technical support.
%
processPost(_, noAnswer) :-
   outputAnswer(noAnswer),
   writeResultsLog(noAnswer),
   deleteFactFile.

%
% Simple output of the needs list to the log file (called by ProcessPost)
%
needsLog([]).
needsLog([Attr : Val | Rest]) :-
   cgiLog([Attr, $ $]),
   needsLog(Rest).

%
% Writes the results of the inference to a log file
%
writeResultsLog(Answer) :-
   openResultsLog(Handle),
   write(Handle, $session($),
   write(Handle, Answer),
   write(Handle, $, $),
   findall(X : Y, fact(X, Y), FL),
   write(Handle, FL),
   write(Handle, $).\n$),
   fclose(Handle).
writeResultsLog(_).

openResultsLog(Handle) :-
   system('Results File', ResultsFile),   
   set_mode(file_errors, off),
   repeat,
   fopen(Handle, ResultsFile, a),
   set_mode(file_errors, on).


% ---------------------- Logic-Base Execution ----------------------- %

%
% getAV, getNotAV, prove and checkFact implement a backward chaining
% inference engine.  All of these predicates pass a 'needs' list up
% and down.  This list contains attribute:value pairs that are needed
% to prove a particular hypothesis.  If the needs list is empty, then
% the hypothesis is true.
%

%
% Tries to find a rule for the specified attribute, Attr.  Once it
% finds a rule, it tries to prove its conditions.  getNotAV is used when
% trying to check if a value is not equal.
%
getAV( Attr, Value, Na, N, D ) :-
   if Conditions then Attr = Value,
   logCallFail([Attr, $ = $, Value, $\n$], D),
   DD is D + 3,
   prove(Conditions, Na, N, DD),
   logExitRedo([Attr, $ = $, Value, $\n$], D).
getAV( Attr, Value, Na, N, D ) :-
   not( if _ then Attr = _ ),
   checkFact(Attr, Value, Na, N, D).

getNotAV( Attr, Value, Na, N, D ) :-
   if Conditions then Attr = X,
   Value \= X,
   logCallFail([Attr, $ \\= $, Value, $\n$], D),
   DD is D + 3,
   prove(Conditions, Na, N, DD),
   logExitRedo([Attr, $ \= $, Value, $\n$], D).
getNotAV( Attr, Value, N, N, D ) :-
   fact(Attr, X), !,
   Value \= X.
getNotAV( Attr, Value, Na, N, D ) :-
   not( if _ then Attr = _ ),
   checkFact( Attr, X, Na, N, D).

%
% prove handles the operators:  =, \=, and, or
%
prove( Condition and Rest, Na, N, D ) :-
   prove(Condition, Na, Nnext, D),
   prove(Rest, Nnext, N, D).
prove( Conditions1 or Conditions2, Na, N, D ) :-
   prove(Conditions1, Na, N, D)
   ;
   prove(Conditions2, Na, N, D).
prove( Attr = Value, Na, N, D) :-
   cgiLog(tab(D)), cgiLog([$Trying $, Attr, $ = $, Value, $\n$]),
   getAV(Attr, Value, Na, N, D),
   cgiLog(tab(D)), cgiLog([$Matching $, Attr, $ = $, Value, $\n$]).
prove( Attr \= Value, Na, N, D ) :-
   cgiLog(tab(D)), cgiLog([$Trying $, Attr, $ \\= $, Value, $\n$]),
   getNotAV(Attr, Value, Na, N, D),
   cgiLog(tab(D)), cgiLog([$Matching $, Attr, $ \\= $, Value, $\n$]).
prove( Attr < Value, Na, N, D ) :-
   getAV(Attr, Aval, Na, N, D),
   cgiLog(tab(D)), cgiLog([$Trying $, Attr, $ &lt; $, Value, $\n$]),
   Aval @< Value,
   cgiLog(tab(D)), cgiLog([$Matching $, Attr, $ &lt; $, Value, $\n$]).
prove( Attr > Value, Na, N, D ) :-
   getAV(Attr, Aval, Na, N, D),
   cgiLog(tab(D)), cgiLog([$Trying $, Attr, $ &gt; $, Value, $\n$]),
   Aval @> Value,
   cgiLog(tab(D)), cgiLog([$Matching $, Attr, $ &gt; $, Value, $\n$]).
prove( Attr <= Value, Na, N, D ) :-
   getAV(Attr, Aval, Na, N, D),
   cgiLog(tab(D)), cgiLog([$Trying $, Attr, $ &lt;= $, Value, $\n$]),
   Aval @=< Value,
   cgiLog(tab(D)), cgiLog([$Matching $, Attr, $ &lt;= $, Value, $\n$]).
prove( Attr >= Value, Na, N, D ) :-
   getAV(Attr, Aval, Na, N, D),
   cgiLog(tab(D)), cgiLog([$Trying $, Attr, $ &gt;= $, Value, $\n$]),
   Aval @>= Value,
   cgiLog(tab(D)), cgiLog([$Matching $, Attr, $ &gt;= $, Value, $\n$]).
prove( Attr include Value, Na, N, D ) :-
   getAV(Attr, Aval, Na, N, D),
   cgiLog(tab(D)), cgiLog([$Trying $, Attr, $ include $, Value, $\n$]),
   member(Value, Aval),
   cgiLog(tab(D)), cgiLog([$Matching $, Attr, $ include $, Value, $\n$]).

% Allows any Prolog term on the if side
%prove( X, Na, N, D ) :-
%   call(X).

%
% Checks the value of a fact and succeeds or fails
%
checkFact(Attr, Value, N, N, D) :-
   fact(Attr, X),         % is there a known value for this attribute?
   !,                     % if so dont ask again
   X = Value.             % succeed or fail based on the expected value.
checkFact(Attr, Value, Na, [Attr:Val|Na], D).

%
% Tracing predicates for the above predicates.  Note how the second
% clause of each fails so as to not interrupt Prolog's normal
% backtracking flow.
%
logCallFail(Stuff, D) :-
   cgiLog(tab(D)), cgiLog($Trying $), cgiLog(Stuff).
logCallFail(Stuff, D) :-
   cgiLog(tab(D)), cgiLog($Failing $), cgiLog(Stuff),
   fail.
logExitRedo(Stuff, D) :-
   cgiLog(tab(D)), cgiLog($Matching $), cgiLog(Stuff).
logExitRedo(Stuff, D) :-
   cgiLog(tab(D)), cgiLog($Retrying $), cgiLog(Stuff),
   fail.


% -------------- Outputting HTML Headers and Footers ---------------- %

sendHeader :-
   cgiSend($Content-type: text/html$),
   cgiSend($$),
   system('Title', Title),
   cgiSend([$<HTML><HEAD><TITLE>$, Title, $</TITLE></HEAD>$]),
   (system('Body Args', BArgs) ->
      true
   ;
      BArgs = $bgcolor=#FFFFEE text=#000000$
   ),
   cgiSend([$<BODY $, BArgs, $><H1>$, Title, $</H1>$]).

sendFooter :-
   (system('Footer', Footer) ->
      true
   ;
      Footer = $$
   ),
   cgiSend([$<P><HR><CENTER>$, Footer, $<P><FONT SIZE=-1><I>WebLS&#153; Freeware by <A HREF="http://www.amzi.com/">Amzi!</A></I>$]), 
   (system('Log File URL', LFU) -> 
      cgiSend([$<BR>Press <A HREF="$, LFU, $">here</A> (then your browser's reload button) for Logic-Base Debugging Trace$]) 
      ; 
      true
   ),
   cgiSend($</CENTER></BODY></HTML>$),
   cgiSend($$).


% ------------------ Outputting Questions (Asks) -------------------- %

%
% Outputs the HTML header then processes the list of unknown facts.
%
needsAskAll([]) :- !.
needsAskAll(Unknown) :-
   sendHeader,
   system('Form Action', CGIKey),
   cgi(CGIKey, EP),
   cgiSend([$<FORM METHOD="POST" ACTION="$, EP, $">$]),
   needsAsk(Unknown, []).

%
% Actually walks the unknown list calling outputAsk for each fact.  After all the
% facts the HTML footer is outputted.  If there is no outputAsk defined, then an
% error is thrown making debugging easier.
%
needsAsk([], AL) :-
   system('Fact File', FileName),
   cgiSend([$<P><INPUT NAME="$, FileName, $" TYPE="Submit">$]),
   cgiSend($</FORM>$),
   sendFooter.
needsAsk([Attr : _ | Rest], AL) :-
   needsAskOut(Attr, AL, AL2),
   needsAskRelated(Attr, AL2, AL3),
   needsAsk(Rest, AL3).

needsAskOut(Attr, AL, AskedList) :-
   member(Attr, AL).
needsAskOut(Attr, AL, [Attr | AL]) :-
   outputAsk(Attr).

%
% For each unknown fact, we also check if there are any related facts.
% If so, we ask about them now.  We also take care not to ask for facts we've
% already asked for.
%
needsAskRelated(Attr, AL, AskedList) :-
   getAttrInfo(Attr, related, RelList),
   cgiLog([$     Related facts are: $, RelList]),
   needsAskRelatedOut(RelList, AL, AskedList).
needsAskRelated(_, AskedList, AskedList).

needsAskRelatedOut([], AskedList, AskedList).
needsAskRelatedOut([Attr | Rest], AL, AskedList) :-
   fact(Attr, _),         % ask only if we don't already have an answer
   needsAskRelatedOut(Rest, AL, AskedList).
needsAskRelatedOut([Attr | Rest], AL, AskedList) :-
   member(Attr, AL),      % ask only if not already asked
   needsAskRelatedOut(Rest, AL, AskedList).
needsAskRelatedOut([Attr | Rest], AL, AskedList) :-
   outputAsk(Attr),
   needsAskRelatedOut(Rest, [Attr | AL], AskedList).

%
% Outputs the question specified for the attribute 
%
outputAsk(Attr) :-
   getAttrInfo(Attr, prompt, Prompt),
   getAttrInfo(Attr, ask, Ask),
   getAttrInfo(Attr, length, Length),
   outputAskType(Attr, Ask, Prompt, Length).
outputAsk(Attr) :-
   attr(Attr, _), !,
   throw( [$ask= is not defined for attribute $, Attr] ).
outputAsk(Attr) :-
   throw( [$attr($, Attr, $...) is not defined in your logic-base$] ).

outputAskType(Attr, field, Prompt, Length) :-
   !,
   cgiAskField(Attr, Prompt, Length).
outputAskType(Attr, menu(ChoiceList), Prompt, _) :-
   !,
   attr(Attr, AttrList),
   (member(multivalued, AttrList) ->
      cgiAskMenu(Attr, Prompt, ChoiceList, multiple)
   ;
      cgiAskMenu(Attr, Prompt, ChoiceList, single)
   ).
outputAskType(Attr, yesno, Prompt, _) :-
   !,
   cgiAskYesNo(Attr, Prompt).

%
% getAttrInfo retrieves fields from the attribute definitions.  If no
% value is specified, then a default may be returned.
%

% prompt = 
% default is "What is the value for <Attr>"

getAttrInfo(Attr, prompt, Prompt) :-
   attr(Attr, InfoList),
   member(prompt = Prompt, InfoList), !.
getAttrInfo(Attr, prompt, Prompt) :-
   attr(Attr, InfoList), !,
   string_atom(SAttr, Attr),
   strcat($What is the value for $, Attr, Prompt).

% length =
% default is none
getAttrInfo(Attr, length, Length) :-
   attr(Attr, InfoList),
   member(length = Length, InfoList), !.
getAttrInfo(Attr, length, none) :- !.

% related = [<list>]
% default is [] (the empty list)
getAttrInfo(Attr, related, RelatedList) :-
   attr(Attr, InfoList),
   member(related = RelatedList, InfoList), !.
getAttrInfo(Attr, related, []) :- !.

% gets the value for a required field, otherwise throws an error
getAttrInfo(Attr, Name, Info) :-
   attr(Attr, InfoList),
   member(Name = Info, InfoList), !.
getAttrInfo(Attr, Name, _) :-
   attr(Attr, _), !,
   throw( [$attr $, Attr, $ must specify a value for $, Name] ).
getAttrInfo(Attr, Name, Info) :-
   throw( [$attr($, Attr, $ ...) is not defined in your logic-base$] ).


% ------------------ Outputting Answers and Notes ------------------- %

%
% Outputs the specified answer.  Handles text, url and htmlFile type
% answers.  For text answers, also outputs any notes.  Missing
% information is handled here by using throw.
%
outputAnswer(Name) :-
   answer(Name, InfoList),
   member(text = AnswerList, InfoList), !,
   (member(note = NoteList, InfoList) -> 
      sendAnswer(AnswerList, NoteList)
   ;
      sendAnswer(AnswerList, [])
   ).
outputAnswer(Name) :-
   answer(Name, InfoList),
   member(url = URL, InfoList), !,
   cgiSendURL(URL).
outputAnswer(Name) :-
   answer(Name, InfoList),
   member(htmlFile = File, InfoList), !,
   cgiSendLocalHTMLFile(File).
outputAnswer(Name) :-
   member(Name, [noAnswer]), !,
   sendAnswer([$No answer was found.$], []).
outputAnswer(Name) :-
   answer(Name, InfoList), !,
   system('Goal', Goal),
   throw( [$No text=, url= or htmlFile= specified for $, Goal, $ $, Name, $ in list $, InfoList] ).
outputAnswer(Name) :-
   throw( [$answer($, Name, $ ...) is not defined in your logic-base$] ).

%
% Outputs a single note separating them by paragraph markers.  If 
% called to output a special system note and one is not specified,
% then this predicate succeeds, otherwise an error is thrown.
%
outputNote(Name) :-
   note(Name, InfoList),
   member(text = AnswerList, InfoList), !,
   cgiSend($<P>$),
   cgiSend(AnswerList).
outputNote(Name) :-
   member(Name, [systemError, answerHeader, answerFooter]), !.
outputNote(Name) :-
   throw( [$note($, Name, $ ...) is not defined in your logic-base$] ).

%
% For HTML text answers, a proper HTML header is outputted followed by the 
% answer text followed by any notes.
%
sendAnswer(AnswerList, NoteList) :-
   sendHeader,
   outputNote(answerHeader),
   cgiSend(AnswerList),
   (NoteList = [] -> true ; cgiSend($<P><HR>$)),
   sendNoteList(NoteList),
   outputNote(answerFooter),
   sendFooter.

sendNoteList([]).
sendNoteList([H | T]) :-
   outputNote(H),
   sendNoteList(T).


% ---------------------- Fact File Maintenance ---------------------- %

%
% First time through create a fact file by generating a 7-digit random number
% and prefixing it with a 't' and ending in '.pro'.  These are kept in the 
% 'Temp Directory'. The name is remembered in the system fact 'Fact File'.
%
openFactFile(submitProblem) :-
   repeat,
   NewReal is random * 10000000,
   NewNum is integer(NewReal),
   string_integer(StrNum, NewNum),
   strcat($t$, StrNum, NewFile0),
   strcat(NewFile0, $.pro$, NewFile),
   system('Temp Directory', TempDir),
   strcat(TempDir, NewFile, NewPath),
   (file_exists(NewPath) -> fail ; true),
   assertz(system('Fact File', NewFile)).

%
% On subsequent times through we consult the fact file which is hidden in
% the name of the submit button
%
openFactFile(FileNameAtom) :-
   string_atom(FileName, FileNameAtom),
   system('Temp Directory', TempDir),
   strcat(TempDir, FileName, PathT),
   string_atom(PathT, PathA),
   consult(PathA),
   assertz(system('Fact File', FileName)).

openFactFile(FileNameAtom) :-
   system('Temp Directory', TempDir),
   throw([$Unable to open fact file $, TempDir, FileNameAtom, $ -- check permissions$]).

%
% If an answer has not yet been found, we need to remember all the facts
% for the next time through
%
writeFactFile :-
   system('Fact File', FileName),
   system('Temp Directory', TempDir),
   strcat(TempDir, FileName, Path),
   string_atom(Path, PathA),
   tell(PathA),
   listing(fact/2),
   told.
writeFactFile :-
   system('Temp Directory', TempDir),
   throw([$Unable to write fact file in directory $, TempDir, $ -- check permissions$]).

%
% When the answer is found we delete the fact file, and take the opportunity
% to do a little garbage collecting on files leftover from incomplete sessions.
%
deleteFactFile :-
   % Delete the fact file we were using
   system('Fact File', FileName),
   system('Temp Directory', TempDir),
   strcat(TempDir, FileName, Path),
   delfile(Path, _),

   % And, all fact files over 1 hour old (approximately)
   system('Temp Directory', TempDir),
   cgiPurgeFactFiles(TempDir),
   true.
deleteFactFile.

