/*
   AmziSub.C -- Extended Predicates for CGI Processing

   Link with AmziCGI.C

   Copyright (c) 1996 Amzi! inc.  All Rights Reserved.
   See license.htm for Amzi! License Agreement
*/

/*#define STDCGI*/			/* define either STDCGI or WINCGI */
/*#define WINCGI*/			/* here or in your make file */

#include <ctype.h>

#ifdef _WIN32
#include <direct.h>
#include <io.h>
#else
#define	_MAX_PATH	512
#endif

#include <stdio.h>
#include <string.h>
#include <time.h>

#ifdef __unix__
#include <dirent.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#endif

#include "amzi.h"

void slashslash1(char *s);
void slashslash2(char* sout, const char* sin);


/* built-in predicates, callable from Prolog */
/* ----------------------------------------- */

/* function prototypes */

/* File/directory utilities */

TF EXPFUNC p_delfile(ENGid);
TF EXPFUNC p_chdir(ENGid);
TF EXPFUNC p_rmdir(ENGid);
TF EXPFUNC p_rename(ENGid);
#ifdef	_WIN32
TF EXPFUNC p_mkdir(ENGid);
TF EXPFUNC p_setdrive(ENGid);
TF EXPFUNC p_getdrive(ENGid);
TF EXPFUNC p_findfirst(ENGid);
TF EXPFUNC p_findnext(ENGid);

/* SDK functions */

TF EXPFUNC p_GetPrivateProfileString(ENGid);
TF EXPFUNC p_GetPrivateProfileSection(ENGid);
TF EXPFUNC p_GetPrivateProfileSectionNames(ENGid);
#endif
TF EXPFUNC p_gmtime_str(ENGid);

/* CGI support functions */

TF EXPFUNC p_cgiOpen(ENGid);
TF EXPFUNC p_cgiWrite(ENGid);
TF EXPFUNC p_cgiRewind(ENGid);
TF EXPFUNC p_cgiClose(ENGid);
TF EXPFUNC p_cgiPurgeFactFiles(ENGid);

/* extended predicate table definitions */

PRED_INIT awinPreds[] = 
{
	/* File/directory utility predicates */
	{"delfile", 2, p_delfile},
	{"chdir", 2, p_chdir},
	{"rmdir", 2, p_rmdir},
	{"rename", 3, p_rename},
#ifdef	_WIN32
	{"mkdir", 2, p_mkdir},
	{"setdrive", 1, p_setdrive},
	{"getdrive", 1, p_getdrive},
	{"findfirst", 3, p_findfirst},
	{"findnext", 1, p_findnext},

	/* SDK functions */
	{"getPrivateProfileString", 4, p_GetPrivateProfileString},
	{"getPrivateProfileSection", 3, p_GetPrivateProfileSection},
	{"getPrivateProfileSectionNames", 2, p_GetPrivateProfileSectionNames},
#endif
	{"gmtime_str", 1, p_gmtime_str},

	/* CGI support functions */
	{"cgiOpen", 1, p_cgiOpen},
	{"cgiWrite", 1, p_cgiWrite},
	{"cgiRewind", 0, p_cgiRewind},
	{"cgiClose", 0, p_cgiClose},
	{"cgiPurgeFactFiles", 1, p_cgiPurgeFactFiles},

	{NULL, 0, NULL}
};


/* Global Variables */
/* ---------------- */

#define MAX_PROFILE 4096


/* Initialization Functions */
/* ------------------------ */

/* These are used when this is built as an LSX for debugging under the
   listener.  If you modify AMZICGI.PRO, we strongly suggest you build
   this file as an LSX, otherwise it is very hard to debug changes to
   the WebLS engine. */

/*
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD dwReason, LPVOID lpRes)
{
	switch(dwReason)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
	}
	return TRUE;
}

__declspec(dllexport) RC EXPFUNC InitPreds(ENGid eid, void* p)
*/
extern RC InitPreds(ENGid eid, void* p)
 {
	RC rc;
	char buf[80];

	rc = lsInitPreds(eid, awinPreds);
	if (rc)
	{
#ifdef	_WIN32
		wsprintf(buf, "Error #%d Loading CGI Predicates", rc);
		MessageBox(NULL, buf, "CGI LSX", MB_OK);
#else
		printf("Error #%d Loading CGI Predicates", rc);
#endif
	}

	return 0;
}


/* File and Directory Manipulation */
/* ------------------------------- */

/* chdir(Dir, Err) - Changes directory to Dir, returns error code in Err.
   An error code of 0 means success. */

TF EXPFUNC p_chdir(ENGid CurEng)
{
	int	  rc;
	char  Buffer[_MAX_PATH];

	if (_MAX_PATH <= lsStrParmLen(CurEng, 1) )
		lsErrRaise(CurEng, "input path too long");
	lsGetParm(CurEng, 1, cSTR, Buffer);
	rc = chdir(Buffer);
	return(lsUnifyParm(CurEng, 2, cINT, &rc));
}

#ifdef _WIN32
/* mkdir(Dir, Err) - Makes the directory Dir, returns error code in Err.
   An error code of 0 means success. */

TF EXPFUNC p_mkdir(ENGid CurEng)
{
	int	rc;
	char  Buffer[_MAX_PATH];

	if (_MAX_PATH <= lsStrParmLen(CurEng, 1) )
		lsErrRaise(CurEng, "input path too long");
	lsGetParm(CurEng, 1, cSTR, Buffer);
	rc = mkdir(Buffer);
	return(lsUnifyParm(CurEng, 2, cINT, &rc));
}
#endif /* _WIN32 */

/* rmdir(Dir, Err) - Removes directory Dir, returns error code in Err.
   An error code of 0 means success. */

TF EXPFUNC p_rmdir(ENGid CurEng)
{
	int	 rc;
	char  Buffer[_MAX_PATH];

	if (_MAX_PATH <= lsStrParmLen(CurEng, 1) )
		lsErrRaise(CurEng, "input path too long");
	lsGetParm(CurEng, 1, cSTR, Buffer);
	rc = rmdir(Buffer);
	return(lsUnifyParm(CurEng, 2, cINT, &rc));
}

#ifdef	_WIN32
/* setdrive(Drive, Err) - Makes Drive current, returns error code in Err.
   An error code of 0 means success. */

TF EXPFUNC p_setdrive(ENGid CurEng)
{
	char	  c;
	unsigned  i;
	char	  Buffer[_MAX_PATH];

	if (_MAX_PATH <= lsStrParmLen(CurEng, 1) )
		lsErrRaise(CurEng, "input path too long");
	lsGetParm(CurEng, 1, cSTR, Buffer);
	c = Buffer[0];
	if (c >= 'a' && c <= 'z')
		i = 1 + c - 'a';
	else if (c >= 'A' && c <= 'Z')
		i = 1 + c - 'A';
	else
		lsErrRaise(CurEng, "Bad drive spec sent to setdrive/1");

	_chdrive(i);
	return(TRUE);
}

/* getdrive(Drive) - Returns the current drive as a letter followed
   by a colon. */

TF EXPFUNC p_getdrive(ENGid CurEng)
{
	unsigned  i;
	char  drive[3];

	i = _getdrive();
	drive[0] = 'a' + i - 1;
	drive[1] = ':';
	drive[2] = '\0';
	lsUnifyParm(CurEng, 1, cSTR, drive);
	return(TRUE);
}

#endif /* _WIN32 */

/* delfile(File, Err) - Removes file File, returns error code in Err.
   An error code of 0 means success. */

TF EXPFUNC p_delfile(ENGid eid)
{
	char	sFile[_MAX_PATH];
	RC		rc, ecode;

	if (_MAX_PATH <= lsStrParmLen(eid, 1) )
		lsErrRaise(eid, "input path too long");
	rc = lsGetParm(eid, 1, cSTR, sFile);

	if (remove(sFile))
		ecode = 0;
	else
		ecode = -1;

	rc = lsUnifyParm(eid, 2, cINT, &ecode);

	return TRUE;
}

/* rename(OldFile, NewFile, Err) - Renames file, returns error code in Err.
   An error code of 0 means success. */

TF EXPFUNC p_rename(ENGid CurEng)
{
	RC	rc;
	int	ecode;
	char	sOldName[_MAX_PATH], sNewName[_MAX_PATH];

	if (_MAX_PATH <= lsStrParmLen(CurEng, 1) )
		lsErrRaise(CurEng, "input path too long");
	if (_MAX_PATH <= lsStrParmLen(CurEng, 2) )
		lsErrRaise(CurEng, "output path too long");
	rc = lsGetParm(CurEng, 1, cSTR, sOldName);
	rc = lsGetParm(CurEng, 2, cSTR, sNewName);

	ecode = rename(sOldName, sNewName);

	rc= lsUnifyParm(CurEng, 3, cINT, &ecode);
	return(TRUE);
}

#ifdef _WIN32

/* Global definitions for findfirst family */

long			FileHand;
struct			_finddata_t FT;
unsigned int	mask;

/* findfirst(Pattern, Mask, 
   fileinfo(Name,Attr,time(Hour,Min,Sec),date(Year,Mon,Day),Size))
   Finds the first file that matches the Pattern and Mask */

TF EXPFUNC p_findfirst(ENGid CurEng)
{
	unsigned short	attrib;
	long			time, date;
	unsigned short	hour, min, sec;
	unsigned short	year, month, day;
	TERM			fileinfoT, dateT, timeT;
	char			pattern[_MAX_PATH];

	if (_MAX_PATH <= lsStrParmLen(CurEng, 1) )
		lsErrRaise(CurEng, "input path too long");
	lsGetParm(CurEng, 1, cSTR, &pattern);
	lsGetParm(CurEng, 2, cINT, &mask);
	if (-1 == (FileHand = _findfirst(pattern, &FT)))
		return(FALSE);
	/* Make sure the one we have is an acceptable type */
	while (((FT.attrib & 31) & mask ) == 0 && (FT.attrib & 31) != mask)
	{
			if (-1 == _findnext(FileHand, &FT))
				return(FALSE);
	}

	lsMakeFA(CurEng, &fileinfoT, "fileinfo", 5);

	lsUnifyArg(CurEng, &fileinfoT, 1, cSTR, &(FT.name));

	attrib = FT.attrib;
	lsUnifyArg(CurEng, &fileinfoT, 2, cSHORT, &attrib);

	time = FT.time_access;
	lsMakeFA(CurEng, &timeT, "time", 3);
	hour = (time & 0xf800) >> 11;
	min = (time & 0x07e0) >> 5;
	sec = time & 0x001f;
	lsUnifyArg(CurEng, &timeT, 1, cSHORT, &hour);
	lsUnifyArg(CurEng, &timeT, 2, cSHORT, &min);
	lsUnifyArg(CurEng, &timeT, 3, cSHORT, &sec);
	lsUnifyArg(CurEng, &fileinfoT, 3, cTERM, &timeT);

	date = FT.time_access;
	lsMakeFA(CurEng, &dateT, "date", 3);
	year = (((date & 0xfe00) >> 9) + 80) % 100;
	month = (date & 0x1e0) >> 5;
	day = date & 0x001f;
	lsUnifyArg(CurEng, &dateT, 1, cSHORT, &year);
	lsUnifyArg(CurEng, &dateT, 2, cSHORT, &month);
	lsUnifyArg(CurEng, &dateT, 3, cSHORT, &day);
	lsUnifyArg(CurEng, &fileinfoT, 4, cTERM, &dateT);

	lsUnifyArg(CurEng, &fileinfoT, 5, cLONG, &(FT.size));

	return(lsUnifyParm(CurEng, 3, cTERM, &fileinfoT));
}

/* findnext(fileinfo(Name,Attr,time(Hour,Min,Sec),date(Year,Mon,Day),Size)) */
/*   Finds the next file that matches the previous Pattern and Mask */

TF EXPFUNC p_findnext(ENGid CurEng)
{
	TERM				fileinfoT, dateT, timeT;
	unsigned short	attrib;
	long				time, date;
	unsigned short	hour, min, sec;
	unsigned short	year, month, day;

	if (-1 == _findnext(FileHand, &FT))
		return(FALSE);

	/* Make sure the one we have is an acceptable type */
	while (((FT.attrib & 31) & mask) == 0 && (FT.attrib & 31) != mask)
	{
		if (-1 == _findnext(FileHand, &FT))
			return(FALSE);
	}

	lsMakeFA(CurEng, &fileinfoT, "fileinfo", 5);

	lsUnifyArg(CurEng, &fileinfoT, 1, cSTR, &(FT.name));

	attrib = FT.attrib;
	lsUnifyArg(CurEng, &fileinfoT, 2, cSHORT, &attrib);

	time = FT.time_access;
	lsMakeFA(CurEng, &timeT, "time", 3);
	hour = (time & 0xf800) >> 11;
	min = (time & 0x07e0) >> 5;
	sec = time & 0x001f;
	lsUnifyArg(CurEng, &timeT, 1, cSHORT, &hour);
	lsUnifyArg(CurEng, &timeT, 2, cSHORT, &min);
	lsUnifyArg(CurEng, &timeT, 3, cSHORT, &sec);
	lsUnifyArg(CurEng, &fileinfoT, 3, cTERM, &timeT);

	date = FT.time_access;
	lsMakeFA(CurEng, &dateT, "date", 3);
	year = (((date & 0xfe00) >> 9) + 80) % 100;
	month = (date & 0x1e0) >> 5;
	day = date & 0x001f;
	lsUnifyArg(CurEng, &dateT, 1, cSHORT, &year);
	lsUnifyArg(CurEng, &dateT, 2, cSHORT, &month);
	lsUnifyArg(CurEng, &dateT, 3, cSHORT, &day);
	lsUnifyArg(CurEng, &fileinfoT, 4, cTERM, &dateT);

	lsUnifyArg(CurEng, &fileinfoT, 5, cLONG, &(FT.size));

	return(lsUnifyParm(CurEng, 1, cTERM, &fileinfoT));
}

/* SDK Functions */
/* ------------- */

/* GetPrivateProfileString(Section, Key, Value, INIFile)
	  Section is instantiated with a string naming the section
	  Key is the string value of the key
	  Value is the returned parameter value
	  INIFile is the name of the file
	  
	GetPrivateProfileString reads a value from the INIFile. */

TF EXPFUNC p_GetPrivateProfileString(ENGid eid)
{
	char	sSection[MAX_PROFILE];
	char	sKey[MAX_PROFILE];
	char	sValue[MAX_PROFILE];
	char	sFile[MAX_PROFILE];
	TF		tf;
	DWORD	ecode;
	RC		rc;

	if (MAX_PROFILE <= lsStrParmLen(eid, 1) )
		lsErrRaise(eid, "section name too long");
	if (MAX_PROFILE <= lsStrParmLen(eid, 2) )
		lsErrRaise(eid, "key name too long");
	if (MAX_PROFILE <= lsStrParmLen(eid, 4) )
		lsErrRaise(eid, "file name too long");

	rc = lsGetParm(eid, 1, cSTR, sSection);
	rc = lsGetParm(eid, 2, cSTR, sKey);
	rc = lsGetParm(eid, 4, cSTR, sFile);
	
	ecode = GetPrivateProfileString(sSection, sKey, "", sValue, MAX_PROFILE, sFile);
	if (ecode == 0 || ecode == MAX_PROFILE - 1 || ecode == MAX_PROFILE -2)
		*sValue = '\0';

	slashslash1(sValue);
	tf = lsUnifyParm(eid, 3, cSTR, sValue);				

	return tf;
}


/* GetPrivateProfileSection(Section, List, INIFile)
	  Section is instantiated with a string naming the section
	  List	  is the of string keynames and values in Section in the
	          form fact(keyname, value)
	  INIFile is the name of the file
	  
	GetPrivateProfileSection reads all the key names in the section 
	from the INIFile. */

TF EXPFUNC p_GetPrivateProfileSection(ENGid eid)
{
	char	sSection[MAX_PROFILE];
	char	sValue[MAX_PROFILE];
	char	sFile[_MAX_PATH];
	char	termStr[MAX_PROFILE], sBuf[MAX_PROFILE];
	char	*p, *v, *u;
	TERM	plist, pitem;
	TF		tf;
	DWORD	ecode;
	RC		rc;

	if (MAX_PROFILE <= lsStrParmLen(eid, 1) )
		lsErrRaise(eid, "section name too long");
	if (_MAX_PATH <= lsStrParmLen(eid, 3) )
		lsErrRaise(eid, "file name too long");

	rc = lsGetParm(eid, 1, cSTR, sSection);
	rc = lsGetParm(eid, 3, cSTR, sFile);

	ecode = GetPrivateProfileSection(sSection, sValue, MAX_PROFILE, sFile);
	if (ecode == 0 || ecode == MAX_PROFILE - 1 || ecode == MAX_PROFILE -2)
		*sValue = '\0';

	rc = lsMakeList(eid, &plist);

	p = sValue;
	while (*p != '\0')
	{
		/* Build fact( */
		strcpy(termStr, "fact('");

		/* Find the = sign and take the key */
		v = strtok(p, "=");
		slashslash2(sBuf, v);

		/* If its a multi-valued field hack off the _<num> part */
		u = strrchr(sBuf, '_');
		if (u != NULL)
			if (isdigit(*(u+1))) *u = '\0';

		strcat(termStr, sBuf);
		strcat(termStr, "', '");

		/* Get the rest as the value */
		p = p + strlen(v);
		p++;
		slashslash2(sBuf, p);
		strcat(termStr, sBuf);
		strcat(termStr, "')");
			
		/* Put it on the list if there was a value */
		if (strlen(p) > 0)
		{
			rc = lsStrToTerm(eid, &pitem, termStr);
			rc = lsPushList(eid, &plist, pitem);
		}
		p = p + strlen(p);
		p++;
	}

	tf = lsUnifyParm(eid, 2, cTERM, &plist);

	return tf;
}


/* GetPrivateProfileSectionNames(List, INIFile)
      List is the of string keynames in Section
      INIFile is the name of the file
	  
	GetPrivateProfileSectionNames reads all the key names in the section 
	from the INIFile. */

TF EXPFUNC p_GetPrivateProfileSectionNames(ENGid eid)
{
	char	sFile[_MAX_PATH];
	char	sValue[MAX_PROFILE];
	char	*p;
	TERM	plist, pitem;
	TF		tf;
	DWORD	ecode;
	RC		rc;

	if (MAX_PROFILE <= lsStrParmLen(eid, 2) )
		lsErrRaise(eid, "file name too long");
	rc = lsGetParm(eid, 2, cSTR, sFile);

	ecode = GetPrivateProfileSectionNames(sValue, MAX_PROFILE, sFile);
	if (ecode == 0 || ecode == MAX_PROFILE - 1 || ecode == MAX_PROFILE -2)
		*sValue = '\0';

	rc = lsMakeList(eid, &plist);

	p = sValue;
	while (*p != '\0')
	{
		rc = lsMakeStr(eid, &pitem, p);
		rc = lsPushList(eid, &plist, pitem);
		p = p + strlen(p);
		p++;
	}

	tf = lsUnifyParm(eid, 2, cTERM, &plist);

	return tf;
}

#endif /* _WIN32 */

/* gmtime_str(time) returns the current GMT time. */

TF EXPFUNC p_gmtime_str(ENGid eid)
{
	struct tm	*gmt;
	long		ltime;
	char		*sTime, sBuf[30];
	TF			tf;

	time(&ltime);
	gmt = gmtime(&ltime);
	sTime = asctime(gmt);

	strcpy(sBuf, sTime);
	sBuf[24] = '\0';		/* remove the newline */

	tf = lsUnifyParm(eid, 1, cSTR, sBuf);

	return tf;
}

/* CGI Support Functions */
/* --------------------- */

/* Global definitions for CGI output functions */

FILE	*outFile = NULL;
char	outName[_MAX_PATH];

/* These routines are used to open, write, rewind and close the
   output file, which is stdout in standard CGI and a file in
   Win-CGI */

/* cgiOpen(File) opens the named file (or if blank stdout) for write */

TF EXPFUNC p_cgiOpen(ENGid eid)
{
	int		rc;
	
	if (_MAX_PATH <= lsStrParmLen(eid, 1) )
		lsErrRaise(eid, "file name too long");
	rc = lsGetParm(eid, 1, cSTR, outName);

	if (strlen(outName) > 0)
		outFile = fopen(outName, "w");
	else
		outFile = stdout;

	if (outFile == NULL)
		return(FALSE);
	else
		return(TRUE);
}

/* cgiWrite(Term) writes the string to the output file */

TF EXPFUNC p_cgiWrite(ENGid eid)
{
	char	sBuf[4096];
	TERM	term;
	int		rc, ecode;

	rc = lsGetParm(eid, 1, cTERM, &term);
	if (rc != 0) return(FALSE);

	rc = lsTermToStr(eid, term, sBuf, 4096);
	if (rc != 0) return(FALSE);

	if (strlen(sBuf) == 0)
		return(TRUE);

	ecode = fputs(sBuf, outFile);
	if (ecode < 0)
		return(FALSE);
	else
		return(TRUE);
}

/* cgiRewind rewinds the output file to the beginning */

TF EXPFUNC p_cgiRewind(ENGid eid)
{
	if (outFile == stdout)
		rewind(outFile);
	else
	{
		if (outFile != NULL) fclose(outFile);
		outFile = fopen(outName, "w");
	}

	if (outFile == NULL)
		return(FALSE);
	else
		return(TRUE);
}


/* cgiClose closes the output file */

TF EXPFUNC p_cgiClose(ENGid eid)
{
	if (outFile != stdout && outFile != NULL)
		fclose(outFile);
	return(TRUE);
}

#ifdef _WIN32
/* cgiPurgeFactFiles deletes all fact files beginning with t in
   the specified directory that are at least 1 hour old */

TF EXPFUNC p_cgiPurgeFactFiles(ENGid eid)
{
	char	sPath[_MAX_PATH], sPattern[_MAX_PATH], sFile[_MAX_PATH];
	int		rc, ecode;
	struct	_finddata_t FT;
	unsigned short	prevhr;
	struct	tm *ltimep, now;
	struct	tm *ftimep, filwrt;
	time_t	ltime, ftime;
	long	findem;

	/* Get the current time and calculate previous hour */
	time(&ltime);
	ltimep = localtime(&ltime);
	now = *ltimep;
	if (now.tm_hour == 0) prevhr = 24; else prevhr = now.tm_hour - 1;

	/* Get the path to purge files from */
	if (_MAX_PATH <= lsStrParmLen(eid, 1) )
		lsErrRaise(eid, "String too long");
	rc = lsGetParm(eid, 1, cSTR, sPath);

	/* Save the path for appending to the file name later */
	strcpy(sPattern, sPath);

	/* Look for all files starting with t */
	strncat(sPattern, "t*.*", _MAX_PATH);
	findem = _findfirst(sPattern, &FT);
	if (findem == -1) return(TRUE);		/* no files to purge */

	do
	{
		/* Get the time the file was last modified */
		ftime = FT.time_write;
		ftimep = localtime(&ftime);
		filwrt = *ftimep;

		/* Remove it if its more than an hour old */
		if (filwrt.tm_hour != now.tm_hour && filwrt.tm_hour != prevhr)
		{
			strcpy(sFile, sPath);
			strncat(sFile, FT.name, _MAX_PATH );
			ecode = remove(sFile);
		}

		rc = _findnext(findem, &FT);
	}
	while (rc != -1);

	return(TRUE);
}
#endif /* _WIN32 */

#ifdef __unix__
TF EXPFUNC p_cgiPurgeFactFiles(ENGid eid)
{
	char			sPath[_MAX_PATH], sFile[_MAX_PATH];
	int				rc, ecode;
	DIR				*dirp;
	struct dirent	*dp;
	struct stat		filstat;
	unsigned short	prevhr;
	struct tm		*ltimep, now;
	struct tm		*ftimep, filwrt;
	time_t			ltime, ftime;

	/* Get the current time and calculate previous hour */
	time(&ltime);
	ltimep = localtime(&ltime);
	now = *ltimep;
	if (now.tm_hour == 0) prevhr = 24; else prevhr = now.tm_hour - 1;

	/* Get the path to purge files from */
	if (_MAX_PATH <= lsStrParmLen(eid, 1) )
		lsErrRaise(eid, "String too long");
	rc = lsGetParm(eid, 1, cSTR, sPath);

	/* Open the directory and save the path for appending to 
	   the file name later */
	sPath[strlen(sPath)-1] = '\0';
	dirp = opendir(sPath);
	if (dirp == NULL) return(FALSE);
	strncat(sPath, "/", _MAX_PATH);

	while ((dp = readdir(dirp)) != NULL)
	{
		/* If the name begins with t then check it out */
		if (*(dp->d_name) == 't' || *(dp->d_name) == 'T')
		{
			/* Build the path name */
			strcpy(sFile, sPath);
			strncat(sFile, dp->d_name, _MAX_PATH );

			/* Get the time the file was last modified */
			stat(sFile, &filstat);
			ftime = filstat.st_mtime;
			ftimep = localtime(&ftime);
			filwrt = *ftimep;

			/* Remove it if its more than an hour old */
			if (filwrt.tm_hour != now.tm_hour && filwrt.tm_hour != prevhr)
				ecode = remove(sFile);
		}
	}

	(void)closedir(dirp);

	return(TRUE);
}
#endif /* __unix__ */


/* -------------------- Utility Routines -------------------- */

/* Any string of characters sent to Prolog is read using the
   Prolog reader, which interprets a backslash as an escape
   character.  So, this means if you really want a backslash,
   such as in a file path, then it has to be a double
   backslash.  This function makes that conversion. */

void slashslash2(char* sout, const char* sin)
{
	while(*sin)
	{
		if (*sin == '\\')
		{
			*sout++ = *sin;
			*sout++ = *sin++;
		}
		else
			*sout++ = *sin++;
	}
	*sout = *sin;
	return;
}

/* Same as slashslash2 except conversion is done in place.
   Therefore, the input buffer must be large enough to handle
   the expanded string. */

void slashslash1(char* s)
{
	int	nslash=0;
	int	len=0;
	int	i;
	char* s1;
	char* s2;
	
	/* Count the slashes, then copy in place from right to
		left so we don't write on ourself. */
		
	s1 = s;
	while(*s1)
	{
		len++;
		if (*s1++ == '\\') nslash++;
	}
	
	s2 = s + len + nslash;
	s1 = s + len;
	
	for(i=0; i<=len; i++)
	{
		if (*s1 == '\\')
		{
			*s2-- = *s1;
			*s2-- = *s1--;
		}
		else
			*s2-- = *s1--;
	}
	
	return;
}


