%
% Amzi! Web-Based Tech Support Logic-Base
% Copyright (c) 1996 by Amzi! inc. All Rights Reserved
%
% See license.htm for Amzi! License Agreement
%

 
% ------------------------------------------------------------------- %
%                          The Logic-Base                             %
% ------------------------------------------------------------------- %

%
% Required definitions for Amzi!  Do not change or remove.  Must be
% at start of file.
%
:- op(790, fx, if).     	% prefix operator
:- op(780, xfx, then).  	% infix operator
:- op(775, xfy, or).    	% infix that can be linked
:- op(770, xfy, and).   	% infix that can be linked
:- op(700, xfx, <=).		% infix operator
:- op(700, xfx, include).	% infix operator

% ------------------------------------------------------------------- %
%                         System Variables                            %
% ------------------------------------------------------------------- %

%
% System configuration: set these for your particular environment
%
% (Note to Amzi! users--comment these out when compiling your logic-base
%  see related comment in cgiMain in amzicgi.pro)
%

% For Windows

system('Log File', 'C:\\WebLS\\logs\\bigtrace.htm').
system('Log File URL', 'file:///C:\\WebLS\\logs\\bigtrace.htm').
system('Results File', $C:\\WebLS\\logs\\bigrslts.pro$).
system('Initial Form', $C:\\WebLS\\html\\bigrules.htm$).
system('Temp Directory', $C:\\WebLS\\temp\\$).
system('Body Args', $bgcolor=#CEFAFF text=#000000$).
system('Title', $Amzi! Problem Resolver$).
system('Footer', $<FONT SIZE=-1>Copyright &#169;1996 Amzi! inc.  All Rights Reserved.$).
system('Form Action', 'Executable Path').
system('Goal', 'problem').

% For BSD Unix

%system('Log File', '/usr/me/html/bigtrace.htm').
%system('Log File URL', 'http://www.company.com/bigtrace.htm').
%system('Results File', $/usr/me/logs/bigrslts.pro$).
%system('Initial Form', $/usr/me/html/bigrules.htm$).
%system('Temp Directory', $/usr/me/temp/$).

% ------------------------------------------------------------------- %
%                               Attrs                                 %
% ------------------------------------------------------------------- %

%
% Defines how to ask for the value of each fact.  
% Note all facts that are not synthesized or asked for on the
% initial form must be included here, otherwise an error message will
% be displayed.
%
% These fields are required in an attr:
%
% prompt = $prompt-text$
% ask = field | yesno | menu(['choice1', 'choice2', ... 'choiceN'])
%
% These fields are optional:
%
% length = N                              (only applies to ask = field)
% related = [fact1, fact2, ... factN]
%

attr(symptoms,
   [
   multivalued
   ]).

attr(ideCommand,
   [
   prompt =  $What IDE command is causing the problem?$,
   ask = menu(['none', 'File/Open', 'File/Save', 'File/Print', 'Edit', 'View', 
               'Listener/Start End', 'Debug/On Off', 'Listener/(Re)Consult', 
               'Build/Project', 'Build/Compile', 'Build/Link', 'Build/Run', 
               'Window', 'Help']),
   related = [ideProblemType]
   ]).

attr(ideProblemType,
   [
   prompt = $What type of problem are you having in the IDE?$,
   ask = menu(['other', 'Edit window is blank', 'IDE appears to be unstable',
               'Screen/printer font size is too small/large'])
   ]).

attr(delphiVersion,
   [
   prompt = $What version of Delphi are you using?$,
   ask = menu(['1.0', '2.0'])
   ]).

attr(amziSampleName,
   [
   prompt = $What Amzi! sample program are you running?$,
   ask = menu(['dbgene', 'dbgenevb'])
   ]).

attr(iniStringEscOff,
   [
   prompt = $Is string_esc off in your .INI file or Prolog program?$,
   ask = yesno
   ]).

attr(usingLongFileNames,
   [
   prompt = $Are you using long file names (>8 char name and >3 char extension)?$,
   ask = yesno
   ]).

attr(usingDCG,
   [
   prompt = $Are you using DCG in your Prolog module?$,
   ask = yesno
   ]).

attr(usingImportExport,
   [
   prompt = $Are you using import or export directives in your Prolog module?$,
   ask = yesno
   ]).


% ------------------------------------------------------------------- %
%                              Rules                                  %
% ------------------------------------------------------------------- %

%
% These rules determine the problem based on the facts.  Rules always
% have the form:  if <conditions> then <attribute> = <value>.
%
% The rule engine supports these operators:
%   = (equal)
%   \= (not equal)
%   >  <  >=  <=
%   include
%   and
%   or
%
% The value for an attribute can be any legal Prolog term.
%

%
% IDE Related Problems
%
if programType = 'Amzi! IDE' and
   errorMessage = 'Local stack full' or 
   errorMessage = 'Control stack full'
then problem = ideStack.

if programType = 'Amzi! IDE' and
   environmentNameVer = 'Windows 95' and
   ideCommand = 'Build/Project'
then problem = ideProject.

if programType = 'Amzi! IDE' and
   environmentNameVer = 'Windows 3.x' and
   ideCommand  = 'File/Open' and
   ideProblemType = 'Edit window is blank'
then problem = ideProject.

if programType = 'Amzi! IDE' and
   environmentNameVer = 'Windows 95' and
   ideCommand = 'Listener/(Re)consult' and
   iniStringEscOff = 'Yes'
then problem = ideStringEscWin95.

if programType = 'Amzi! IDE' and
   ideCommand = 'None' and
   (symptoms include 'System crash or GPF (General Protection Fault)' or
   ideProblemType = 'IDE appears to be unstable')
then problem = ideInstability.

if programType = 'Amzi! IDE' and
   ideProblemType = 'Screen/printer font size is too small/large'
then problem = ideFontSmall.

if programType = 'Amzi! IDE' and
   environmentNameVer = 'Windows 95' and
   usingLongFileNames = 'Yes' and
   releaseDate <= '19960302'
then problem = ideLongFileNames.

if programType = 'Amzi! IDE' and
   ideCommand = 'Build/Compile' and
   errorMessage = 'Bad argument for arithmetic evaluation'
then problem = ideCmpUnusedVars.

%
% Amzi! Sample specific problems
%

if programType = 'Amzi! Sample Program' and
   errorMessage = '[ODBC] Datasource name not found' and
   (amziSampleName = 'dbgene' or amziSampleName = 'dbgenevb')
then problem = geneNotRegistered.

%
% Logic Server API call specific problems
%
if (apiFunction = 'lsGetParmType' or
   apiFunction = 'lsGetArgType') and
   releaseDate < '19960302'
then problem = apiTypeAlwaysVar.

%
% Prolog predicate specific problems
%
if environment = 'Windows' and
   predicate = 'findfiles'
then problem = predWinFile.

if predicate = 'findall' or
   predicate = 'setof' or
   predicate = 'bagof'
then problem = predFindallExport.


%
% Host language specific problems
%
if languageTool = 'Access'
then problem = accessOutOfResources.

if language = 'C/C++' and
   apiFunction = 'lsInit' and
   ((symptoms include 'System crash or GPF (General Protection Fault)' and environmentNameVer = 'Windows 3.x') or
   environmentNameVer = 'DOS')
then problem = cLargeModelRequired.

if languageTool = 'Delphi' and
   delphiVersion = '2.0' and
   releaseDate < '19960302'
then problem = delphiStdcall.

if languageTool = 'Delphi' and
   symptoms include 'Cannot open or Missing file' and
   releaseDate < '19960302'
then problem = delphiMissingDelGUI.

if languageTool = 'Visual Basic' and
   symptoms include 'Cannot open or Missing file' and
   releaseDate < '19960302'
then problem = vbMissingAmziVB.   

if errorMessage = 'StrToTerm undefined' and
   languageTool = 'Visual Basic'
then problem = vbStrToTerm.

%
% Problems determined by error message or error code
%
if errorMessage = 'Out of thing space' and
   releaseDate < '19960302'
then problem = thingSpace.

if errorMessage = 'Cannot open file'
then problem = cannotFindFile.

if errorMessage = 'Code too long to load'
then problem = srcbufTooSmall.

if errorMessage = 'Load buffer overflow'
then problem = destbufTooSmall.

if (errorCode = '234' or errorMessage = 'Redefining system or compiled predicate') and
   usingImportExport \= 'Yes'
then problem = redefiningPredicate.

if (errorCode = '234' or errorMessage = 'Redefining system or compiled predicate') and
   usingImportExport = 'Yes'
then problem = badImport.

if errorMessage = 'Atom table disaster' or
   errorMessage = 'Bad atom in compiled code'
then problem = badImport.

if errorMessage = 'Dangerous cut in not or if-then-else'
then problem = dangerousCut.

if errorMessage = 'Local stack full' and
   usingDCG = 'Yes'
then problem = dcgVariableGoal.

if (errorCode = '600' or errorMessage = 'Error loading .XPL file') and
   programType = 'Amzi! Hello Program' 
then problem = helloXPL.

if (errorCode = '600' or errorMessage = 'Error loading .XPL file') and
   programType = 'Amzi! Sample Program' 
then problem = sampleXPL.

if (errorCode = '600' or errorMessage = 'Error loading .XPL file') and
   programType = 'My Program' 
then problem = missingXPL.

if errorMessage = 'Term too complex to read'
then problem = readOverflow.

if errorMessage = 'Too many variables'
then problem = maxVarsOverflow.

if errorMessage = 'Read stack full'
then problem = readStackOverflow.

if errorMessage = 'System pred - bad argument'
then problem = badSysPredArg.

%
% These rules synthesize facts into other facts
%
if languageTool = 'Visual C++' or
   languageTool = 'Visual C' or
   languageTool = 'Borland C++' or
   languageTool = 'Borland C' or
   languageTool = 'Watcom C' or
   languageTool = 'Watcom C++'
then language = 'C/C++'.

if environmentNameVer = 'Windows 3.x' or
   environmentNameVer = 'Windows NT' or
   environmentNameVer = 'Windows 95'
then environment = 'Windows'.

if apiFunction = 'lsCall' or
   apiFunction = 'lsCallStr' or
   apiFunction = 'lsExec' or
   apiFunction = 'lsExecStr' or
   apiFunction = 'lsMain'
then callingProlog = 'Yes'.

if amziVersion = '3.3 Jun96'
then releaseDate = '19960601'.

if amziVersion = '3.3 Mar96'
then releaseDate = '19960302'.

if amziVersion = '3.3 Jan96'
then releaseDate = '19960102'.

if amziVersion = '3.3 Aug95'
then releaseDate = '19950801'.

if amziVersion = '3.0'
then releaseDate = '19950101'.

if amziVersion = '2.0 or earlier'
then releaseDate = '00000000'.


% ------------------------------------------------------------------- %
%                             Answers                                 %
% ------------------------------------------------------------------- %

%
% The answers.  They can be direct HTML output, local HTML files or 
% references to other URLs.
% 
% To send text, use:
%
%   text = [$string1$, $string2$, ... $stringN$]
%   or
%   text = $string1-only$
%
% Text answers have an HTML header and footer added automatically.
% They can also contain a list of notes to output after the answer
% by adding:
%
% note = [note1, note2, ... noteN]
%
% To redirect to a different URL use:
%
% url = 'url'
%
% To output a local HTML file which includes an HTML header and footer,
% use:
%
% htmlFile = 'filename'
%

%
% Problems determined by error message or error code
%
answer(thingSpace,
   [
   text = [$If you do a lot of string handling, such as in an ODBC $,
           $application, you might run out of thing space. $,
           $This is because string space was not garbage collected. $,
           $<P>A new version of the 32-bit DLL, A_MW3D.DLL, $,
           $that fixes this problem is available from our FTP site $,
           $in <A HREF="ftp://ftp.amzi.com/pub/users/amzi/sw_patches/A_MW3D.DLL">$,
           $sw_patches/A_MW3D.DLL</A>.$,
           $<P>If you need a 16-bit DLL, let us know.$]
   ]).

answer(helloXPL,
   [
   text = [$The program is unable to locate the Amzi! Prolog object module, an XPL file. $,
           $Make sure the AMZI installation directory is in your path, and that HELLO.XPL file $,
           $exists in the path or the current directory.$]
   ]).

answer(sampleXPL,
   [
   text = [$The program is unable to locate the Amzi! Prolog object module, an XPL file. $,
           $Some of the Amzi! samples include source only, $,
           $and the PRO file needs to be compiled and linked $,
           $into an XPL file.$,
           $<P>If the XPL file exists, then it must be in your $,
           $path or the current directory.$]
   ]).

answer(missingXPL,
   [
   text = [$The program is unable to locate the Amzi! Prolog object module, an XPL file. $,
           $Make sure the the XPL file exists in your path $,
           $or the current directory.$,
           $<P>Note: Some development environments such as Visual Basic and $,
           $Delphi set the current directory to a directory you might not $,
           $have expected.$]
   ]).

answer(cannotFindFile,
   [
   text = [$There are 2 possible sources to the problem.<OL>$,
           $<LI>These problems are usually due to path errors. $,
           $This happens in Windows 95 and NT environments with Amzi! $,
           $system .XPL files if the system can't find the AMZI directory. $,
           $It can also happen very easily in VB or Delphi programs after $,
           $you've browsed in different directories looking for files.$,
           $<LI>If you have string_esc turned on (the default case) $,
           $you need to use double backslashes in path names.  string_esc$,
           $can be turned on and off in .INI files and by the predicate set_mode/2.$,
           $</OL>$]
   ]).

answer(srcbufTooSmall,
   [
   text = [$One of your predicates is too large to load.  You can $,
           $either increase 'srcbuf' in your .INI file, or break up $,
           $the predicate into multiple predicates.$]
   ]).

answer(destbufTooSmall,
   [
   text = [$One of your predicates is too large to load.  You can $,
           $either increase 'destbuf' in your .INI file, or break up $,
           $the predicate into multiple predicates.$]
   ]).

answer(redefiningPredicate,
   [
   text = [$Besides the obvious reason for this message, there are some $,
           $subtler reasons why you might get it when the program works in $,
           $the Listener.  Compiled predicates must have contiguous clauses $,
           $in their definition.  If you have the clauses split, that is $,
           $another predicate definition is in between the clauses, then $,
           $for first set will be compiled, and when the second set is $,
           $encountered, the system complains of redefinition. $,
           $<P>This condition can also occur inadvertently.  If you have $,
           $a punctuation error in a list of clauses, you might have legal $,
           $Prolog, but not what you mean.  For example, you might have $,
           $an extra argument in one clause by mistake, or you might have $,
           $mixed up periods and commas that lead the system to thinking $,
           $you're defining a comma as a predicate.  You can check for $,
           $these errors by looking at the output from the compiler, and $,
           $checking the names and arities of the clauses being compiled.$]
   ]).

answer(dangerousCut,
   [
   text = [$Cuts in if-then-else are ill-advised as they don't work the $,
           $same in compiled and interpreted code for various reasons, $,
           $one being the compiler generates clever code that makes the $,
           $if-then-else execute much faster than if it acted like a $,
           $separate predicate, as it does in the interpreter.  $,
           $<P>The problem is, you can't tell that you're in a if-then-else $,
           $and cuts affect the larger predicate rather than being local $,
           $as they are in the interpreter, so you get different behavior.$,
           $And, implicit calls generate the same type of compiled code.$]
   ]).

answer(dcgVariableGoal,
   [
   text = [$This can be caused by attempting to compile a DCG statement $,
           $with a variable as a goal, e.g.  sentence--&gt;noun, X, verb.$]
   ]).

answer(badImport,
   [
   text = [$These errors are often triggered by errors in :-import &amp; $,
           $:-export directives.  You must use the functor/arity syntax $,
           $in these directives, e.g. :-import test/3.$,
           $<P>Also missing quotes, commas, $,
           $periods, and slashes can all contribute to these errors. $,
           $<P>If you are using op directives, they must be <B>after</B> the $,
           $import/export directives.$],
   note = [modules]
   ]).

answer(readOverflow,
   [
   text = [$The term being read overflowed the input read buffer. $,
           $Either break the term up into smaller components or $,
           $increase the .INI file parameter 'readbuffer'.$]
   ]).

answer(maxVarsOverflow,
   [
   text = [$The term being read had more variables than the reader $,
           $could handle.  Either simplify the term or increase $,
           $the .INI file parameter 'maxvars'.$]
   ]).

answer(readStackOverflow,
   [
   text = [$The term being read overflowed the parse stack.  Either $,
           $simplify the term or increase the .INI file parameter $,
           $'readdepth'.$]
   ]).

answer(badSysPredArg,
   [
   text = [$A system predicate, such as 'write' or 'is' or '=..' $,
           $or 'sub_string' was called with a bad argument. $,
           $Unfortunately this same message is put out by all of them. $,
           $If you are running in the listener, it will tell you $,
           $what predicate was called, but if you are running an $,
           $embedded program you don't automatically get this $,
           $information.  See <A HREF="/embederr.htm">Trapping Errors in Embedded Prolog Modules</A>.$]
   ]).

%
% IDE Related Problems
%
answer(ideStack,
   [
   text = [$The debugger requires more stack space. $,
           $Increase the stack sizes in the AMZI.INI file $,
           $(see help or documentation).$]
   ]).

answer(ideProject,
   [
   text = [$Under Win95 the Project/Edit function does not work. $,
           $<P>Until we get this fixed, you can manually maintain a .ppj file. $,
           $It is simply a text file with a list of the files you want included in the project. $,
           $You can change the extension from .pro to .plm as desired to affect which gets loaded by the listener. $,
           $<P>You can do this all from within the IDE with one exception. $,
           $You cannot, under Win95, save the file as x.ppj. $,
           $It stills tacks a .pro on the end, leaving x.ppj.pro. $,
           $You have to rename this file to have the extension .ppj. $,
           $Once youve done this, you can then maintain your .ppj file using the IDE editor. $]
   ]).

answer(ideBigEdit,
   [
   text = [$This will happen if the file is greater than 64k or so. $,
           $It seems the edit control is still constrained by the $,
           $segmented architecture under Win32s, only rather than $,
           $presenting a message, it simply fails to load the file.$]
   ]).

answer(ideStringEscWin95,
   [
   text = [$Under Win95, if string_esc is off via either the .INI file $,
           $or a set_mode/2 call, then reconsult and consult hangs for $,
           $a few seconds then fails to find the file.$]
   ]).

answer(ideInstability,
   [
   text = [$The IDE, especially in the 16-bit world, can become unstable. $,
           $This is especially true after failure to respond to listener $,
           $prompts.  By not answering a prompt for information from the $,
           $listener, you can sometimes leave the Prolog environment in $,
           $an unstable state.  This is true for various reads and gets $,
           $in your program as well.  $,
           $<P>Unfortunately, because of the way DLLs work in 16-bit $,
           $windows, it might be necessary to exit the IDE and restart.$]
   ]).

answer(ideLongFileNames,
   [
   text = [$There have been reports of problems with long file names, $,
           $resulting in instability in the IDE.  We have reproduced some $,
           $of those problems using names with spaces in them.  If you have $,
           $more details of repeatable failures, let us know.  In any case, $,
           $using short file names works for the time being.$]
   ]).

answer(ideFontSmall,
   [
   text = [$The same font is used in the listener, debug and edit windows $,
           $and for printing. You can change the font size by using $,
           $the View/Set Font command.$]
   ]).

answer(ideCmpUnusedVars,
   [
   text = [$This will occur when unifying with variables that are not used. $,
           $For example: foo(X) :- B = A, X = B.$,
           $<P>This situation is usually the result of a coding error, but $,
           $does get the compiler confused, resulting in the error message.$]
   ]).

%
% Amzi! Sample specific problems
%
answer(geneNotRegistered,
   [
   text = [$These sample programs require that the Access 2.0 database $,
           $'gene' be registered in 16 or 32-bit ODBC.  See the readme $,
           $file in the sample directory for detailed instructions.$]
   ]).

%
% Logic Server API call specific problems
%

answer(apiTypeAlwaysVar,
   [
   text = [$These two functions erroneously report that the type is pVAR $,
           $no matter what the parm or argument type is.  There is a work $,
           $around. $,
           $<P>Use either lsGetArg or lsGetParm to pick up the argument as $,
           $a TERM.  Then use lsGetTermType on the term.  It, unlike its $,
           $friends, returns the correct type. $,
           $<P>For example:<P>$,
           $<PRE>TERM t;<BR>$,
           $lsGetParm(eid, 2, cTERM, &ampt);<BR>$,
           $ptype = lsGetTermType(eid, t);</PRE>$,
           $<P>This problem has also been fixed in the upcoming Mar96 update.$]
   ]).


%
% Prolog predicate specific problems
%
answer(predWinFile,
   [
   text = [$findfiles, delfile, mkdir, rmdir, chdir and rename are $,
           $implemented under Windows in the AWIN.LSX (logic server $,
           $extension.  This LSX must be loaded before the predicates $,
           $can be called.$,
           $<P>To load AWIN.LSX add the line 'lsxload=awin' to your $,
           $AMZI.INI file, or call lsAddLSX() in your host program.$]
   ]).

answer(predFindallExport,
   [
   text = [$These predicates cannot find local compiled predicates, $,
           $so if you have made your source code file into a module by $,
           $adding the directives :-import or :-export, then you must $,
           $also :-import any predicates that are referred to in findall, $,
           $setof or bagof.  This makes them global compiled predicates $,
           $and everything works as desired.$],
   note = [modules]
   ]).

%
% Host language specific problems
%
answer(accessOutOfResources,
   [
   text = [$If you have many add-ins, Access can run out of resources $,
           $for running everything.  Remove add-ins and change swap $,
           $file to permanent.$]
   ]).

answer(cLargeModelRequired,
   [
   text = [$16-bit C/C++ applications require the large memory model. $,
           $Failure to use it leads to immediate GPFs.$],
   note = [debugEmbed]
   ]).

answer(delphiMissingDelGUI,
   [
   text = [$In the Personal Edition of the Jan96 release, DELGUI.PAS $,
           $was inadvertently omitted.  You can download it from sw_patches in $,
           $<A HREF="ftp://ftp.amzi.com/pub/users/amzi/sw_patches/DELGUI.PAS">$,
           $sw_patches/DELGUI.PAS</A>.$]
   ]).

answer(delphiStdcall,
   [
   text = [$The AMZI.PAS file in 3.3 Jan96b and earlier does not work $,
           $with Delphi 2.0.  You can either download a new copy of $,
           $<A HREF="ftp://ftp.amzi.com/pub/users/amzi/sw_patches/AMZI.PAS">AMZI.PAS</A> and $,
           $<A HREF="ftp://ftp.amzi.com/pub/users/amzi/sw_patches/AMZI.DCR">AMZI.DCR</A> $,
           $(the Amzi! icon) from the sw_patches $,
           $directory of our ftp site, or fix your existing copy. $,
           $The fix is simply to change the 'far;' declarations to $,
           $'stdcall;' for all of the DLL calls to A_MW3D.DLL. $,
           $(Leave the calls to A_MW2D.DLL as they were.)$],
   note = [debugEmbed]
   ]).

answer(vbMissingAmziVB,
   [
   text = [$In the Personal Edition of the Jan96 release, AMZIVB.BAS $,
           $and AMZIVB3.BAS were inadvertently omitted.  You can download $,
           $both files from sw_patches in $,
           $<A HREF="ftp://ftp.amzi.com/pub/users/amzi/sw_patches/AMZIVB.BAS">$,
           $sw_patches/AMZIVB.BAS</A> and$, 
           $<A HREF="ftp://ftp.amzi.com/pub/users/amzi/sw_patches/AMZIVB3.BAS">$,
           $sw_patches/AMZIVB3.BAS</A>.$]
   ]).

answer(vbStrToTerm,
   [
   text = [$lsStrToTerm is incorrectly spelled in AMZIVB.BAS as lsStrtoTerm. $,
           $You can correct it yourself, or download a new copy from sw_patches in $,
           $<A HREF="ftp://ftp.amzi.com/pub/users/amzi/sw_patches/AMZIVB.BAS">$,
           $sw_patches/AMZIVB.BAS</A>.$],
   note = [debugEmbed]
   ]).

%
% noAnswer is a special answer that is used when there are no more questions
% to ask the user and no answer can be found
%
answer(noAnswer,
   [
   text = [$We are unable to diagnose your problem at this time. $,
           $Please e-mail a description of your problem to $,
           $<A HREF="mailto:support@amzi.com">support@amzi.com</A>. $,
           $You should receive a reply in 48 hours or less.$]
   ]).

% ------------------------------------------------------------------- %
%                               Notes                                 %
% ------------------------------------------------------------------- %

note(debugEmbed,
   [
   text = [$For more information on debugging embedded Prolog modules $,
           $we suggest you see <A HREF="ftp://ftp.amzi.com/pub/users/amzi/articles/APIDEBUG.TXT">Debugging Hints</A>.$]
   ]).

note(modules,
   [
   text = [$There are two types of compiled predicates, global and local.  Global $,
           $predicates can be used from other modules, local ones only in the module $,
           $they are defined in.$,
           $<P>When you consult a number of interpreted files, all of the interpreted $,
           $predicates are effectively global.$,
           $<P>When you compile a file and do not have any :-import or :-export $,
           $directives, then all the compiled predicates are, by default, global.  So if $,
           $you don't use any :-import or :-export statements, then when you load a $,
           $number of compiled files it is the same as consulting those files interpreted.$,
           $<P>If you add a :-import or :-export directive to a file, then, by default all $,
           $of the predicates in that file become local.  Only the predicates listed in $,
           $the :-import or :-export are global.  Note that :-import and :-export have $,
           $the same function, making a predicate global.  The two forms are just for $,
           $convenience.$,
           $<P>If any predicates in a module with :-import/export directives refer to $,
           $dynamic database predicates, then those predicates must be declared as $,
           $:-import (or :-export).  Also, any predicates referred to in findall, bagof $,
           $or setof need to be declared.$]
   ]).

% ------------------------------------------------------------------- %
%                           Special Notes                             %
% ------------------------------------------------------------------- %
%
% systemError is a special note that is appended to an error message
%
note(systemError,
   [
   text = [$<HR>Please write down the above message and what you were $,
           $doing when this problem occurred, including the values you $,
           $typed into the forms.  E-mail this information $,
           $to <A HREF="mailto:info@amzi.com">us</A>.  Thank you.$]
   ]).

%
% answerHeader and answerFooter are special notes that are used to
% bracket the output from sendAnswer
%
note(answerHeader, 
   [
   text = [$We have found an answer to your problem!  It is:<P>$]
   ]).

note(answerFooter,
   [
   text = [$<P><HR><P>This is an experimental version of our new automated $,
           $technical support system.  We are very interested in hearing $,
           $your feedback at <A HREF="mailto:info@amzi.com">mary@amzi.com</A>. $,
           $We would be especially grateful if this system does not provide you an answer, if you $,
           $can tell us what your form inputs were and describe your problem. $,
           $We'll e-mail you an answer and try to add it to this system.$,
           $<P>Thank you! $]
   ]).
