/*------------------------------------------------------------------------
                  HTMLEx V2.17 - HTML Enhancement Filter
      ISAPI Filter (DLL) for Microsoft Internet Information Server
	 ISAPI Extension (DLL) for Microsoft Internet Information Server
         ... and Microsoft Personal Web Server 1.0 (Windows 95)  
--------------------------------------------------------------------------
(c)Zoltan Pekic, 1996. zoltanp@rijeka.riteh.hr, administrator@www.riteh.hr
--------------------------------------------------------------------------
 THIS SOFTWARE IS FREEWARE - IT MAY BE FREELY COPIED, DISTRIBUTED
  AND ENHANCED IN FUNCTIONALITY AS LONG AS THE ORIGINAL AUTHOR'S
                         NAME IS PRESERVED
------------------------------------------------------------------------*/
#include <windows.h>
#include <winsock.h>  // include wsock32.lib in Project Settings (Link)
#include <stdio.h>
#include <stdlib.h>
#include <httpfilt.h> // From ISAPI SDK
#include <httpext.h>  // From ISAPI SDK
#include <time.h>
#include <math.h>

#define MAX_WRITEBUFF 8192 // Magic number...

#define MAX_EXPAND    8192 // Filtered HTM max. 8k longer than the original!
#define MAX_TAG         64 // Place for upto 64 different HTMLEx tags
#define MAX_TAGNAME     32 // Max length of tag's name
#define MAX_TAGVALUE  2048 // Max length of tag's expanded value

//#define KEY_MIMEMAP "SYSTEM\\CurrentControlSet\\Services\\InetInfo\\Parameters\\MimeMap"
#define KEY_W3SVC   "SYSTEM\\CurrentControlSet\\Services\\W3SVC"
#define KEY_PARAMS  "Parameters"
#define KEY_HTMLEX  "HTMLEx"
#define KEY_DIGITS  "Digits"
#define KEY_TAGS    "Tags"
#define KEY_VARS    "Variables"
#define KEY_SUBS    "Subroutines"

// Hard-coded defaults which can be overriden by registry and/or INI
#define DEFAULTLOADFILE        "Default.htm" 
#define EXPANDABLEEXTENSIONS   ".htm,.html,.stm,.shtm,.shtml"
#define UNEXPANDABLEEXTENSIONS ".exe,.dll,.asp"
#define DATEFORMAT             "%A, %B %d, %Y."
#define TIMEFORMAT             "%H:%M:%S"
#define DIGITCNT               5
#define EXPANDBYDEFAULT        1  // YES, <!HTMLEx noexpand> needed to skip expansion!
#define EXPANDDEFAULTPAGES     1
#define EXPANDOTHERPAGES       1
#define GENERATELOGS           1
#define GENERATEERRFILES       1  // Errors and warning into separate file!
#define MAXLOGFILESIZE         64 // By default, let LOG files grow to a maximum of 64k
#define IGNOREADDRESS          "0.0.0.0" // ie, none!
#define TAG_START              "%"
#define TAG_END                "%"
#define LOG404ERRORS           0  // Do not trash App Evt log with 'File not found' messages!
//
// SCRIPT LANGUAGE DEFINITIONS / VARIABLES
//
#define EXECUTEBYDEFAULT       0  // No, <!HTMLEx execute> needed to do it!
#define CMD_START              "{"
#define CMD_END                "}"
#define MAXLOOP                64
enum KeywordID          {KW_IF,KW_WHILE,KW_REPEAT,KW_WRITE,KW_FILE,KW_LOG,KW_EVENT,KW_RET  ,KW_EXIT,KW_SETHEADER,KW_ADDHEADER,KW_HEADER,KW_CALL,KW_ELSE,KW_ENDIF,KW_UNTIL,KW_ENDWHILE,KW_SUB      ,KW_INVALID,KW_SETVAR,KW_COMMENT};
const char *pcKeyword[]={"IF" ,"WHILE" ,"REPEAT" ,"WRITE" ,"FILE" ,"LOG" ,"EVENT" ,"RETURN","EXIT" ,"SETHEADER" ,"ADDHEADER" ,"HEADER" ,"CALL" ,"ELSE" ,"ENDIF" ,"UNTIL" ,"ENDWHILE" ,"SUBROUTINE","\0"};

const char *pcHTMLExDescF="HTMLEx/2.17 - HTML Enhancement Filter",
           *pcHTMLExDescE="HTMLEx/2.17 - HTML Enhancement Extension",
           *pcHTMLExVer= "<a href=\042http://161.53.40.11/HTMLEx/\042>HTMLEx/2.17</a>",
           *pcHTMLExLogo="<a HREF=\042http://161.53.40.11/HTMLEx/\042><img src=\042/HTMLExR.GIF\042 alt=\042HTMLEx enhanced web site\042 border=0></a>",
		   *pcHTMLExAuthor="&copy;<a href=\042mailto:zoltanp@rijeka.riteh.hr\042>Zoltan Pekic</a>, 1996.",
           *pcEmpty="\0\0";

//
// Needed in Debug mode
//
BOOL   bDebugMode=FALSE;
HANDLE hStdErr=NULL;

HANDLE hEvent;
HKEY   hKeyHTMLEx,hKeyTags,hKeyVariables,hKeySubroutines;
char   cHostname[64];
// From KEY_PARAMS
char cRegDefault_Load_File[128];
char cRegAdminEmail[64];
char cRegAdminName[64];
// From KEY_HTMLEX
char cRegS404Reply[256];
char cRegD404Reply[128];
char cRegTempDir[128];
char cRegExpandableExtensions[128];
char cRegUnexpandableExtensions[128];
char cRegTagStart[16],cRegTagEnd[16];
char cRegIgnoreAddress[128];
char cRegDateFormat[128];
char cRegTimeFormat[128];
char cRegDigitImg[10][128];   
const char *pcDefaultDigitImg[]={"/0.gif","/1.gif","/2.gif","/3.gif","/4.gif","/5.gif","/6.gif","/7.gif","/8.gif","/9.gif"};
DWORD dwRegDigitCnt;
DWORD dwRegExpandByDefault;
DWORD dwRegExpandDefaultPages;
DWORD dwRegExpandOtherPages;
DWORD dwRegGenerateLogs;
DWORD dwRegGenerateErrFiles;
DWORD dwRegMaxLogFileSize;
DWORD dwRegLog404Errors;
//
DWORD dwRegExecuteByDefault;
DWORD dwRegMaxLoop;
char  cRegCmdStart[16],cRegCmdEnd[16];
//
// 
//
//#define THUMBNAIL
#ifdef  THUMBNAIL
#define THUMBNAIL_VROOT     "/Thumbnail"    // IIS virtual thumbnail directory
#define IMGDLLNAME          "QVDLL32.DLL"	// Use this DLL do the imaging stuff...
#define KEY_VROOTS          "Virtual Roots"	// IIS keeps virtual directory definitions here
#define MIN_THUMBNAILEXTENT 32              // Smallest thumbnail is 32*32
char    cRegThumbnailPath[256];				// Thumbnail cache physical location
HINSTANCE  hInstImgDLL;                     // Imaging DLL handle
#endif

//#define WINET 
#ifdef  WINET
#include  "d:\msdev\InetSDK\Include\WinInet.h"
#define   MAX_WINET_FILE 256*1024 // Allow 256k to be read via Internet
#define   WINETDLLNAME "WININET.DLL"
INTERNET_VERSION_INFO   WinetVersion;
BOOL      (WINAPI *fpInternetQueryOption)(IN HINTERNET hInternet OPTIONAL,IN DWORD dwOption,OUT LPVOID lpBuffer OPTIONAL,IN OUT LPDWORD lpdwBufferLength);
HINTERNET (WINAPI *fpInternetOpenA)(IN LPCSTR lpszAgent,IN DWORD dwAccessType,IN LPCSTR lpszProxy OPTIONAL,IN LPCSTR lpszProxyBypass OPTIONAL,IN DWORD dwFlags);
HINTERNET (WINAPI *fpInternetOpenUrlA)(IN HINTERNET hInternet,IN LPCSTR lpszUrl,IN LPCSTR lpszHeaders OPTIONAL,IN DWORD dwHeadersLength,IN DWORD dwFlags,IN DWORD dwContext);
BOOL      (WINAPI *fpInternetReadFile)(IN HINTERNET hFile,IN LPVOID lpBuffer,IN DWORD dwNumberOfBytesToRead,OUT LPDWORD lpdwNumberOfBytesRead);
BOOL      (WINAPI *fpInternetCloseHandle)(IN HINTERNET hInternet);
HINSTANCE hInstWinetDLL;
#endif

//
// If URLs destination in one of these directories, then skip it!
//
#define MAX_SKIPDIR  16
int     iSkipdirCount=0;
char    cSkipdir[MAX_SKIPDIR][256];
//
// Memory blocks which make the server's response...
//
#define MAX_MBLOCK 16384  
typedef struct stMemBlock
{
 int     iLast;
 char*   MB[MAX_MBLOCK];
} MEMBLOCK,*PMEMBLOCK;
//
// Custom http headers returned by server
//
#define MAX_HNAME    64
#define MAX_HVALUE 1024
#define MAX_HBLOCK    8 
typedef struct stHBlock
{
 int iLast;
 char cHName [MAX_HBLOCK][MAX_HNAME];
 char cHValue[MAX_HBLOCK][MAX_HVALUE];
 BOOL bAdd[MAX_HBLOCK];
} HBLOCK,*PHBLOCK;

typedef struct stOutputDataBlock
{
 PHBLOCK pHeader;
 char    *pcData;
} ODBLOCK,*PODBLOCK;

struct stExecConf
{
 char  *pcScriptStart;            // Script image in memory
 char  *pcScriptEnd;              // 
 char  cSubStart[64];             // Subroutine start marking
 BOOL  bIsFilter;                 // If FALSE, means we are doing a ISAPI extension...
 char  cCmdStart[16],cCmdEnd[16]; // Default: cRegCmd...
 DWORD dwLenCS,      dwLenCE;
 DWORD dwMaxLoop;
 char  cMaxLoop[32]; 
};

const HKEY RootKey[]={HKEY_CLASSES_ROOT,HKEY_CURRENT_USER,HKEY_LOCAL_MACHINE,HKEY_USERS};

enum StackType {ST_EMPTY,ST_INT,ST_REAL,ST_STRING,ST_VARPTR};
#define MAX_STACKSTRING 1024
typedef struct stStackEntry
{
 enum StackType Type;
 DWORD          dwVal;
 double         fpVal;
 char           cVal[MAX_STACKSTRING];
} STACKENTRY,*PSTACKENTRY;

#define MAX_DATASTACK 16
typedef struct stDataStack
{
 int iDSP;	// Points to last occupied entry!
 STACKENTRY Data[MAX_DATASTACK];
} DATASTACK,*PDATASTACK;

const stStackEntry vNothing={ST_EMPTY,0,0.,""};
const stStackEntry v0={ST_INT,0,0.,""};
const stStackEntry v1={ST_INT,1,0.,""};
const stStackEntry v2={ST_INT,2,0.,""};
//--------------------------------------------------

//--------------------------------------------------
#define MAX_NAMELEN 64
enum OpType {OP_EMPTY,OP_UN,OP_BIN,OP_FUN};
typedef struct stOpEntry
{
 enum OpType Type;
 int         iPri;
 char        cName[MAX_NAMELEN];
 int         iPar;
 BOOL (*fpF)(PDATASTACK,int,char*,char*,int);
} OPENTRY,*POPENTRY;

#define MAX_OPSTACK 16
typedef struct stOpStack
{
 int iOSP; // Points to last occupied entry!
 OPENTRY Op[MAX_DATASTACK];
} OPSTACK,*POPSTACK;

typedef struct stVariable
{
 char cXName[MAX_NAMELEN];
 char cName [MAX_NAMELEN];
 STACKENTRY V;
} STVARIABLE,*PSTVARIABLE;

#define MAX_VBLOCK 64
typedef struct stVarBlock
{
 int iLast;
 PSTVARIABLE Var[MAX_VBLOCK];
} VARBLOCK,*PVARBLOCK;
   
// Forward definition... 
char* EvalEx(char*,char*,struct stExecConf,struct stStackEntry*,struct stVarBlock*,BOOL,char* pcHTM,char* pcERR,int iCnt);

//////////////////////////////////////////////
//               MISC ROUTINES
//////////////////////////////////////////////
BOOL FileExists(char *pcFileName)
{
 HANDLE FE=CreateFile(pcFileName,0,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
 if (FE==INVALID_HANDLE_VALUE) return (FALSE);
 else CloseHandle(FE);
 return (TRUE);
};

BOOL TempFileExists(char *pcTempFileName,DWORD dwFileID)
{
 sprintf(pcTempFileName,"%s\\%08X.TMP",cRegTempDir,dwFileID);
 return FileExists(pcTempFileName);
}

void Report(WORD wEventType,DWORD dwEventID,char* pcFormat,char* pcArg,char *pcHTMName,char *pcERRName,int iSection)
{
 char cBuff[1024];

 if (pcERRName) // .ERR file name given?
 {
  char cSection[32],cEntryName[32];
  sprintf(cSection,"%i",iSection);
  switch(wEventType)
  {
   case EVENTLOG_ERROR_TYPE      :sprintf(cEntryName,"E%03i",dwEventID);break;
   case EVENTLOG_WARNING_TYPE    :sprintf(cEntryName,"W%03i",dwEventID);break;
   case EVENTLOG_INFORMATION_TYPE:sprintf(cEntryName,"I%03i",dwEventID);break;
   default                       :sprintf(cEntryName,"U%03i",dwEventID);break;
  };
  sprintf(cBuff,pcFormat,pcArg);
  WritePrivateProfileString(cSection,cEntryName,cBuff,pcERRName);
 }
 else           // No
 {
  char cFormat[256];
  if (bDebugMode) // Debug: output to console!
  {
   DWORD dwWritten;
   switch(wEventType)
   {
    case EVENTLOG_ERROR_TYPE      :sprintf(cFormat,"\n***E%03i - %s",dwEventID,pcFormat);break;
    case EVENTLOG_WARNING_TYPE    :sprintf(cFormat,"\n***W%03i - %s",dwEventID,pcFormat);break;
    case EVENTLOG_INFORMATION_TYPE:sprintf(cFormat,"\n***I%03i - %s",dwEventID,pcFormat);break;
    default                       :sprintf(cFormat,"\n***U%03i - %s",dwEventID,pcFormat);break;
   };
   WriteConsole(hStdErr,cBuff,sprintf(cBuff,cFormat,pcArg),&dwWritten,NULL);
   return;
  };
  if (hEvent)     // IIS: generate an entry in application event log
  {
   const char *pcBuff=cBuff;
   if (pcHTMName) sprintf(cFormat,"\r\n%s:\r\n%s\0",pcHTMName,pcFormat); else sprintf(cFormat,"\r\n%s\0",pcFormat);
   sprintf(cBuff,cFormat,pcArg);
   ReportEvent(hEvent,wEventType,0,dwEventID,NULL,1,0,&pcBuff,NULL);
  };
 };
};

char *WSErrMsg(int WSErrCode)
{
 switch (WSErrCode)
 {
  case WSANOTINITIALISED:return ("WSAStartup failure.");break;
  case WSAENETDOWN      :return ("Network subsystem failure.");break;
  case WSAHOST_NOT_FOUND:return ("Authoritative Answer Host not found.");break;
  case WSATRY_AGAIN	    :return ("Non-Authoritative Host not found, or SERVERFAIL.");break;
  case WSANO_RECOVERY   :return ("Non recoverable errors, FORMERR, REFUSED, NOTIMP.");break;
  case WSANO_DATA       :return ("Valid name, no data record of requested type.");break;
  case WSAEINPROGRESS   :return ("A blocking Windows Sockets operation is in progress.");break;
  case WSAEINTR         :return ("The (blocking) call was canceled via WSACancelBlockingCall.");break;
  default               :return ("Unspecified WinSock error.");
 };
};

// This can be a time consuming task on slow networks!
// Use the %CurrentClientHost% tag sparingly!
void ResolveIPAddress(char* pcHost,char* pcHTM,char* pcErr,int iCnt)
{
 hostent *pHE=NULL;
 long lAddress;
 char cBuff[256],*pcHN;
 BOOL bDottedDecimal=TRUE;
  
 pcHN=pcHost;
 while (*pcHN && bDottedDecimal)
 {
  bDottedDecimal=((*pcHN>='.') && (*pcHN<='9'));
  ++pcHN;
 };
 if (bDottedDecimal)	                // Already done?
 {										// No, some Winsock labour required...
  lAddress=inet_addr(pcHost);
  if (lAddress==INADDR_NONE)
  {
   sprintf(cBuff,"<IP address %s is invalid>",pcHost);
   Report(EVENTLOG_WARNING_TYPE,103,"%s",cBuff,pcHTM,pcErr,iCnt);
   strcpy(pcHost,cBuff);
  }
  else                      
  {
   if (pHE=gethostbyaddr((char*) &lAddress,4,PF_INET)) strcpy(pcHost,pHE->h_name);
   else	
   {
    sprintf(cBuff,"<%s ... %s>",pcHost,WSErrMsg(WSAGetLastError()));
    Report(EVENTLOG_WARNING_TYPE,104,"%s",cBuff,pcHTM,pcErr,iCnt);
    strcpy(pcHost,cBuff);
   }
  }
 }
};

void SystemTime2TM(SYSTEMTIME ST,struct tm* pTM) // Glue C++ runtime and Win32 API together
{
 pTM->tm_year=(int) ST.wYear-1900;
 pTM->tm_mon= (int) ST.wMonth-1;
 pTM->tm_mday=(int) ST.wDay;
 pTM->tm_hour=(int) ST.wHour;
 pTM->tm_min= (int) ST.wMinute;
 pTM->tm_sec= (int) ST.wSecond;
 pTM->tm_wday=(int) ST.wDayOfWeek;
 pTM->tm_yday=0;   // No need for complicated calculations...
 pTM->tm_isdst=-1; // Unknown daylight status, but who cares for a few little hours...
};

void GetRidOfNullChars(char* pcS,char cReplacement)
{
 BOOL bCont=TRUE;

 while (bCont)
 {
  if(*pcS=='\0')
  {
   *pcS++=cReplacement;
   bCont=(*pcS != '\0');
  }
  else pcS++;
 };
};

void GraphCnt(int iCnt,int iLen,char* pcImg,char* pcGraphCnt)
{
 char  cImg[MAX_TAGVALUE],cDigits[16],cFormat[16]; 

 if (iLen) 
 {
  sprintf(cFormat," 0%ii",(iLen>9 ? 9 : (iLen<2 ? 2 : iLen)));
  cFormat[0]='%';
  sprintf(cDigits,cFormat,iCnt);
 }
 else sprintf(cDigits,"%i",iCnt);
 char* pcDigits=cDigits;
 while (*pcDigits)
 {
  sprintf(cImg,"<img src=\042%s\042 alt=\042%c\042>",pcImg+128*((int) *pcDigits-'0'),*pcDigits);
  strcat(pcGraphCnt,cImg);
  pcDigits++;
 };
};

/////////////////////////////////////////////////////////////////
//	             SCRIPT LANGUAGE ROUTINES
/////////////////////////////////////////////////////////////////
BOOL IsValidVarNameChar(char c)
{
 if (isalnum(c)) return (TRUE);
 if (strchr("_$",c)) return (TRUE);
 return (FALSE);
};

int FindKeyword(char *pcPos)
{
 char *pcS=pcPos;
 char cK[MAX_NAMELEN],*pcK=cK;

 if (isalpha(*pcS))
 {
  do
  {
   *pcK++=*pcS++;
   *pcK='\0';
  } while (IsValidVarNameChar(*pcS));
  for(int i=0;pcKeyword[i][0];i++) if (stricmp(cK,pcKeyword[i])==0) return (i);
  return (KW_SETVAR);
 }
 else
 {
// A nasty bug in the following line corrected by 
// Robert Wojciechowski Jr. (robertw@wojo.com)
// Thanx!
  if (*pcS=='!') return (KW_COMMENT);
  if ((*pcS++=='/') && (*pcS=='/')) return (KW_COMMENT);
 };
 return (KW_INVALID);
};

int FindVar(char *pcName,PVARBLOCK pVBlock,BOOL bShouldBeThere,char *pcHTM,char *pcERR,int iCnt)
{
 for(int i=pVBlock->iLast;i>0;i--)
 if (stricmp(pVBlock->Var[i]->cName,pcName)==0)
 {
  if (i<=10)	// One of the read-only variables?
  {
   WORD ST[8];
   MEMORYSTATUS MS; 
   switch (i)
   {
    case 10:
	     MS.dwLength=sizeof(MEMORYSTATUS);
	     GlobalMemoryStatus(&MS);
         pVBlock->Var[i]->V.dwVal=MS.dwMemoryLoad;
		 break;
    case  9:
         pVBlock->Var[i]->V.dwVal=GetTickCount();
		 break;
    default:
         GetSystemTime((LPSYSTEMTIME) &ST);
         pVBlock->Var[i]->V.dwVal=ST[i-1];
		 break;
   };
  };
  return (i);
 };
 if (bShouldBeThere) Report(EVENTLOG_WARNING_TYPE,152,"Undefined variable '%s' used",pcName,pcHTM,pcERR,iCnt);
 return 0;
};

BOOL AddVBlockEntry(char* pcXName,char *pcName,stStackEntry Value,PVARBLOCK pVBlock,char *pcHTM,char *pcERR,int iCnt)
{
 int i;

 if (i=FindVar(pcName,pVBlock,FALSE,pcHTM,pcERR,iCnt)) // Variable already defined?
 {
  pVBlock->Var[i]->V=Value;
  return (TRUE);
 };
 i=pVBlock->iLast;					  // No, append it to the end of list!
 if(i<MAX_VBLOCK)
 {
  i++;		   
  if (pVBlock->Var[i]=(PSTVARIABLE) GlobalAlloc(GPTR,sizeof(stVariable)))
  {
   if (pcXName)
   {
    strcpy(pVBlock->Var[i]->cXName,pcXName);strupr(pVBlock->Var[i]->cXName);
   }
   else pVBlock->Var[i]->cXName[0]='\0';
   strcpy(pVBlock->Var[i]->cName,pcName);strupr(pVBlock->Var[i]->cName);
   pVBlock->Var[i]->V=Value;
   pVBlock->iLast=i;
   return (TRUE);
  }
  else Report(EVENTLOG_ERROR_TYPE,259,"GlobalAlloc(GPTR,) failed in AddVBlockEntry",NULL,pcHTM,pcERR,iCnt);
 }
 else Report(EVENTLOG_ERROR_TYPE,257,"Out of Variable space in AddVBlockEntry",NULL,pcHTM,pcERR,iCnt);
 return (FALSE);
};

BOOL AddMBlockEntry(char *pcS,char *pcE,PMEMBLOCK pMBlock,char *pcHTM,char *pcERR,int iCnt)
{
 DWORD dwMBlockLen;
 int i;
 char cBuff[64];

 if (*pcS=='\0') return (TRUE); // This should never happen...
 if(pMBlock->iLast<MAX_MBLOCK)
 {
  dwMBlockLen=pcE-pcS+1;
//if ((strncmp(pcS,"\r\n",2)==0) && (dwMBlockLen==2)) return (TRUE); // Just a blank line, ignore it!
  i=pMBlock->iLast+1;
  if (pMBlock->MB[i]=(char*) GlobalAlloc(GPTR,dwMBlockLen+1))
  {
   memcpy((char*) pMBlock->MB[i],pcS,dwMBlockLen);
   pMBlock->iLast=i;
   return (TRUE);
  }
  else
  {
   ltoa(dwMBlockLen,cBuff,10);
   Report(EVENTLOG_ERROR_TYPE,252,"GlobalAlloc(GPTR,%s) failed in AddMBlockEntry",cBuff,pcHTM,pcERR,iCnt);
  };
 }
 else Report(EVENTLOG_ERROR_TYPE,256,"Out of Memory block space in AddMBlockEntry",NULL,pcHTM,pcERR,iCnt);
 return (FALSE);
};


BOOL AssertType(stStackEntry Arg,enum StackType RequiredType,char *pcErrMsg,char *pcHTM,char *pcERR,int iCnt)
{
 if (Arg.Type==RequiredType) return (TRUE);
 Report(EVENTLOG_ERROR_TYPE,255,"%s",pcErrMsg,pcHTM,pcERR,iCnt);
 return (FALSE);
};

BOOL IsTrue(stStackEntry Arg,char *pcHTM,char *pcERR,int iCnt)
{
 if (Arg.Type==ST_INT) return (Arg.dwVal != 0);
 if (Arg.Type==ST_REAL) return (Arg.fpVal != 0);
 if (Arg.Type==ST_STRING) return (strlen(Arg.cVal) != 0);
 Report(EVENTLOG_WARNING_TYPE,150,"Invalid logical value - FALSE returned",NULL,pcHTM,pcERR,iCnt);
 return (FALSE);
};

char* GetVarName(char *pcS,char *pcE,char *pcVarName,stExecConf ECD,PVARBLOCK pVBlock,char *pcHTM,char *pcERR,int iCnt)
{
// const char* pcVFormat[]={"%s","%s[%03i]","%s[%03i][%03i]","%s[%03i][%03i][%03i]"};
 int iXCnt=0;
// int iX[3];
 STACKENTRY X;
 char cName[MAX_NAMELEN],*pcName=cName;
 char cStringIndex[MAX_NAMELEN];
 char *pcP=pcS;

 *pcVarName='\0';                  // Means failure!
 *pcName='\0';
// if (!isalpha(*pcP)) return (pcP); // Variable name must start with a letter!
 while (IsValidVarNameChar(*pcP) && (pcP<=pcE))
 {
  *pcName++=*pcP++;
  *pcName='\0';
 };
 strcpy(pcVarName,cName);
 if (pcP<pcE)
 {
  if (*pcP=='[')
  {
   if (iXCnt>3)
   {
    Report(EVENTLOG_ERROR_TYPE,260,"Too many indexes",NULL,pcHTM,pcERR,iCnt);
    return (pcP);
   }
   iXCnt++;
   pcP++;
   pcP=EvalEx(pcP,pcE,ECD,&X,pVBlock,FALSE,pcHTM,pcERR,iCnt);
//   if (!AssertType(X,ST_INT,"Array index must be an integer")) return (pcP);
//   else	iX[iXCnt]=X.dwVal;
   switch (X.Type)
   {
    case ST_INT:
	     sprintf(X.cVal,"[%03i]",X.dwVal);
		 strcat(pcVarName,X.cVal);
	     break;
	case ST_STRING:
		 strupr(X.cVal);
	     sprintf(cStringIndex,"[%s]",X.cVal);
	     strcat(pcVarName,cStringIndex);
	     break;
	default:
         Report(EVENTLOG_ERROR_TYPE,281,"Invalid array index - must be integer or string",NULL,pcHTM,pcERR,iCnt);
		 return(pcP);
	     break;
   };
   if (*pcP==']') pcP++;
   else
   {
    Report(EVENTLOG_ERROR_TYPE,261,"Missing ] after array index",NULL,pcHTM,pcERR,iCnt);
    return (pcP);
   };
  }
 };
// sprintf(pcVarName,pcVFormat[iXCnt],cName,iX[1],iX[2],iX[3]);
 return (pcP);
};

BOOL Dummy(PDATASTACK pD,int iPCnt,char *pcHTM,char* pcERR,int iCnt) {return (TRUE);};

BOOL SetStack(PDATASTACK pD,int iPop,enum StackType NewType,char *pcHTM,char *pcERR,int iCnt)
{
 pD->iDSP -=iPop;
 if (pD->iDSP < 1)
 {
  Report(EVENTLOG_ERROR_TYPE,280,"Data stack underflow",NULL,pcHTM,pcERR,iCnt);
  return (FALSE);
 }
 else pD->Data[pD->iDSP].Type=NewType;
 return (TRUE);
};

//////////// O P E R A T O R S ////////////
BOOL OpSHL(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-1],ST_INT,"<< left operand must be integer",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP],ST_INT,"<< right operand must be integer",pcHTM,pcERR,iCnt))
  {	
   pD->Data[pD->iDSP-1].dwVal = pD->Data[pD->iDSP-1].dwVal << pD->Data[pD->iDSP].dwVal;
   return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};

BOOL OpSHR(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-1],ST_INT,">> left operand must be integer",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP],ST_INT,">> right operand must be integer",pcHTM,pcERR,iCnt))
  {
   pD->Data[pD->iDSP-1].dwVal = pD->Data[pD->iDSP-1].dwVal >> pD->Data[pD->iDSP].dwVal;
   return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};

BOOL OpNEG(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 switch (pD->Data[pD->iDSP].Type)
 {
  case ST_INT:
       pD->Data[pD->iDSP].dwVal=- (long) pD->Data[pD->iDSP].dwVal;
       return (TRUE); 
  case ST_REAL:
       pD->Data[pD->iDSP].fpVal=-(pD->Data[pD->iDSP].fpVal);
       return (TRUE); 
  default:
       Report(EVENTLOG_ERROR_TYPE,282,"Negation defined on numeric argument only",NULL,pcHTM,pcERR,iCnt);
 };
 return (FALSE);
};

BOOL OpBITAND(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-1],ST_INT,"AND left operand must be integer",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP],ST_INT,"AND right operand must be integer",pcHTM,pcERR,iCnt))
  {
   pD->Data[pD->iDSP-1].dwVal &= pD->Data[pD->iDSP].dwVal;
   return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};

BOOL OpBITOR(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-1],ST_INT,"OR left operand must be integer",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP],ST_INT,"OR right operand must be integer",pcHTM,pcERR,iCnt))
  {
   pD->Data[pD->iDSP-1].dwVal |= pD->Data[pD->iDSP].dwVal;
   return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};

BOOL OpNOT(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 pD->Data[pD->iDSP].dwVal=IsTrue(pD->Data[pD->iDSP],pcHTM,pcERR,iCnt) ? TRUE : FALSE;
 return (SetStack(pD,0,ST_INT,pcHTM,pcERR,iCnt));
};

BOOL OpLOGOR(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 pD->Data[pD->iDSP-1].dwVal=IsTrue(pD->Data[pD->iDSP-1],pcHTM,pcERR,iCnt) || IsTrue(pD->Data[pD->iDSP],pcHTM,pcERR,iCnt);
 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
};

BOOL OpLOGAND(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 pD->Data[pD->iDSP-1].dwVal=IsTrue(pD->Data[pD->iDSP-1],pcHTM,pcERR,iCnt) && IsTrue(pD->Data[pD->iDSP],pcHTM,pcERR,iCnt);
 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
};

BOOL OpMUL(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 switch (pD->Data[pD->iDSP-1].Type)
 {
  case ST_INT:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
		     pD->Data[pD->iDSP-1].dwVal *= pD->Data[pD->iDSP].dwVal;
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		case ST_REAL:
			 pD->Data[pD->iDSP-1].fpVal = (long) pD->Data[pD->iDSP-1].dwVal * pD->Data[pD->iDSP].fpVal;
			 return (SetStack(pD,1,ST_REAL,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  case ST_REAL:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
             pD->Data[pD->iDSP-1].fpVal = pD->Data[pD->iDSP-1].fpVal * (long) pD->Data[pD->iDSP].dwVal;
			 return (SetStack(pD,1,ST_REAL,pcHTM,pcERR,iCnt));
		case ST_REAL:
		     pD->Data[pD->iDSP-1].fpVal *= pD->Data[pD->iDSP].fpVal;
			 return (SetStack(pD,1,ST_REAL,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  default:break;
 };
 Report(EVENTLOG_ERROR_TYPE,282,"* failed on bad or incompatible arguments",NULL,pcHTM,pcERR,iCnt);
 return (FALSE);
};

BOOL OpDIV(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 char *pcDIV0="Division by 0, 0 returned";

 switch (pD->Data[pD->iDSP-1].Type)
 {
  case ST_INT:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
		     if (pD->Data[pD->iDSP].dwVal==0)
			 {
              Report(EVENTLOG_WARNING_TYPE,156,pcDIV0,NULL,pcHTM,pcERR,iCnt);
			  pD->Data[pD->iDSP-1].dwVal=0;
			 }
			 else pD->Data[pD->iDSP-1].dwVal=pD->Data[pD->iDSP-1].dwVal / pD->Data[pD->iDSP].dwVal;
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		case ST_REAL:
		     if (pD->Data[pD->iDSP].fpVal==0)
			 {
              Report(EVENTLOG_WARNING_TYPE,156,pcDIV0,NULL,pcHTM,pcERR,iCnt);
			  pD->Data[pD->iDSP-1].fpVal=0;
			 }
			 else pD->Data[pD->iDSP-1].fpVal=(long)(pD->Data[pD->iDSP-1].dwVal) / pD->Data[pD->iDSP].fpVal;
			 return (SetStack(pD,1,ST_REAL,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  case ST_REAL:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
		     if (pD->Data[pD->iDSP].dwVal==0)
			 {
              Report(EVENTLOG_WARNING_TYPE,156,pcDIV0,NULL,pcHTM,pcERR,iCnt);
			  pD->Data[pD->iDSP-1].fpVal=0;
			 }
			 else pD->Data[pD->iDSP-1].fpVal= pD->Data[pD->iDSP-1].fpVal / (long)(pD->Data[pD->iDSP].dwVal);
			 return (SetStack(pD,1,ST_REAL,pcHTM,pcERR,iCnt));
		case ST_REAL:
		     if (pD->Data[pD->iDSP].fpVal==0)
			 {
              Report(EVENTLOG_WARNING_TYPE,156,pcDIV0,NULL,pcHTM,pcERR,iCnt);
			  pD->Data[pD->iDSP-1].fpVal=0;
			 }
			 else pD->Data[pD->iDSP-1].fpVal = pD->Data[pD->iDSP-1].fpVal / pD->Data[pD->iDSP].fpVal;
			 return (SetStack(pD,1,ST_REAL,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  default:break;
 };
 Report(EVENTLOG_ERROR_TYPE,282,"/ failed on bad or incompatible arguments",NULL,pcHTM,pcERR,iCnt);
 return (FALSE);
};

BOOL OpADD(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 switch (pD->Data[pD->iDSP-1].Type)
 {
  case ST_INT:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
		     pD->Data[pD->iDSP-1].dwVal += pD->Data[pD->iDSP].dwVal;
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		case ST_REAL:
			 pD->Data[pD->iDSP-1].fpVal=(long)(pD->Data[pD->iDSP-1].dwVal)+pD->Data[pD->iDSP].fpVal;
			 return (SetStack(pD,1,ST_REAL,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  case ST_REAL:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
			 pD->Data[pD->iDSP-1].fpVal=pD->Data[pD->iDSP-1].fpVal+(long)(pD->Data[pD->iDSP].dwVal);
			 return (SetStack(pD,1,ST_REAL,pcHTM,pcERR,iCnt));
		case ST_REAL:
			 pD->Data[pD->iDSP-1].fpVal += pD->Data[pD->iDSP].fpVal;
			 return (SetStack(pD,1,ST_REAL,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  case ST_STRING:
       switch (pD->Data[pD->iDSP].Type)
	   {
		case ST_STRING:
			 strcat(pD->Data[pD->iDSP-1].cVal,pD->Data[pD->iDSP].cVal);
			 return (SetStack(pD,1,ST_STRING,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  default:break;
 };
 Report(EVENTLOG_ERROR_TYPE,282,"+ failed on bad or incompatible arguments",NULL,pcHTM,pcERR,iCnt);
 return (FALSE);
};

BOOL OpSUB(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 switch (pD->Data[pD->iDSP-1].Type)
 {
  case ST_INT:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
		     pD->Data[pD->iDSP-1].dwVal -= pD->Data[pD->iDSP].dwVal;
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		case ST_REAL:
			 pD->Data[pD->iDSP-1].fpVal=(long)(pD->Data[pD->iDSP-1].dwVal)-pD->Data[pD->iDSP].fpVal;
			 return (SetStack(pD,1,ST_REAL,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  case ST_REAL:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
			 pD->Data[pD->iDSP-1].fpVal=pD->Data[pD->iDSP-1].fpVal-(long)(pD->Data[pD->iDSP].dwVal);
			 return (SetStack(pD,1,ST_REAL,pcHTM,pcERR,iCnt));
		case ST_REAL:
			 pD->Data[pD->iDSP-1].fpVal -= pD->Data[pD->iDSP].fpVal;
			 return (SetStack(pD,1,ST_REAL,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  default:break;
 };
 Report(EVENTLOG_ERROR_TYPE,282,"- failed on bad or incompatible arguments",NULL,pcHTM,pcERR,iCnt);
 return (FALSE);
};

BOOL OpEQ(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 switch (pD->Data[pD->iDSP-1].Type)
 {
  case ST_INT:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
			 pD->Data[pD->iDSP-1].dwVal=(pD->Data[pD->iDSP-1].dwVal == pD->Data[pD->iDSP].dwVal);
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		case ST_REAL:
			 pD->Data[pD->iDSP-1].dwVal=(long)(pD->Data[pD->iDSP-1].dwVal) == pD->Data[pD->iDSP].fpVal;
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  case ST_REAL:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
			 pD->Data[pD->iDSP-1].dwVal=pD->Data[pD->iDSP-1].fpVal == (long)(pD->Data[pD->iDSP].dwVal);
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		case ST_REAL:
			 pD->Data[pD->iDSP-1].dwVal=(pD->Data[pD->iDSP-1].fpVal == pD->Data[pD->iDSP].fpVal);
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  case ST_STRING:
       switch (pD->Data[pD->iDSP].Type)
	   {
		case ST_STRING:
			 pD->Data[pD->iDSP-1].dwVal=(strcmp(pD->Data[pD->iDSP-1].cVal,pD->Data[pD->iDSP].cVal)==0);
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  default:break;
 };
 Report(EVENTLOG_ERROR_TYPE,282,"== failed on bad or incompatible arguments",NULL,pcHTM,pcERR,iCnt);
 return (FALSE);
};

BOOL OpNE(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 if (OpEQ(pD,iOCnt,pcHTM,pcERR,iCnt))
 {
  pD->Data[pD->iDSP].dwVal=(pD->Data[pD->iDSP].dwVal ? FALSE : TRUE);
  return (TRUE); 
 };
 return (FALSE);
};

BOOL OpLT(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 switch (pD->Data[pD->iDSP-1].Type)
 {
  case ST_INT:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
			 pD->Data[pD->iDSP-1].dwVal=(int) pD->Data[pD->iDSP-1].dwVal < (int) pD->Data[pD->iDSP].dwVal;
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		case ST_REAL:
			 pD->Data[pD->iDSP-1].dwVal=(long)pD->Data[pD->iDSP-1].dwVal < pD->Data[pD->iDSP].fpVal;
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  case ST_REAL:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
			 pD->Data[pD->iDSP-1].dwVal=(pD->Data[pD->iDSP-1].fpVal < (long)pD->Data[pD->iDSP].dwVal);
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		case ST_REAL:
			 pD->Data[pD->iDSP-1].dwVal=(pD->Data[pD->iDSP-1].fpVal < pD->Data[pD->iDSP].fpVal);
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  case ST_STRING:
       switch (pD->Data[pD->iDSP].Type)
	   {
		case ST_STRING:
			 pD->Data[pD->iDSP-1].dwVal=(strcmp(pD->Data[pD->iDSP-1].cVal,pD->Data[pD->iDSP].cVal)<0);
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  default:break;
 };
 Report(EVENTLOG_ERROR_TYPE,282,"< failed on bad or incompatible arguments",NULL,pcHTM,pcERR,iCnt);
 return (FALSE);
};

BOOL OpGE(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 if (OpLT(pD,iOCnt,pcHTM,pcERR,iCnt))
 {
  pD->Data[pD->iDSP].dwVal=(pD->Data[pD->iDSP].dwVal ? FALSE : TRUE);
  return (TRUE); 
 };
 return (FALSE);
};

BOOL OpGT(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 switch (pD->Data[pD->iDSP-1].Type)
 {
  case ST_INT:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
			 pD->Data[pD->iDSP-1].dwVal=(int) pD->Data[pD->iDSP-1].dwVal > (int) pD->Data[pD->iDSP].dwVal;
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		case ST_REAL:
			 pD->Data[pD->iDSP-1].dwVal=(long)pD->Data[pD->iDSP-1].dwVal > pD->Data[pD->iDSP].fpVal;
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  case ST_REAL:
       switch (pD->Data[pD->iDSP].Type)
	   {
	    case ST_INT:
			 pD->Data[pD->iDSP-1].dwVal=(pD->Data[pD->iDSP-1].fpVal > (long)pD->Data[pD->iDSP].dwVal);
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		case ST_REAL:
			 pD->Data[pD->iDSP-1].dwVal=(pD->Data[pD->iDSP-1].fpVal > pD->Data[pD->iDSP].fpVal);
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  case ST_STRING:
       switch (pD->Data[pD->iDSP].Type)
	   {
		case ST_STRING:
			 pD->Data[pD->iDSP-1].dwVal=(strcmp(pD->Data[pD->iDSP-1].cVal,pD->Data[pD->iDSP].cVal)>0);
			 return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
		default:break;
	   };break;
  default:break;
 };
 Report(EVENTLOG_ERROR_TYPE,282,"> failed on bad or incompatible arguments",NULL,pcHTM,pcERR,iCnt);
 return (FALSE);
};

BOOL OpLE(PDATASTACK pD,int iOCnt,char *pcHTM,char *pcERR,int iCnt)
{
 if (OpGT(pD,iOCnt,pcHTM,pcERR,iCnt))
 {
  pD->Data[pD->iDSP].dwVal=(pD->Data[pD->iDSP].dwVal ? FALSE : TRUE);
  return (TRUE); 
 };
 return (FALSE);
};

///////////// F U N C T I O N S /////////////
void SetVarViaPointer(PSTVARIABLE pVar,StackType VType,DWORD dwV,double fpV,char *pcV)
{
 pVar->V.Type=VType;
 pVar->V.dwVal=dwV;
 pVar->V.fpVal=fpV;
 memset(pVar->V.cVal,'\0',MAX_STACKSTRING);
 if (pcV) strcpy(pVar->V.cVal,pcV); 
};

BOOL FunValType(PDATASTACK pD,int iPCnt,char *pcHTM,char *pcERR,int iCnt)
{
 switch(pD->Data[pD->iDSP].Type)
 {
  case ST_INT:   pD->Data[pD->iDSP].dwVal=1;break;
  case ST_REAL:  pD->Data[pD->iDSP].dwVal=2;break;
  case ST_STRING:pD->Data[pD->iDSP].dwVal=3;break;
  case ST_VARPTR:pD->Data[pD->iDSP].dwVal=4;break;
  default:       pD->Data[pD->iDSP].dwVal=0;break;
 };
 return (SetStack(pD,0,ST_INT,pcHTM,pcERR,iCnt));
};

BOOL FunStrLen(PDATASTACK pD,int iPCnt,char *pcHTM,char *pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_STRING,"STRLEN() arg must be string",pcHTM,pcERR,iCnt))
 {
  pD->Data[pD->iDSP].dwVal=strlen(pD->Data[pD->iDSP].cVal);
  return (SetStack(pD,0,ST_INT,pcHTM,pcERR,iCnt));
 };
 return (FALSE);
};

BOOL FunSubStr(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-2],ST_STRING,"STRLEN() arg 1 must be string",pcHTM,pcERR,iCnt))
 {
  char *pcDst=pD->Data[pD->iDSP-2].cVal;
  if (AssertType(pD->Data[pD->iDSP-1],ST_INT,"STRLEN() arg 2 must be integer",pcHTM,pcERR,iCnt))
  {
   char cSrc[MAX_STACKSTRING];
   char *pcSrc=strnset(cSrc,'\0',MAX_STACKSTRING);
   strcpy(pcSrc,pD->Data[pD->iDSP-2].cVal);
   strnset(pcDst,'\0',MAX_STACKSTRING);
   pcSrc+=pD->Data[pD->iDSP-1].dwVal;
   if (AssertType(pD->Data[pD->iDSP],ST_INT,"SUBSTR() arg 3 must be integer",pcHTM,pcERR,iCnt))
   {
    int iCnt=pD->Data[pD->iDSP].dwVal;
    while (*pcSrc && iCnt)
    {
     *pcDst++=*pcSrc++;
	 iCnt--;
	};
	return (SetStack(pD,2,ST_STRING,pcHTM,pcERR,iCnt));
   };
  };
 };
 return (FALSE);
};

BOOL FunLTrim(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_STRING,"LTRIM() arg must be string",pcHTM,pcERR,iCnt))
 {
  if (pD->Data[pD->iDSP].cVal[0])
  {
   char cTmp[MAX_STACKSTRING];
   char *pcS=strcpy(cTmp,pD->Data[pD->iDSP].cVal);
   while (*pcS && (*pcS<'!')) pcS++; // skip all control chars and spaces
   strcpy(pD->Data[pD->iDSP].cVal,pcS);
  };
  return (TRUE);
 };
 return (FALSE);
};

BOOL FunRTrim(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_STRING,"RTRIM() arg must be string",pcHTM,pcERR,iCnt))
 {
  if (pD->Data[pD->iDSP].cVal[0])
  {
   char *pcS=pD->Data[pD->iDSP].cVal;
   char *pcD=pcS+strlen(pD->Data[pD->iDSP].cVal);
   while ((*pcD<'!') && (pcD>=pcS)) *pcD--='\0';
  };
  return (TRUE);
 };
 return (FALSE);
};

BOOL FunStrUpr(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_STRING,"STRUPR() arg must be string",pcHTM,pcERR,iCnt))
 {
  strupr(pD->Data[pD->iDSP].cVal);
  return (TRUE);
 };
 return (FALSE);
};

BOOL FunStrPos(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-2],ST_STRING,"STRPOS() arg 1 must be string",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-1],ST_STRING,"STRPOS() arg 2 must be string",pcHTM,pcERR,iCnt))
  {
   if (AssertType(pD->Data[pD->iDSP],ST_INT,"STRPOS() arg 3 must be integer",pcHTM,pcERR,iCnt))
   {
    DWORD dwStart=pD->Data[pD->iDSP].dwVal;
    char *pcPos,*pcStart=pD->Data[pD->iDSP-2].cVal;
	if ((dwStart > strlen(pcStart)) || (pD->Data[pD->iDSP-1].cVal[0]=='\0')) pD->Data[pD->iDSP-2].dwVal=(DWORD) -1;
	else
	{
     if (pcPos=strstr(pcStart+dwStart,pD->Data[pD->iDSP-1].cVal)) pD->Data[pD->iDSP-2].dwVal=(DWORD) (pcPos-pcStart);
     else                                                         pD->Data[pD->iDSP-2].dwVal=(DWORD) -1;
	};
    return (SetStack(pD,2,ST_INT,pcHTM,pcERR,iCnt));
   };
  };
 };
 return (FALSE);
};

BOOL FunStrCpy(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-2],ST_STRING,"STRCPY() arg 1 must be string",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-1],ST_STRING,"STRCPY() arg 2 must be string",pcHTM,pcERR,iCnt))
  {
   if (AssertType(pD->Data[pD->iDSP],ST_INT,"STRCPY() arg 3 must be integer",pcHTM,pcERR,iCnt))
   {
    DWORD dwOffset=pD->Data[pD->iDSP].dwVal;
    char *pcDest=pD->Data[pD->iDSP-2].cVal;
	if (dwOffset<strlen(pcDest))
	{
	 pcDest+=dwOffset;
	 int iSrcLen;
	 if (iSrcLen=strlen(pD->Data[pD->iDSP-1].cVal))
	 {
	  strncpy(pcDest,pD->Data[pD->iDSP-1].cVal,min(iSrcLen,MAX_STACKSTRING - (int) dwOffset));
	 }
	 else *pcDest='\0';
	};
    return (SetStack(pD,2,ST_STRING,pcHTM,pcERR,iCnt));
   };
  };
 };
 return (FALSE);
};

BOOL GetAddrMask(PDATASTACK pD,DWORD *pdwMask,char* pcHTM,char* pcERR,int iCnt)
{
 switch (pD->Data[pD->iDSP].Type)
 {
  case ST_INT:
       *pdwMask=pD->Data[pD->iDSP].dwVal;
       return (TRUE);
  case ST_STRING:
       *pdwMask=inet_addr(pD->Data[pD->iDSP].cVal);
       return (TRUE);
  default:
       Report(EVENTLOG_WARNING_TYPE,158,"Address mask must be int or string. Defaulted to 255.255.255.255",NULL,pcHTM,pcERR,iCnt);
	   *pdwMask=0xffffffff;
	   return (TRUE); 
 };
 return (FALSE);
};

BOOL FunAddr2S(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-1],ST_INT,"ADDR2S arg 1 must be integer",pcHTM,pcERR,iCnt))
 {
  DWORD dwMask;
  if (GetAddrMask(pD,&dwMask,pcHTM,pcERR,iCnt))
  {
   char *pcDotted;
   in_addr IA;
   pD->Data[pD->iDSP-1].dwVal=pD->Data[pD->iDSP-1].dwVal & dwMask;
   memcpy(&IA,&(pD->Data[pD->iDSP-1].dwVal),4);
   if (pcDotted=inet_ntoa(IA)) strcpy(pD->Data[pD->iDSP-1].cVal,pcDotted);
   else                        strcpy(pD->Data[pD->iDSP-1].cVal,"0.0.0.0");
   return (SetStack(pD,1,ST_STRING,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};

BOOL FunS2Addr(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-1],ST_STRING,"S2ADDR arg 1 must be string",pcHTM,pcERR,iCnt))
 {
  DWORD dwMask;
  if (GetAddrMask(pD,&dwMask,pcHTM,pcERR,iCnt))
  {
   pD->Data[pD->iDSP-1].dwVal=inet_addr(pD->Data[pD->iDSP-1].cVal) & dwMask;
   return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};

BOOL FunRnd(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_INT,"RND() arg 2 must be integer",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-1],ST_INT,"RND() arg 1 must be integer",pcHTM,pcERR,iCnt))
  {
   pD->Data[pD->iDSP-1].dwVal+=MulDiv(rand(),pD->Data[pD->iDSP].dwVal-pD->Data[pD->iDSP-1].dwVal,RAND_MAX);
   return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};

BOOL FunGraphCnt(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_INT,"GRAPHCNT() arg 2 must be integer",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-1],ST_INT,"GRAPHCNT() arg 1 must be integer",pcHTM,pcERR,iCnt))
  {
   char cININame[256],cBuff[256],cDigitImg[10][128];
   strcpy(cININame,pcHTM);strcpy(strrchr(cININame,'.'),".ini");
   int iDigitCnt=pD->Data[pD->iDSP].dwVal;
   if (iDigitCnt<1) iDigitCnt=GetPrivateProfileInt("Log","iDigitCnt",(int) dwRegDigitCnt,cININame);
   strcpy(cBuff,"cX");           // Keys:     c0=... , c1=... , ... , c9=...
   for(int j=0;j<10;j++)
   {
    cBuff[1]=(char) 48+j;
    GetPrivateProfileString(KEY_DIGITS,cBuff,cRegDigitImg[j],cDigitImg[j],sizeof(cDigitImg[0]),cININame);
    if (strlen(cDigitImg[j])==0) strcpy(cDigitImg[j],pcDefaultDigitImg[j]);
   };
   GraphCnt(pD->Data[pD->iDSP-1].dwVal,iDigitCnt,&cDigitImg[0][0],pD->Data[pD->iDSP-1].cVal);
   return (SetStack(pD,1,ST_STRING,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};

void DecodeDateOrTime(char *pcDecoded,WORD wDosDate,WORD wDosTime,char *pcFormat,char *pcHTM,char *pcFormatEntry,char *pcDefaultFormat)
{
 tm DateTime;
 char cFormat[128];
 FILETIME   FT;
 SYSTEMTIME ST;
   
 if (*pcFormat) strcpy(cFormat,pcFormat);
 else	
 {
  char cININame[256];
  strcpy(cININame,pcHTM);strcpy(strrchr(cININame,'.'),".ini");
  GetPrivateProfileString("Log",pcFormatEntry,pcDefaultFormat,cFormat,sizeof(cFormat),cININame);
 };
 DosDateTimeToFileTime(wDosDate,wDosTime,&FT);
 FileTimeToSystemTime(&FT,&ST);
 SystemTime2TM(ST,&DateTime);
 strftime(pcDecoded,MAX_STACKSTRING,cFormat,&DateTime);
};

BOOL FunDecodeDate(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_STRING,"DECODEDATE() arg 2 must be string",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-1],ST_INT,"DECODEDATE() arg 1 must be integer",pcHTM,pcERR,iCnt))
  {
   DecodeDateOrTime(pD->Data[pD->iDSP-1].cVal,
                    LOWORD(pD->Data[pD->iDSP-1].dwVal),
                    HIWORD(pD->Data[pD->iDSP-1].dwVal),
                    pD->Data[pD->iDSP].cVal,
                    pcHTM,
                    "cDateFormat",
                    cRegDateFormat);
   return (SetStack(pD,1,ST_STRING,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};

BOOL FunDecodeTime(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_STRING,"DECODEDATETIME() arg 2 must be string",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-1],ST_INT,"DECODEDATETIME() arg 1 must be integer",pcHTM,pcERR,iCnt))
  {
   DecodeDateOrTime(pD->Data[pD->iDSP-1].cVal,
                    LOWORD(pD->Data[pD->iDSP-1].dwVal),
                    HIWORD(pD->Data[pD->iDSP-1].dwVal),
                    pD->Data[pD->iDSP].cVal,
                    pcHTM,
                    "cTimeFormat",
                    cRegTimeFormat);
   return (SetStack(pD,1,ST_STRING,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};


BOOL FunStr(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-1],ST_STRING,"STR() arg 1 must be string",pcHTM,pcERR,iCnt))
 {
  char cFormat[MAX_STACKSTRING];
  strcpy(cFormat,pD->Data[pD->iDSP-1].cVal);
  switch (pD->Data[pD->iDSP].Type)
  {
   case ST_INT:
   case ST_VARPTR:
        sprintf(pD->Data[pD->iDSP-1].cVal,cFormat,pD->Data[pD->iDSP].dwVal);
        break;
   case ST_STRING:
        sprintf(pD->Data[pD->iDSP-1].cVal,cFormat,pD->Data[pD->iDSP].cVal);
        break;
   case ST_REAL:
        sprintf(pD->Data[pD->iDSP-1].cVal,cFormat,pD->Data[pD->iDSP].fpVal);
        break;
   default:
        strcpy(pD->Data[pD->iDSP-1].cVal,"\0\0");
        Report(EVENTLOG_WARNING_TYPE,155,"STR() on invalid argument returned empty string",NULL,pcHTM,pcERR,iCnt);
		break;
  };
  return (SetStack(pD,1,ST_STRING,pcHTM,pcERR,iCnt));
 };
 return (FALSE);
};

BOOL FunReal(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 switch (pD->Data[pD->iDSP].Type)
 {
  case ST_REAL:
	   return (TRUE);
  case ST_STRING:
       pD->Data[pD->iDSP].fpVal=atof(pD->Data[pD->iDSP].cVal);
	   break;
  case ST_INT:	 
       pD->Data[pD->iDSP].fpVal=(long) pD->Data[pD->iDSP].dwVal;
	   break;
  default:
       Report(EVENTLOG_WARNING_TYPE,154,"REAL() on bad argument returned 0.",NULL,pcHTM,pcERR,iCnt);
	   pD->Data[pD->iDSP].fpVal=0;
	   break;
 };
 return (SetStack(pD,0,ST_REAL,pcHTM,pcERR,iCnt));
};

BOOL FunInt(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 switch (pD->Data[pD->iDSP].Type)
 {
  case ST_INT:
	   break;
  case ST_STRING:
       pD->Data[pD->iDSP].dwVal=strtoul(pD->Data[pD->iDSP].cVal,NULL,0);
	   break;
  case ST_REAL:	  // The most idiotic conversion I've ever written...
       sprintf(pD->Data[pD->iDSP].cVal,"%f",pD->Data[pD->iDSP].fpVal);
       pD->Data[pD->iDSP].dwVal=atoi(pD->Data[pD->iDSP].cVal);
	   break;
  default:
       Report(EVENTLOG_WARNING_TYPE,154,"INT() on bad argument returned 0",NULL,pcHTM,pcERR,iCnt);
	   pD->Data[pD->iDSP].dwVal=0;
	   break;
 };
 return (SetStack(pD,0,ST_INT,pcHTM,pcERR,iCnt));
};

BOOL FunSqrt(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 char *pcSQRT0="SQRT() on negative argument returned 0";

 switch (pD->Data[pD->iDSP].Type)
 {
  case ST_INT:
       pD->Data[pD->iDSP].Type=ST_REAL;
       if ((int) pD->Data[pD->iDSP].dwVal<0)
	   {
        Report(EVENTLOG_WARNING_TYPE,154,pcSQRT0,NULL,pcHTM,pcERR,iCnt);
	    pD->Data[pD->iDSP].fpVal=0;
	   }
	   else pD->Data[pD->iDSP].fpVal=sqrt((double) pD->Data[pD->iDSP].dwVal);
	   break;
  case ST_REAL:
       if (pD->Data[pD->iDSP].fpVal<0)
	   {
        Report(EVENTLOG_WARNING_TYPE,154,pcSQRT0,NULL,pcHTM,pcERR,iCnt);
	    pD->Data[pD->iDSP].fpVal=0;
	   }
	   else pD->Data[pD->iDSP].fpVal=sqrt(pD->Data[pD->iDSP].fpVal);
	   break;
  default:
       Report(EVENTLOG_WARNING_TYPE,154,pcSQRT0,NULL,pcHTM,pcERR,iCnt);
	   pD->Data[pD->iDSP]=v0;
	   break;
 };
 return (TRUE);
};

BOOL FunGetFileNames(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-3],ST_VARPTR  ,"GETFILENAMES() arg 1 must point to variable",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-2],ST_VARPTR ,"GETFILENAMES() arg 2 must point to variable",pcHTM,pcERR,iCnt))
  {
   if (AssertType(pD->Data[pD->iDSP-1],ST_VARPTR,"GETFILENAMES() arg 3 must point to variable",pcHTM,pcERR,iCnt))
   {
    if (AssertType(pD->Data[pD->iDSP],ST_VARPTR ,"GETFILENAMES() arg 4 must point to variable",pcHTM,pcERR,iCnt))
    {
	 char cLOG[256],cINI[256];
     strcpy(cLOG,pcHTM);strcpy(strrchr(cLOG,'.'),".log"); // Regenerate LOG filename!
     strcpy(cINI,pcHTM);strcpy(strrchr(cINI,'.'),".ini"); // Regenerate LOG filename!
	 SetVarViaPointer((PSTVARIABLE) pD->Data[pD->iDSP-3].dwVal,ST_STRING,0,0.,pcHTM);
	 SetVarViaPointer((PSTVARIABLE) pD->Data[pD->iDSP-2].dwVal,ST_STRING,0,0.,cINI);
	 SetVarViaPointer((PSTVARIABLE) pD->Data[pD->iDSP-1].dwVal,ST_STRING,0,0.,cLOG);
	 SetVarViaPointer((PSTVARIABLE) pD->Data[pD->iDSP-0].dwVal,ST_STRING,0,0.,pcERR);
     pD->Data[pD->iDSP-2].dwVal=(DWORD) TRUE;
     return (SetStack(pD,3,ST_INT,pcHTM,pcERR,iCnt));
	};
   };
  };
 };
 return (FALSE);
};

BOOL FunFindFirst(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-1],ST_STRING,"FINDFIRST() arg 1 must be string",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP],ST_VARPTR,"FINDFIRST() arg 2 must point to variable",pcHTM,pcERR,iCnt))
  {
   WIN32_FIND_DATA FD;
   HANDLE FFHandle=FindFirstFile(pD->Data[pD->iDSP-1].cVal,&FD);
   SetVarViaPointer((PSTVARIABLE) pD->Data[pD->iDSP].dwVal,ST_STRING,0,0.,FD.cFileName);
   pD->Data[pD->iDSP-1].dwVal=(FFHandle==INVALID_HANDLE_VALUE) ? 0 : (DWORD) FFHandle;
   return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};

BOOL FunFindNext(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-1],ST_INT,"FINDNEXT() arg 1 must be search handle",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP],ST_VARPTR,"FINDNEXT() arg 2 must point to variable",pcHTM,pcERR,iCnt))
  {
   WIN32_FIND_DATA FD;
   pD->Data[pD->iDSP-1].dwVal=FindNextFile((HANDLE) pD->Data[pD->iDSP-1].dwVal,&FD);
   SetVarViaPointer((PSTVARIABLE) pD->Data[pD->iDSP].dwVal,ST_STRING,0,0.,FD.cFileName);
   return (SetStack(pD,1,ST_INT,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};

BOOL FunFindClose(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_INT,"FINDCLOSE() arg 1 must be search handle",pcHTM,pcERR,iCnt))
 {
  pD->Data[pD->iDSP].dwVal=FindClose((HANDLE) pD->Data[pD->iDSP].dwVal);
  return (TRUE);
 };
 return (FALSE);
};					   

BOOL FunReadINIorReg(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType( pD->Data[pD->iDSP-3],ST_STRING,"READINIORREG() arg 2 must be string",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-2],ST_STRING,"READINIORREG() arg 3 must be string",pcHTM,pcERR,iCnt))
  {
   if (AssertType( pD->Data[pD->iDSP],ST_VARPTR,"READINIORREG() arg 5 must point to a variable",pcHTM,pcERR,iCnt))
   {
    PSTVARIABLE pVar=(PSTVARIABLE) pD->Data[pD->iDSP].dwVal;
	HKEY hKeyR;
	DWORD dwValType,dwBuffLen=MAX_STACKSTRING;
	char cReadBuff[MAX_STACKSTRING];
 	switch (pD->Data[pD->iDSP-4].Type)
	{
	 case ST_INT:
	      if (RegOpenKeyEx(RootKey[abs(pD->Data[pD->iDSP-4].dwVal) % 3],pD->Data[pD->iDSP-3].cVal,0,KEY_READ,&hKeyR)==ERROR_SUCCESS) 
		  {
		   if (RegQueryValueEx(hKeyR,pD->Data[pD->iDSP-2].cVal,NULL,&dwValType,(LPBYTE) cReadBuff,&dwBuffLen)==ERROR_SUCCESS)
		   {
		    if (dwValType==REG_DWORD)
			{
             pVar->V.Type=ST_INT;
             memcpy((void*) &(pVar->V.dwVal),(void*) cReadBuff,4);
             pVar->V.fpVal=0.;
		     *(pVar->V.cVal)='\0';
			}
			else SetVarViaPointer(pVar,ST_STRING,0,0.,cReadBuff);
		    pD->Data[pD->iDSP-4].dwVal=1; // OK, return TRUE
		   }
		   else
		   {
		    pVar->V=pD->Data[pD->iDSP-1]; // Set to default, whatever that might be
		    pD->Data[pD->iDSP-4].dwVal=0; // Failed, return FALSE
		   };
		   RegCloseKey(hKeyR);
           return (SetStack(pD,4,ST_INT,pcHTM,pcERR,iCnt));
  	      }
		  else 
		  {
		   pVar->V=pD->Data[pD->iDSP-1]; // Set to default, whatever that might be
		   pD->Data[pD->iDSP-4].dwVal=0; // Failed, return FALSE
           return (SetStack(pD,4,ST_INT,pcHTM,pcERR,iCnt));
		  };
		  break;
	 case ST_STRING:
          if (AssertType(pD->Data[pD->iDSP-1],ST_STRING,"READINIORREG() arg 4 should be string when reading from .INI file",pcHTM,pcERR,iCnt))
          {
	       pD->Data[pD->iDSP-4].dwVal=GetPrivateProfileString(pD->Data[pD->iDSP-3].cVal,
	                                                          pD->Data[pD->iDSP-2].cVal,
	                                                          pD->Data[pD->iDSP-1].cVal,
	                                                          cReadBuff,
	                                                          MAX_STACKSTRING,
	                                                          pD->Data[pD->iDSP-4].cVal);
		   SetVarViaPointer(pVar,ST_STRING,0,0.,cReadBuff);
           return (SetStack(pD,4,ST_INT,pcHTM,pcERR,iCnt));
		  };
		  break;
	 default:
          Report(EVENTLOG_ERROR_TYPE,284,"READINIORREG() arg 1 should be string or integer",NULL,pcHTM,pcERR,iCnt);
	};
   };
  };
 };
 return (FALSE);
};

BOOL FunWriteINIorReg(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType( pD->Data[pD->iDSP-2],ST_STRING,"WRITEINIORREG() arg 2 must be string",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-1],ST_STRING,"WRITEINIORREG() arg 3 must be string",pcHTM,pcERR,iCnt))
  {
   PSTVARIABLE pVar=(PSTVARIABLE) pD->Data[pD->iDSP].dwVal;
   HKEY hKeyW;
   DWORD dwValType,dwValLen;
   BYTE *pbWData;
   switch (pD->Data[pD->iDSP].Type)
   {
    case ST_INT:
         itoa(pD->Data[pD->iDSP].dwVal,pD->Data[pD->iDSP].cVal,10);
		 dwValType=REG_DWORD;
		 dwValLen=4;
		 pbWData=(BYTE*) &(pD->Data[pD->iDSP].dwVal);
  	     break;
    case ST_REAL:
         sprintf(pD->Data[pD->iDSP].cVal,"%20g",pD->Data[pD->iDSP].fpVal);
		 dwValType=REG_SZ;
		 dwValLen=strlen(pD->Data[pD->iDSP].cVal)+1;
		 pbWData=(BYTE*) pD->Data[pD->iDSP].cVal;
	     break;
    case ST_STRING:
		 dwValType=REG_SZ;
		 dwValLen=strlen(pD->Data[pD->iDSP].cVal)+1;
		 pbWData=(BYTE*) pD->Data[pD->iDSP].cVal;
         break;
    default:
         Report(EVENTLOG_ERROR_TYPE,285,"WRITEINIORREG() arg 4 should be string or numeric",NULL,pcHTM,pcERR,iCnt);
         return (FALSE);
   };
   switch (pD->Data[pD->iDSP-3].Type)
   {
    case ST_INT:
         if (RegOpenKeyEx(RootKey[abs(pD->Data[pD->iDSP-3].dwVal) % 3],pD->Data[pD->iDSP-2].cVal,0,KEY_WRITE,&hKeyW)==ERROR_SUCCESS) 
		 {
		  pD->Data[pD->iDSP-3].dwVal=(RegSetValueEx(hKeyW,pD->Data[pD->iDSP-1].cVal,0,dwValType,pbWData,dwValLen)==ERROR_SUCCESS);
		  RegCloseKey(hKeyW);
          return (SetStack(pD,3,ST_INT,pcHTM,pcERR,iCnt));
  	     }
		 else
		 {
          Report(EVENTLOG_ERROR_TYPE,286,"WRITEINIORREG() failed on '%s' key",pD->Data[pD->iDSP-2].cVal,pcHTM,pcERR,iCnt);
          return (FALSE);
		 };
		 break;
	case ST_STRING:
	     pD->Data[pD->iDSP-3].dwVal=WritePrivateProfileString(pD->Data[pD->iDSP-2].cVal,
	                                                          pD->Data[pD->iDSP-1].cVal,
	                                                          pD->Data[pD->iDSP  ].cVal,
	                                                          pD->Data[pD->iDSP-3].cVal);
         return (SetStack(pD,3,ST_INT,pcHTM,pcERR,iCnt));
		 break;
    default:
         Report(EVENTLOG_ERROR_TYPE,284,"WRITEINIORREG() arg 1 should be string or integer",NULL,pcHTM,pcERR,iCnt);
   };
  };
 };
 return (FALSE);
};


BOOL FunFileSize(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_STRING,"FILESIZE() arg must be string",pcHTM,pcERR,iCnt))
 {
  DWORD dwFA=GetFileAttributes(pD->Data[pD->iDSP].cVal);
  if (dwFA==0xFFFFFFFF) 
  {
   pD->Data[pD->iDSP].dwVal=0;            // INVALID FILE
   return (SetStack(pD,0,ST_INT,pcHTM,pcERR,iCnt));
  };
  if (dwFA & FILE_ATTRIBUTE_DIRECTORY)
  {
   pD->Data[pD->iDSP].dwVal=(DWORD) -1;   // DIRECTORY (FOLDER)
   return (SetStack(pD,0,ST_INT,pcHTM,pcERR,iCnt));
  };
// OFSTRUCT OFS;
// HFILE FHandle=OpenFile(pD->Data[pD->iDSP].cVal,&OFS,OF_READ | OF_SHARE_DENY_NONE);
  HANDLE FHandle=CreateFile(pD->Data[pD->iDSP].cVal,GENERIC_READ,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
  if (FHandle != INVALID_HANDLE_VALUE) 
  {
   BY_HANDLE_FILE_INFORMATION FInfo;
   GetFileInformationByHandle(FHandle,&FInfo);
   pD->Data[pD->iDSP].dwVal=FInfo.nFileSizeLow;
   CloseHandle(FHandle);
  }
  else pD->Data[pD->iDSP].dwVal=0;        // INVALID FILE 
  return (SetStack(pD,0,ST_INT,pcHTM,pcERR,iCnt));
 };
 return (FALSE);
};

BOOL FunIIF(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 pD->Data[pD->iDSP-2]=(IsTrue(pD->Data[pD->iDSP-2],pcHTM,pcERR,iCnt) ? pD->Data[pD->iDSP-1] : pD->Data[pD->iDSP]);
 return (SetStack(pD,2,pD->Data[pD->iDSP-2].Type,pcHTM,pcERR,iCnt));
};

// --------------------------------------------------------------------
// Note: these 2 functions are exported via .def to be used by debugger!
// (sometime later, not right now...)
DWORD WINAPI DecodeFormFieldData(char *pcClear,char *pcEncoded,DWORD dwMaxLen)
{
 char cHi,cLo,*pcDst=pcClear,*pcSrc=pcEncoded;

 *pcDst='\0'; 
 while ((*pcSrc) && ((DWORD)(pcSrc-pcEncoded)<dwMaxLen))
 {
  switch (*pcSrc)
  {
   case '+':
	    pcSrc++;
	    *pcDst++=' ';
        break;
   case '%':
        if (isxdigit(cHi=toupper(*(pcSrc+1))))
        {
         if (isxdigit(cLo=toupper(*(pcSrc+2))))
         {
          *pcDst++=(char) (16*(cHi>'9' ? cHi-'7' : cHi-'0')+(cLo>'9' ? cLo-'7' : cLo-'0'));
          pcSrc+=3;
         }
         else *pcDst++=*pcSrc++; 
        }
        else *pcDst++=*pcSrc++; 
        break;
   default:
        *pcDst++=*pcSrc++;
		break;
  }; // switch
  *pcDst='\0';
 };  // while
 return 0;
};

DWORD WINAPI EncodeFormFieldData(char *pcEncoded,char *pcClear,DWORD dwMaxLen)
{
 char HexChar[]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
 char *pcDst=pcEncoded,*pcSrc=pcClear;

 *pcDst='\0';
 while ((*pcSrc) && ((DWORD)(pcSrc-pcClear)<dwMaxLen))
 {
  if (isalnum(*pcSrc)) *pcDst++=*pcSrc++;
  else
  {
   switch (*pcSrc)
   {
    case ' ':
         *pcDst++='+';
	     break;
    case '@':
    case '_':
    case '.':
    case '*':
    case '-':
    	  *pcDst++=*pcSrc;
		  break;
	default:
	     *pcDst++='%';
	     *pcDst++=HexChar[(((int) *pcSrc) & 0xF0) >> 4];
	     *pcDst++=HexChar[((int) *pcSrc) & 0x0F];
	     break;
   };
   pcSrc++;
  }; // not alphanumeric!
  *pcDst='\0';
 };  // while
 return 0;
};
//
//
//----------------------------------------------------------------------
	
BOOL FunDecodeFormFieldData(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_STRING,"DECODEFORMFIELDDATA() arg must be string",pcHTM,pcERR,iCnt))
 {
  char cDecoded[MAX_STACKSTRING];
  DecodeFormFieldData(cDecoded,pD->Data[pD->iDSP].cVal,MAX_STACKSTRING);
  strcpy(pD->Data[pD->iDSP].cVal,cDecoded);
  return (TRUE);
 };
 return (FALSE);
};

BOOL FunEncodeFormFieldData(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_STRING,"ENCODEFORMFIELDDATA() arg must be string",pcHTM,pcERR,iCnt))
 {
  char cEncoded[MAX_STACKSTRING];
  EncodeFormFieldData(cEncoded,pD->Data[pD->iDSP].cVal,MAX_STACKSTRING);
  strcpy(pD->Data[pD->iDSP].cVal,cEncoded);
  return (TRUE);
 };
 return (FALSE);
};

char* stristr(char *pcWhere,char *pcWhat)
{
 int iLenWhat=strlen(pcWhat);
 int iSlides=strlen(pcWhere)-iLenWhat;

 for (int i=0;i<=iSlides;i++) if (strnicmp(pcWhere+i,pcWhat,iLenWhat)==0) return (pcWhere+i);
 return (NULL);
};

char* FindURLStartPos(char *pcFromPos)
{
 char* URLStart[]={"file://","http://","gopher://","ftp://","mailto:","telnet:","https://","news:"};
 char* pcPos=NULL,*pcFirstURLPos=NULL;
 for(int i=0;i<8;i++)
 {
  if (pcPos=stristr(pcFromPos,URLStart[i]))
  {
   if (pcFirstURLPos) 
   {
    if ((DWORD) pcPos<(DWORD) pcFirstURLPos) pcFirstURLPos=pcPos;
   }
   else pcFirstURLPos=pcPos;
  };
 }
 return(pcFirstURLPos);
};

char* FindURLEndPos(char *pcFromPos)
{
 char *pcPos=pcFromPos;
 while (isalnum(*pcPos) || strchr(".:_@/-\\?|&=",*pcPos)) pcPos++;
 return(pcPos);
};

BOOL FunCreateAnchors(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP],ST_STRING,"CREATEANCHORS() arg must be string",pcHTM,pcERR,iCnt))
 {
  char cOut[MAX_STACKSTRING],cHotSpot[MAX_STACKSTRING];
  char *pcOut=cOut          ,*pcHotSpot=cHotSpot;
  char *pcSearchStart=pD->Data[pD->iDSP].cVal;
  char *pcURLStart,*pcURLEnd,*pcEnd=strchr(pcSearchStart,'\0');

  memset(pcOut,'\0',sizeof(cOut));
//  memset(pcHotSpot,'\0',sizeof(cHotSpot));
  if (pcURLStart=FindURLStartPos(pcSearchStart))
  {
   BOOL bContinue=TRUE;
   while (bContinue)
   {
    pcURLEnd=FindURLEndPos(pcURLStart);
    strncat(pcOut,pcSearchStart,pcURLStart-pcSearchStart); // Stuff before URL
    strcat(pcOut,"<a href=\042");                          // Anchor start
    strncat(pcOut,pcURLStart,pcURLEnd-pcURLStart);         // Anchor tag
    strcat(pcOut,"\042>");                                 // Close the tag
    memset(cHotSpot,'\0',sizeof(cHotSpot));
    strncpy(cHotSpot,pcURLStart,pcURLEnd-pcURLStart);      // Hot spot=URL for starters
    if (pcHotSpot=strchr(cHotSpot,':'))                    // Paranoia! Colon always there!?
    {
     pcHotSpot++;                                          // Skip the colon 
     while (*pcHotSpot=='/') pcHotSpot++;
     strcat(pcOut,pcHotSpot);
    }
    else strcat(pcOut,cHotSpot);
    strcat(pcOut,"</a>");	                               // Anchor end
    if (pcURLEnd>=pcEnd) bContinue=FALSE;
    else 
    {
     pcSearchStart=pcURLEnd;
	 pcURLStart=FindURLStartPos(pcSearchStart);
	 if (!pcURLStart)
	 {
	  bContinue=FALSE;
	  strcat(pcOut,pcSearchStart);
	 }
    } // while
   }; // if
   strcpy(pD->Data[pD->iDSP].cVal,cOut);
  };
  return (TRUE);
 };
 return (FALSE);
};

BOOL FunGetFileInfo(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-5],ST_STRING,"GETFILEINFO() arg 1 must be string",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-4],ST_STRING,"GETFILEINFO() arg 2 must be string",pcHTM,pcERR,iCnt))
  {
   for (int i=0;i<4;i++) if (!AssertType(pD->Data[pD->iDSP-i],ST_VARPTR,"GETFILEINFO() args 3,4,5 must point to variable",pcHTM,pcERR,iCnt)) return (FALSE);
   char cFileName[256];
   if (pD->Data[pD->iDSP-5].cVal[0])
   {
    strcpy(cFileName,pD->Data[pD->iDSP-5].cVal);
    if (pD->Data[pD->iDSP-4].cVal[0])
    {
     if ((cFileName[strlen(cFileName)-1] != '\\') && (pD->Data[pD->iDSP-4].cVal[0] != '\\')) strcat(cFileName,"\\");
     strcat(cFileName,pD->Data[pD->iDSP-4].cVal);
    }
   } else strcpy(cFileName,pD->Data[pD->iDSP-4].cVal);
   DWORD dwFA=GetFileAttributes(cFileName);
   if (dwFA==0xFFFFFFFF) pD->Data[pD->iDSP-5].dwVal=FALSE; // Invalid file name or file not found
   else 
   {
    HANDLE FHandle=CreateFile(cFileName,0,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,FILE_FLAG_BACKUP_SEMANTICS,NULL);
    if (FHandle == INVALID_HANDLE_VALUE) pD->Data[pD->iDSP-5].dwVal=FALSE; // Invalid file name or file not found
    else
    {
     BY_HANDLE_FILE_INFORMATION FInfo;
     WORD                       wDosTime,wDosDate;
     FILETIME                   FTLocal;
     GetFileInformationByHandle(FHandle,&FInfo);
     CloseHandle(FHandle);
     FileTimeToLocalFileTime(&(FInfo.ftLastWriteTime),&FTLocal);
     FileTimeToDosDateTime(&FTLocal,&wDosTime,&wDosDate);
     SetVarViaPointer((PSTVARIABLE) pD->Data[pD->iDSP  ].dwVal,ST_INT,MAKELONG(wDosTime,wDosDate),0.,NULL);
     SetVarViaPointer((PSTVARIABLE) pD->Data[pD->iDSP-1].dwVal,ST_INT,FInfo.nFileSizeLow,0.,NULL);
     SetVarViaPointer((PSTVARIABLE) pD->Data[pD->iDSP-2].dwVal,ST_INT,dwFA,0.,NULL);
     SetVarViaPointer((PSTVARIABLE) pD->Data[pD->iDSP-3].dwVal,ST_STRING,0,0.,NULL);
     pD->Data[pD->iDSP-5].dwVal=TRUE;
    };
   };
   return (SetStack(pD,5,ST_INT,pcHTM,pcERR,iCnt));
  };
 };
 return (FALSE);
};

BOOL FunStrSplit(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-3],ST_STRING,"STRSPLIT() arg 1 must be string",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-2],ST_STRING,"STRSPLIT() arg 2 must be string",pcHTM,pcERR,iCnt))
  {
   if (AssertType(pD->Data[pD->iDSP-1],ST_VARPTR,"STRSPLIT() arg 3 must point to variable",pcHTM,pcERR,iCnt))
   {
    if (AssertType(pD->Data[pD->iDSP],ST_VARPTR,"STRSPLIT() arg 4 must point to variable",pcHTM,pcERR,iCnt))
    {
	 char *pcSplitPos;
	 PSTVARIABLE     pBefore=(PSTVARIABLE) pD->Data[pD->iDSP-1].dwVal;
     PSTVARIABLE     pAfter =(PSTVARIABLE) pD->Data[pD->iDSP  ].dwVal;
	 if (pcSplitPos=strstr(pD->Data[pD->iDSP-3].cVal,pD->Data[pD->iDSP-2].cVal))
	 {
	  *pcSplitPos='\0'; // cut it in half there!
	  SetVarViaPointer(pBefore,ST_STRING,0,0.,pD->Data[pD->iDSP-3].cVal);
	  SetVarViaPointer(pAfter,ST_STRING,0,0.,pcSplitPos+strlen(pD->Data[pD->iDSP-2].cVal));
	  pD->Data[pD->iDSP-3].dwVal=TRUE;
	 }
	 else
	 {
	  SetVarViaPointer(pBefore,ST_STRING,0,0.,pD->Data[pD->iDSP-3].cVal);
	  SetVarViaPointer(pAfter,ST_STRING,0,0.,NULL);
	  pD->Data[pD->iDSP-3].dwVal=FALSE;
	 };
     return (SetStack(pD,3,ST_INT,pcHTM,pcERR,iCnt));
	};
   };
  };
 };
 return (FALSE);
};

#ifdef THUMBNAIL
BOOL TCache(char *pcHREF,char *pcTCacheName,POFSTRUCT pOFS)
{
 char *pcLastDot=strrchr(pcHREF,'.');
 if (pcLastDot)
 {
  pcLastDot++;
  sprintf(pcTCacheName,"%s\\%s%s.DAT",cRegTempDir,"Cache",strupr(pcLastDot));
  return (OpenFile(pcTCacheName,pOFS,OF_EXIST) != HFILE_ERROR);
 };
 return (FALSE);
};

BOOL QueryThumbnailCache(char *pcHREF,int iH,int iV,char *pcIMGSRC)
{
 OFSTRUCT OFS;
 BY_HANDLE_FILE_INFORMATION TCInfo;
 char     cTCacheName[256],cSearchEntry[512];
 char     *pcCache,*pcFoundEntry;
 BOOL     bFound=FALSE;
 
 if (TCache(pcHREF,cTCacheName,&OFS))
 {
  HFILE TCHandle=OpenFile(cTCacheName,&OFS,OF_READ | OF_SHARE_DENY_NONE);
  if (TCHandle != HFILE_ERROR)
  {
   GetFileInformationByHandle((HANDLE) TCHandle,&TCInfo);
   DWORD dwCacheSize=TCInfo.nFileSizeLow+16;
   if (pcCache=(char*) GlobalAlloc(GPTR,dwCacheSize))
   {
    DWORD dwFileRW;
    ReadFile((HANDLE) TCHandle,pcCache,dwCacheSize,&dwFileRW,NULL);
    int iSELen=sprintf(cSearchEntry,"%s,%04i,%04i,%s,",pcHREF,iH,iV);
    if (pcFoundEntry=strstr(pcCache,cSearchEntry))
    {
     pcFoundEntry+=iSELen; // ARE WE MISSING A CHAR HERE??
     *pcIMGSRC='\0';
     while (*pcFoundEntry>' ')
     {
      bFound=TRUE;// Will be true if at least one char is copied...
      *pcIMGSRC++=*pcFoundEntry++;
      *pcIMGSRC='\0';
     };
    };
    GlobalFree(pcCache);
   };
   CloseHandle((HANDLE) TCHandle); 
  };
 };
 return (bFound);
};

BOOL AppendThumbnailCache(char *pcHREF,int iH,int iV,char *pcIMGSRC)
{
 OFSTRUCT OFS;
 HFILE    TCHandle;
 DWORD    dwFileRW;
 char     cTCacheName[256],cNewEntry[512];
 
 if (TCache(pcHREF,cTCacheName,&OFS)) TCHandle=OpenFile(cTCacheName,&OFS,OF_WRITE | OF_SHARE_DENY_NONE);
 else
 {
  TCHandle=OpenFile(cTCacheName,&OFS,OF_CREATE | OF_SHARE_DENY_NONE); 
  SetFilePointer((HANDLE) TCHandle,0,NULL,FILE_END);
 };
 if (TCHandle != HFILE_ERROR)
 {
  WriteFile((HANDLE) TCHandle,cNewEntry,sprintf(cNewEntry,"%s,%04i,%04i,%s\r\n",pcHREF,iH,iV,pcIMGSRC),&dwFileRW,NULL);
  CloseHandle((HANDLE) TCHandle);
  return (TRUE);
 };
 return (FALSE);
};

BOOL EmptyThumbnail(PDATASTACK pD,char* pcHTM,char* pcERR,int iCnt)
{
 strcpy(pD->Data[pD->iDSP-5].cVal,pcEmpty);
 return (SetStack(pD,5,ST_STRING,pcHTM,pcERR,iCnt));
};

BOOL EvalThumbnailExtent(int iOldExtent,int iSel,int *piNewExtent,PDATASTACK pD,char* pcHTM,char* pcERR,int iCnt)
{
 *piNewExtent=0; // Error!
 switch (pD->Data[pD->iDSP-iSel].Type)
 {
  case ST_INT:
       *piNewExtent=(int) pD->Data[pD->iDSP-iSel].dwVal;
       break;
  case ST_REAL:
       *piNewExtent=(int) (pD->Data[pD->iDSP-iSel].fpVal*iOldExtent);
       break;
  default:
       Report(EVENTLOG_ERROR_TYPE,302,"THUMBNAIL() arg %s must be integer or real",iSel==4 ? "2" : "3",pcHTM,pcERR,iCnt);
       return (FALSE);
 };
 if (*piNewExtent<MIN_THUMBNAILEXTENT)
 {
  Report(EVENTLOG_ERROR_TYPE,303,"Invalid THUMBNAIL() %s size",iSel==4 ? "horizontal" : "vertical",pcHTM,pcERR,iCnt);
  return (FALSE);
 };
 return (TRUE);
};

BOOL FunThumbnail(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-5],ST_STRING,"THUMBNAIL() arg 1 must be string",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-2],ST_STRING,"THUMBNAIL() arg 4 must be string",pcHTM,pcERR,iCnt))
  {
   if (AssertType(pD->Data[pD->iDSP-1],ST_STRING,"THUMBNAIL() arg 5 must be string",pcHTM,pcERR,iCnt))
   {
    if (AssertType(pD->Data[pD->iDSP],ST_INT,"THUMBNAIL() arg 6 must be integer",pcHTM,pcERR,iCnt))
    {
     if (hInstImgDLL)   return (EmptyThumbnail(pD,pcHTM,pcERR,iCnt));
     if (hInstWinetDLL) return (EmptyThumbnail(pD,pcHTM,pcERR,iCnt));
     if (cRegThumbnailPath[0]=='\0')	return (EmptyThumbnail(pD,pcHTM,pcERR,iCnt));
     OFSTRUCT OFS;
     char cHREF[256],cIMGSRC[256],cALT[256],cHotSpot[256];

     strcpy(cHREF   ,pD->Data[pD->iDSP-5].cVal);
     strcpy(cIMGSRC ,pD->Data[pD->iDSP-5].cVal); // Under constr!
     strcpy(cALT    ,pD->Data[pD->iDSP-2].cVal);
     strcpy(cHotSpot,pD->Data[pD->iDSP-1].cVal);

     if (OpenFile(cHREF,&OFS,OF_EXIST) != HFILE_ERROR)
     {
//
// TODO!
//     
     };
     sprintf(pD->Data[pD->iDSP-5].cVal,"<a href=\042%s\042><img src=\042%s\042 border=%i alt=\042%s\042>%s</a>",cHREF,cIMGSRC,pD->Data[pD->iDSP].dwVal,cALT,cHotSpot);
     return (SetStack(pD,5,ST_STRING,pcHTM,pcERR,iCnt));
    };
   };
  };
 };
 return (FALSE);
};
#endif

BOOL FunCreateCookie(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 if (AssertType(pD->Data[pD->iDSP-4],ST_STRING,"CREATECOOKIE() arg 1 must be string",pcHTM,pcERR,iCnt))
 {
  if (AssertType(pD->Data[pD->iDSP-3],ST_STRING,"CREATECOOKIE() arg 2 must be string",pcHTM,pcERR,iCnt))
  {												 
   if (AssertType(pD->Data[pD->iDSP-2],ST_STRING,"CREATECOOKIE() arg 3 must be string",pcHTM,pcERR,iCnt))
   {
    if (AssertType(pD->Data[pD->iDSP-1],ST_STRING,"CREATECOOKIE() arg 4 must be string",pcHTM,pcERR,iCnt))
    {
     if (AssertType(pD->Data[pD->iDSP],ST_INT,"CREATECOOKIE() arg 5 must be integer",pcHTM,pcERR,iCnt))
     {
      char cCookie[MAX_HVALUE];
      sprintf(cCookie,"%s=%s",pD->Data[pD->iDSP-4].cVal,pD->Data[pD->iDSP-3].cVal);
      if (strlen(cCookie)>2) // Name and value present?
      {
       if (pD->Data[pD->iDSP-2].cVal[0])
       {
        char cDomain[128];
        sprintf(cDomain,"; domain=%s",pD->Data[pD->iDSP-2].cVal);
        strcat(cCookie,cDomain);
       };
       if (pD->Data[pD->iDSP-1].cVal[0])
       {
        char cPath[128];
        sprintf(cPath,"; path=%s",pD->Data[pD->iDSP-1].cVal);
        strcat(cCookie,cPath);
       };
       if (pD->Data[pD->iDSP].dwVal)
       {
////////////////////////////////////////////////////////////////
// PORTABILITY WARNING: SHOULD BE REWRITTEN FOR BIG-ENDIAN CPUs?
////////////////////////////////////////////////////////////////
// 64 bit integers are added here, where the time to which
//  is added is stored as two 32 bit words in a (FILETIME)
//                       structure.
//////////////////////////////////////////////////////////////// 
        char cExpires[128];
        SYSTEMTIME STNow,STExpires;
        __int64    FTNow,FTExpires;
        struct tm  CookieExpiryDateTime;
        __int64	   TimeOffset=int (pD->Data[pD->iDSP].dwVal);
	   
        GetSystemTime(&STNow);
        SystemTimeToFileTime(&STNow,(PFILETIME) &FTNow);
        TimeOffset*=10000000; // 100ns intervals in a second!
        FTExpires=FTNow+TimeOffset; 
        FileTimeToSystemTime((PFILETIME) &FTExpires,&STExpires);
        SystemTime2TM(STExpires,&CookieExpiryDateTime);
        strftime(cExpires,sizeof(cExpires),"; expires=%a, %d-%b-%y %H:%M:%S GMT",&CookieExpiryDateTime);
//      strftime(cExpires,sizeof(cExpires),"; expires=%A, %d-%b-%y %H:%M:%S GMT",&CookieExpiryDateTime);
        strcat(cCookie,cExpires);
       };
      }
      else cCookie[0]='\0';  // No name or no value, return empty string! 
      strcpy(pD->Data[pD->iDSP-4].cVal,cCookie); 
      return (SetStack(pD,4,ST_STRING,pcHTM,pcERR,iCnt));
     }
	};
   };
  };
 };
 return (FALSE);
};

char* FindSplitter(char *pcS,int iOccurence,char cSplitter)
{
 int i=1;
 char *pcAS=pcS;

 if (iOccurence>=1)
 {
  while ((i<=iOccurence) && pcAS)
  {
   pcAS=strchr(pcAS+1,cSplitter);
   i++;
  };
 };
 return (pcAS);
};

BOOL GetField(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt,char cSplitter,char* pcFunName)
{
 char cErrMsg[64];

 sprintf(cErrMsg,"%s arg 2 must be string",pcFunName);
 if (AssertType(pD->Data[pD->iDSP],ST_STRING,cErrMsg,pcHTM,pcERR,iCnt))
 {
  int iFieldIndex;
  char *pcD=pD->Data[pD->iDSP-1].cVal;
  char *pcS,*pcE;

  switch (pD->Data[pD->iDSP-1].Type)
  {
   case ST_INT:
        iFieldIndex=(int) pD->Data[pD->iDSP-1].dwVal; 
        if (iFieldIndex>0)
        {
	     pcS=FindSplitter(pD->Data[pD->iDSP].cVal,iFieldIndex-1,cSplitter);
	     pcE=FindSplitter(pD->Data[pD->iDSP].cVal,iFieldIndex,cSplitter);
		}
        else // iFieldIndex<=0
        {
         *pcD='\0';  
         return (SetStack(pD,1,ST_STRING,pcHTM,pcERR,iCnt));
        };
        break; 
   case ST_STRING:
        if (pcS=strstr(pD->Data[pD->iDSP].cVal,pcD))
        {
         pcE=strchr(pcS,cSplitter);
        }
        else pcE=NULL;  
        break;
   default:
        Report(EVENTLOG_ERROR_TYPE,301,"%s arg 1 must be integer or string",pcFunName,pcHTM,pcERR,iCnt);
        return (FALSE);
        break;
  }; // switch
  if (pcS)
  {
   if (*pcS==cSplitter) pcS++;
   if (pcE)
   {
    while (pcS<pcE)
    {
     *pcD++=*pcS++;
     *pcD='\0';
    };
   }
   else strcpy(pcD,pcS);
  }
  else *pcD='\0'; // pcS==NULL
  return (SetStack(pD,1,ST_STRING,pcHTM,pcERR,iCnt));
 };
 return (FALSE);
};

BOOL FunGetFormField(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 return (GetField(pD,iPCnt,pcHTM,pcERR,iCnt,'&',"GETFORMFIELD()"));
};

BOOL FunGetCookie(PDATASTACK pD,int iPCnt,char* pcHTM,char* pcERR,int iCnt)
{
 return (GetField(pD,iPCnt,pcHTM,pcERR,iCnt,';',"GETCOOKIE()"));
};

BOOL PushOper(enum OpType OType,int iPri,char *cName,int iPar,int(__cdecl *fpF)(struct stDataStack*,int,char*,char*,int),POPSTACK pOStack,PDATASTACK pDStack,char* pcHTM,char* pcERR,int iCnt)
{
 BOOL bNoErr=TRUE;
 int iOSP=pOStack->iOSP;

 iOSP++;
 pOStack->Op[iOSP].Type=OType;
 pOStack->Op[iOSP].iPri=iPri;
 strcpy(pOStack->Op[iOSP].cName,cName);
 pOStack->Op[iOSP].iPar=iPar;
 pOStack->Op[iOSP].fpF=fpF;
////////////////////////////////// B U G
 if (iOSP>1)
 {
  while (bNoErr && (iOSP>1) && (pOStack->Op[iOSP-1].iPri >= pOStack->Op[iOSP].iPri))
  {
   bNoErr=(pOStack->Op[iOSP-1].fpF)(pDStack,pOStack->Op[iOSP-1].iPar,pcHTM,pcERR,iCnt);
   pOStack->Op[iOSP-1]=pOStack->Op[iOSP];
   iOSP--;
  };
 };
////////////////////////////////// B U G
 pOStack->iOSP=iOSP;
 return (bNoErr);
};

BOOL PushData(enum StackType VType,DWORD dwV,double fV,char *cV,PDATASTACK pDStack,char* pcHTM,char* pcERR,int iCnt)
{
 int iSP=pDStack->iDSP;
 if (iSP>=MAX_DATASTACK) 
 { 
  Report(EVENTLOG_ERROR_TYPE,277,"Data stack overflow",NULL,pcHTM,pcERR,iCnt);
  return (FALSE);
 }
 else
 {
  iSP++;
  pDStack->Data[iSP].Type=VType;
  pDStack->Data[iSP].dwVal=dwV;
  pDStack->Data[iSP].fpVal=fV;
  strcpy(pDStack->Data[iSP].cVal,cV);
  pDStack->iDSP=iSP;
 };
 return (TRUE);
};

int FindFun(char *pcFunName,POPENTRY pFunData)
{
 int i=1;
 OPENTRY FunDef[]={{OP_EMPTY,0,"\0",0,&Dummy},
                   {OP_FUN,8,"STR"         ,2,&FunStr}, 
                   {OP_FUN,8,"IIF"         ,3,&FunIIF},
				   {OP_FUN,8,"STRSPLIT"    ,4,&FunStrSplit},
                   {OP_FUN,8,"GETFORMFIELD",2,&FunGetFormField},
                   {OP_FUN,8,"STRLEN"      ,1,&FunStrLen},
                   {OP_FUN,8,"SUBSTR"      ,3,&FunSubStr},
                   {OP_FUN,8,"STRUPR"      ,1,&FunStrUpr},
                   {OP_FUN,8,"LTRIM"       ,1,&FunLTrim},
                   {OP_FUN,8,"RTRIM"       ,1,&FunRTrim},
                   {OP_FUN,8,"STRPOS"      ,3,&FunStrPos},
                   {OP_FUN,8,"STRCPY"      ,3,&FunStrCpy},
                   {OP_FUN,8,"RND"         ,2,&FunRnd},
                   {OP_FUN,8,"INT"         ,1,&FunInt},
				   {OP_FUN,8,"GETFILENAMES",4,&FunGetFileNames},
                   {OP_FUN,8,"FINDFIRST"   ,2,&FunFindFirst}, 
                   {OP_FUN,8,"FINDNEXT"    ,2,&FunFindNext}, 
				   {OP_FUN,8,"GETFILEINFO" ,6,&FunGetFileInfo},
                   {OP_FUN,8,"DECODEDATE"  ,2,&FunDecodeDate}, 
                   {OP_FUN,8,"DECODETIME"  ,2,&FunDecodeTime}, 
                   {OP_FUN,8,"FINDCLOSE"   ,1,&FunFindClose}, 
				   {OP_FUN,8,"READINIORREG" ,5,&FunReadINIorReg},
				   {OP_FUN,8,"WRITEINIORREG",4,&FunWriteINIorReg},
                   {OP_FUN,8,"DECODEFORMFIELDDATA",1,&FunDecodeFormFieldData},
                   {OP_FUN,8,"ENCODEFORMFIELDDATA",1,&FunEncodeFormFieldData},
                   {OP_FUN,8,"CREATEANCHORS",1,&FunCreateAnchors},
                   {OP_FUN,8,"GRAPHCNT"    ,2,&FunGraphCnt},
                   {OP_FUN,8,"SQRT"        ,1,&FunSqrt},
                   {OP_FUN,8,"VALTYPE"     ,1,&FunValType},
                   {OP_FUN,8,"ADDR2S"      ,2,&FunAddr2S},
                   {OP_FUN,8,"S2ADDR"      ,2,&FunS2Addr},
                   {OP_FUN,8,"REAL"        ,1,&FunReal},
                   {OP_FUN,8,"CREATECOOKIE",5,&FunCreateCookie}, 
                   {OP_FUN,8,"GETCOOKIE"   ,2,&FunGetCookie},
#ifdef THUMBNAIL
                   {OP_FUN,8,"THUMBNAIL"   ,6,&FunThumbnail},
#endif
                   {OP_FUN,8,"FILESIZE"    ,1,&FunFileSize}, 
                   {OP_EMPTY,0,"\0",0,&Dummy}
                  };

 while (FunDef[i].cName[0])
 {
  if (stricmp(FunDef[i].cName,pcFunName)==0)
  {
   *pFunData=FunDef[i];
   return (i);
  }
  i++;
 };
 return 0; // Function not found!
};

//////////////////////////////////////////////////////////////////////////////////////
// Evaluate an expression, return the pointer to the fist char after the expression	//
//////////////////////////////////////////////////////////////////////////////////////
char* EvalEx(char *pcS,char *pcE,stExecConf ECD,PSTACKENTRY pResult,PVARBLOCK pVBlock,BOOL bFun,char* pcHTM,char* pcERR,int iCnt)
{
 char *pcP=pcS;
 BOOL bDataExpected=TRUE;
 BOOL bLoop=TRUE,bNoErr=TRUE;
 int iKlammer; // Number of pending open paretheses
 char *pcEndData;
 DWORD  dwConst;
 double fpConst;
 char   cConst[256];
 int    iVar;
 DATASTACK DStack;
 OPSTACK OStack;
 OPENTRY FunFound;

 iKlammer=bFun ? 1 : 0;
 DStack.iDSP=0; // Set stack pointers!
 OStack.iOSP=0;
 *pResult=vNothing; // Will return this if fails!
 while ((pcP<pcE) && bLoop && bNoErr && (strnicmp(pcP,ECD.cCmdEnd,ECD.dwLenCE) != 0))
 {
  switch (*pcP)
  {
   case ' ':
   case '\t':
   case '\n':
   case '\r':
        pcP++;
		break;
   case '"':
   case '\047':
        if (bDataExpected)
		{
         char cQuote=*pcP++;
		 if (pcEndData=strchr(pcP,cQuote))
		 {
		  if (pcEndData>pcE) 
		  {
		   pcEndData=pcE;
           Report(EVENTLOG_ERROR_TYPE,270,"Missing end quote in string constant",NULL,pcHTM,pcERR,iCnt);
		  };
		 }
		 else 
		 {
		  pcEndData=pcE;
          Report(EVENTLOG_ERROR_TYPE,270,"Missing end quote in string constant",NULL,pcHTM,pcERR,iCnt);
		 };
		 strncpy(cConst,pcP,pcEndData-pcP);
		 cConst[pcEndData-pcP]='\0';
		 pcP=pcEndData+1;
		 bNoErr=PushData(ST_STRING,0,0.,cConst,&DStack,pcHTM,pcERR,iCnt);
		 bDataExpected=FALSE;
		}
		else
		{
         Report(EVENTLOG_ERROR_TYPE,271,"String constant not expected",NULL,pcHTM,pcERR,iCnt);
		 return (pcP);
		};
		break;
   case '.':
        if (bDataExpected)
		{
         pcEndData=pcP;
         while (isdigit(*pcEndData) || (*pcEndData=='.')) pcEndData++; 
		 if (strchr("eEdD",*pcEndData)) // Exponent?
		 {
		  pcEndData++;
		  if (strchr("+-",*pcEndData))
		  {
		   pcEndData++;
           while (isdigit(*pcEndData)) pcEndData++; 
		  }
		  else while (isdigit(*pcEndData)) pcEndData++; 
		 };							   // Exponent
		 if (pcEndData>pcE) pcEndData=pcE;
		 strncpy(cConst,pcP,pcEndData-pcP+1);
		 cConst[pcEndData-pcP+1]='\0';
		 pcP=pcEndData;
		 fpConst=atof(cConst);
		 bNoErr=PushData(ST_REAL,0,fpConst,"",&DStack,pcHTM,pcERR,iCnt);
		 bDataExpected=FALSE;
		}
		else
		{
         Report(EVENTLOG_ERROR_TYPE,271,"Real constant not expected",NULL,pcHTM,pcERR,iCnt);
		 return (pcP);
		};
		break;
   case '=':
   case '>':
   case '<':
        if (bDataExpected)
		{
         Report(EVENTLOG_ERROR_TYPE,272,"Operator not expected",NULL,pcHTM,pcERR,iCnt);
		 return (pcP);
		}
		else
		{
		 char cFirst=*pcP++;
		 switch (*pcP)
		 {
		  case '=':
		       switch (cFirst)
			   {
			    case '=':bNoErr=PushOper(OP_BIN,10*iKlammer+2,"EQ",2,&OpEQ,&OStack,&DStack,pcHTM,pcERR,iCnt);bDataExpected=TRUE;pcP++;break;// == 
				case '>':bNoErr=PushOper(OP_BIN,10*iKlammer+2,"GE",2,&OpGE,&OStack,&DStack,pcHTM,pcERR,iCnt);bDataExpected=TRUE;pcP++;break;// >=
				case '<':bNoErr=PushOper(OP_BIN,10*iKlammer+2,"LE",2,&OpLE,&OStack,&DStack,pcHTM,pcERR,iCnt);bDataExpected=TRUE;pcP++;break;// <=
			   };break;
		  case '>':
		       switch (cFirst)
			   {
			    case '=':bNoErr=PushOper(OP_BIN,10*iKlammer+2,"GE" ,2,&OpGE, &OStack,&DStack,pcHTM,pcERR,iCnt);bDataExpected=TRUE;pcP++;break;// => 
				case '>':bNoErr=PushOper(OP_BIN,10*iKlammer+6,"SHR",2,&OpSHR,&OStack,&DStack,pcHTM,pcERR,iCnt);bDataExpected=TRUE;pcP++;break;// >>
				case '<':bNoErr=PushOper(OP_BIN,10*iKlammer+2,"NE" ,2,&OpNE, &OStack,&DStack,pcHTM,pcERR,iCnt);bDataExpected=TRUE;pcP++;break;// <>
			   };break;
		  case '<':
		       switch (cFirst)
			   {
			    case '=':bNoErr=PushOper(OP_BIN,10*iKlammer+2,"LE",2,&OpLE,&OStack,&DStack,pcHTM,pcERR,iCnt);bDataExpected=TRUE;pcP++;break;// =< 
				case '>':bNoErr=PushOper(OP_BIN,10*iKlammer+2,"NE",2,&OpNE,&OStack,&DStack,pcHTM,pcERR,iCnt);bDataExpected=TRUE;pcP++;break;// ><
				case '<':bNoErr=PushOper(OP_BIN,10*iKlammer+6,"SHL",2,&OpSHL,&OStack,&DStack,pcHTM,pcERR,iCnt);bDataExpected=TRUE;pcP++;break;// <<
			   };break;
		  default:
			   switch (cFirst)
			   {
			    case '>':bNoErr=PushOper(OP_BIN,10*iKlammer+2,"GT",2,&OpGT,&OStack,&DStack,pcHTM,pcERR,iCnt);bDataExpected=TRUE;break; // >
				case '<':bNoErr=PushOper(OP_BIN,10*iKlammer+2,"LT",2,&OpLT,&OStack,&DStack,pcHTM,pcERR,iCnt);bDataExpected=TRUE;break; // <
				default:
                     Report(EVENTLOG_ERROR_TYPE,273,"Invalid operator",NULL,pcHTM,pcERR,iCnt);
		             return (pcP);
					 break;
			   };break;
		 };
		};
		break;
   case '|':
        pcP++;
        if (bDataExpected)
		{
         Report(EVENTLOG_ERROR_TYPE,272,"Operator not expected",NULL,pcHTM,pcERR,iCnt);
		 return (pcP);
		}
		else
		{
		 bDataExpected=TRUE;
		 if (*pcP=='|') 
		 {
		  pcP++;
		  bNoErr=PushOper(OP_BIN,10*iKlammer+1,"LOGOR",2,&OpLOGOR,&OStack,&DStack,pcHTM,pcERR,iCnt);
		 }
		 else bNoErr=PushOper(OP_BIN,10*iKlammer+4,"BITOR",2,&OpBITOR,&OStack,&DStack,pcHTM,pcERR,iCnt);
		};
		break;
   case '&':
        pcP++;
        if (bDataExpected) // This could be a variable pointer!
		{
		 while (strchr(" \t\n\r",*pcP)) pcP++;
		 if (isalpha(*pcP))
		 {
  	      pcP=GetVarName(pcP,pcE,cConst,ECD,pVBlock,pcHTM,pcERR,iCnt);
   	      iVar=(FindVar(cConst,pVBlock,FALSE,pcHTM,pcERR,iCnt)); // Variable already defined?
   	      if (iVar==0)                                           // No, create it now!
		  {
		   if (AddVBlockEntry(NULL,cConst,vNothing,pVBlock,pcHTM,pcERR,iCnt)) iVar=pVBlock->iLast;
		  };
		  if (iVar)	// dwVal contains the pointer to variable
		  {			// cVal  contains the variable name without the preceeding &
		   bNoErr=PushData(ST_VARPTR,(DWORD) pVBlock->Var[iVar],0.,cConst,&DStack,pcHTM,pcERR,iCnt);
		   bDataExpected=FALSE;
		  }
		  else
		  {
           Report(EVENTLOG_ERROR_TYPE,283,"Unable to create pointer to variable '%s'",cConst,pcHTM,pcERR,iCnt);
		   return (pcP);
		  };
		 }
		 else
		 {
          Report(EVENTLOG_ERROR_TYPE,272,"Operator not expected",NULL,pcHTM,pcERR,iCnt);
		  return (pcP);
		 };
		}
		else
		{
		 bDataExpected=TRUE;
		 if (*pcP=='&') 
		 {
		  pcP++;
		  bNoErr=PushOper(OP_BIN,10*iKlammer+3,"LOGAND",2,&OpLOGAND,&OStack,&DStack,pcHTM,pcERR,iCnt);
		 }
		 else bNoErr=PushOper(OP_BIN,10*iKlammer+5,"BITAND",2,&OpBITAND,&OStack,&DStack,pcHTM,pcERR,iCnt);
		};
		break;
   case '!':
 	    pcP++;
        if (bDataExpected) bNoErr=PushOper(OP_UN,10*iKlammer+7,"NOT",1,&OpNOT,&OStack,&DStack,pcHTM,pcERR,iCnt);
        else
	    {
	     if (*pcP=='=') 
		 {
		  pcP++;
		  bDataExpected=TRUE;
		  bNoErr=PushOper(OP_BIN,10*iKlammer+2,"NE",2,&OpNE,&OStack,&DStack,pcHTM,pcERR,iCnt);
		 }
		 else
		 {
          Report(EVENTLOG_ERROR_TYPE,273,"Invalid operator",NULL,pcHTM,pcERR,iCnt);
          return (pcP);
		 };
	    };
		break;
   case '+':
        pcP++;
		if (!bDataExpected)
		{
		 bNoErr=PushOper(OP_BIN,10*iKlammer+4,"ADD",2,&OpADD,&OStack,&DStack,pcHTM,pcERR,iCnt);
		 bDataExpected=TRUE;
		};
		break;
   case '-':
        pcP++;
		if (bDataExpected) bNoErr=PushOper(OP_UN,10*iKlammer+7,"NEG",1,&OpNEG,&OStack,&DStack,pcHTM,pcERR,iCnt);
		else 
		{
		 bNoErr=PushOper(OP_BIN,10*iKlammer+4,"SUB",2,&OpSUB,&OStack,&DStack,pcHTM,pcERR,iCnt);
		 bDataExpected=TRUE;
		};
		break;
   case '*':
   case '/':
		if (bDataExpected)
		{
         pcP++;
         Report(EVENTLOG_ERROR_TYPE,272,"Operator not expected",NULL,pcHTM,pcERR,iCnt);
		 return (pcP);
		}
		else
		{
		 if (*pcP=='*') bNoErr=PushOper(OP_BIN,10*iKlammer+5,"MUL",2,&OpMUL,&OStack,&DStack,pcHTM,pcERR,iCnt); 
		 else           bNoErr=PushOper(OP_BIN,10*iKlammer+5,"DIV",2,&OpDIV,&OStack,&DStack,pcHTM,pcERR,iCnt);
		 bDataExpected=TRUE;
		 pcP++;
		};
		break;
   case '(':
        pcP++;
        if (bDataExpected) iKlammer++;
		else
		{
         Report(EVENTLOG_ERROR_TYPE,273,"( not expected",NULL,pcHTM,pcERR,iCnt);
		 return (pcP);
		};
		break;
   case ')':
        if (bFun) 
		{
		 if (bDataExpected)
		 {
          Report(EVENTLOG_ERROR_TYPE,273,") not expected yet in function",NULL,pcHTM,pcERR,iCnt);
		  return (pcP);
		 }
		 else
		 {
		  iKlammer--;
		  if (iKlammer==0) bLoop=FALSE; // End of last parameter in function!
		  else             pcP++; 
		 };
		}
		else
		{
		 pcP++;
		 if (bDataExpected)
		 {
          Report(EVENTLOG_ERROR_TYPE,273,") not expected",NULL,pcHTM,pcERR,iCnt);
		  return (pcP);
		 }
		 else if (iKlammer>0) iKlammer--;
		};
		break;
   case ',':
        if (bFun) // In function, ie. closing the parameter
		{
		 iKlammer--;
		 if (bDataExpected)
		 {
          Report(EVENTLOG_ERROR_TYPE,274,", not expected yet in function parameter",NULL,pcHTM,pcERR,iCnt);
		  return (pcP);
		 }
		 else bLoop=FALSE; // End of function parameter!
		}
		else
		{
		 if (bDataExpected)
		 {
          Report(EVENTLOG_ERROR_TYPE,274,", not expected",NULL,pcHTM,pcERR,iCnt);
		  return (pcP);
		 }
		 else bLoop=FALSE; // Probably just a list of expressions!
		};
		break;
   default:
	    if (isalpha(*pcP) || (*pcP=='_'))
	    {
		 int iVar,i=1;
		 pcEndData=pcP;
		 while (IsValidVarNameChar(*pcEndData)) pcEndData++;
		 if (pcEndData>pcE) pcEndData=pcE;
		 strncpy(cConst,pcP,pcEndData-pcP);
		 cConst[pcEndData-pcP]='\0';
		 if (FindFun(cConst,&FunFound)) // Function?
		 {
		  if (bDataExpected) 
		  {
		   int iParCnt=0;
		   BOOL bParOK=TRUE;
		   STACKENTRY Parameter;
		   pcP=pcEndData;
		   while (strchr(" \t\n\r",*pcP)) pcP++;
		   if (*pcP != '(')
		   {
            Report(EVENTLOG_ERROR_TYPE,276,"( expected after function name",NULL,pcHTM,pcERR,iCnt);
		    return (pcEndData);
		   };
		   do
		   {
		    pcP++; // Skip ( and ,
		    iParCnt++;
			pcP=EvalEx(pcP,pcE,ECD,&Parameter,pVBlock,TRUE,pcHTM,pcERR,iCnt);
			bParOK=(Parameter.Type == ST_EMPTY ? FALSE : PushData(Parameter.Type,Parameter.dwVal,Parameter.fpVal,Parameter.cVal,&DStack,pcHTM,pcERR,iCnt));
		   } while (bParOK && (*pcP == ','));
		   pcP++;
		   if (!bParOK || ((FunFound.iPar != iParCnt) && (FunFound.iPar>0)))
		   {
            char cBuff[128];
            sprintf(cBuff,"'%s()' requires %i parameter(s), not %i (or param #%i is invalid)",cConst,FunFound.iPar,iParCnt,iParCnt);
            Report(EVENTLOG_ERROR_TYPE,279,cBuff,NULL,pcHTM,pcERR,iCnt);
		    return (pcP);
		   }
		   else 
		   {
		    bNoErr=(bParOK ? (FunFound.fpF)(&DStack,iParCnt,pcHTM,pcERR,iCnt) : FALSE); // Evaluate the function if all params are OK!
		    bDataExpected=FALSE;
		   };
		  }
		  else
		  {
           Report(EVENTLOG_ERROR_TYPE,274,"Function not expected",NULL,pcHTM,pcERR,iCnt);
		   return (pcP);
		  };
		 }
		 else							 // no, so it must be
		 {
		  if (bDataExpected)
		  {
		   pcP=GetVarName(pcP,pcE,cConst,ECD,pVBlock,pcHTM,pcERR,iCnt);
		   iVar=FindVar(cConst,pVBlock,TRUE,pcHTM,pcERR,iCnt); // a variable!
		   bNoErr=PushData(pVBlock->Var[iVar]->V.Type,pVBlock->Var[iVar]->V.dwVal,pVBlock->Var[iVar]->V.fpVal,pVBlock->Var[iVar]->V.cVal,&DStack,pcHTM,pcERR,iCnt);
		   bDataExpected=FALSE;
//		   pcP=pcEndData;
		  }
		  else
		  {
           Report(EVENTLOG_ERROR_TYPE,275,"Variable not expected",NULL,pcHTM,pcERR,iCnt);
		   return (pcP);
		  };
		 };
	    }
		else // not A-Z
		{
         if (isdigit(*pcP))
		 {
		  pcEndData=pcP;  // This is somewhat simplified, will take bad constants also!
		  while (strchr(".0123456789",*pcEndData)) pcEndData++;
		  switch (toupper(*pcEndData))
		  {
		   case 'X':
		        pcEndData++;
		        while (isxdigit(*pcEndData)) pcEndData++;
				break;
		   case 'D':
		   case 'E':
		        pcEndData++;
		        while (strchr("0123456789+-",*pcEndData)) pcEndData++;
				break;
		  };
		  if (pcEndData>pcE) pcEndData=pcE;
		  strncpy(cConst,pcP,pcEndData-pcP+1);
		  cConst[pcEndData-pcP]='\0';
		  pcP=pcEndData;
		  bDataExpected=FALSE;
		  if (strchr(cConst,'.'))
		  {
		   fpConst=atof(cConst);
		   bNoErr=PushData(ST_REAL,0,fpConst,"",&DStack,pcHTM,pcERR,iCnt);
		  }
		  else
		  {
		   dwConst=strtoul(cConst,NULL,0); // Will take decimal, octal and hex!
		   bNoErr=PushData(ST_INT,dwConst,0.,"",&DStack,pcHTM,pcERR,iCnt);
		  };
		 }
		 else bLoop=FALSE;                 // Neither digit nor alpha
		};
		break;
  }; //	switch (*pcP)
 };  // while(pcP<pcE etc...) 
 if (iKlammer>0) Report(EVENTLOG_WARNING_TYPE,157,"Missing ) in expression",NULL,pcHTM,pcERR,iCnt);
 if (iKlammer<0) Report(EVENTLOG_WARNING_TYPE,157,"Missing ( in expression",NULL,pcHTM,pcERR,iCnt);
 if (bNoErr)
 {
  while (bNoErr && (OStack.iOSP > 0)) // Call all operators remaining on the stack!
  {
   bNoErr=(OStack.Op[OStack.iOSP].fpF)(&DStack,OStack.Op[OStack.iOSP].iPar,pcHTM,pcERR,iCnt);
   OStack.iOSP--;
  };
  if (DStack.iDSP==1) *pResult=DStack.Data[1];
  else
  {
   Report(EVENTLOG_ERROR_TYPE,278,"Stack mismatch in expression",NULL,pcHTM,pcERR,iCnt);
   return (pcP);
  };
 };
 return (pcP);
};

char* FindMatch(char *pcS,char *pcE,stExecConf ECD,char *pcSearch,char *pcAddLevel,char *pcSubLevel)
{
 char *pcP=pcS;
 BOOL bSeek=TRUE;
 int iLevel=0;

 while (pcP<pcE)
 {
  if (pcP=strstr(pcP,ECD.cCmdStart))
  {
   pcP+=ECD.dwLenCS;
   if (strnicmp(pcP,pcSearch,strlen(pcSearch)) == 0)
   {
    if (iLevel==0) return (pcP-ECD.dwLenCS);
   };
   if (strnicmp(pcP,pcAddLevel,strlen(pcAddLevel)) == 0) iLevel++;
   if (strnicmp(pcP,pcSubLevel,strlen(pcSubLevel)) == 0) iLevel--;
  }
  else return (NULL);
 };
 return NULL;
};

char* SkipCmdEnd(char *pcP,stExecConf ECD,char *pcCmd)
{
 char *pcEnd;
 if (pcEnd=strstr(pcP,ECD.cCmdEnd)) return (pcEnd+ECD.dwLenCE); else return (pcP+strlen(pcCmd));
};

char* HTMLExecute(char *pcS,char *pcE,stExecConf ECD,PMEMBLOCK pMBlock,PVARBLOCK pVBlock,HANDLE LOGHandle,BOOL *pbCanContinue,char* pcHTM,char* pcERR,int iCnt,BOOL bMain,PHBLOCK pASH)
{
 char         cBuff[1024];
 char         cWriteBuff[8192];
 char         *pcPos,*pcCmd;
 char         *pcRepeat=NULL,*pcWhile=NULL,*pcKeywordHeader;
 DWORD        dwRepeat=0,    dwWhile=0;
 BOOL         bSeekCmd=TRUE;
 BOOL         bWriteNext=TRUE;
 int          iVarDef;
 char         cVarName[16][MAX_NAMELEN];
 char         cFunName[MAX_NAMELEN];
 char         *pcFunName=cFunName;
 OPENTRY      FunCmd;
 char         *pcPosBuff;
 stStackEntry Result,Par1,Par2,Par3;
 DWORD        dwFileRW;
 char         *pcLPar;
//
 pcPos=pcS;
 while (bSeekCmd && (pcPos<pcE) && *pbCanContinue)
 {
  if (pcCmd=strstr(pcPos,ECD.cCmdStart))
  {
   if (pcCmd>=pcE)
   {
    AddMBlockEntry(pcPos,pcE,pMBlock,pcHTM,pcERR,iCnt);
    bSeekCmd=FALSE;
   }
   else
   {
    *pbCanContinue=AddMBlockEntry(pcPos,pcCmd-1,pMBlock,pcHTM,pcERR,iCnt);
	if (!(*pbCanContinue)) break;
    pcPos=pcCmd+ECD.dwLenCS;
    while (strchr(" \n\t\r",*pcPos)) pcPos++;
    switch (FindKeyword(pcPos))
    {
     case KW_COMMENT:
 		  pcPos=SkipCmdEnd(pcPos,ECD,"");
          break;
     case KW_INVALID:
          Report(EVENTLOG_ERROR_TYPE,250,"%s","Syntax: Keyword invalid or missing",pcHTM,pcERR,iCnt);
	      break; 
	 case KW_CALL:
	      pcPos+=4;
		  while (strchr(" \n\r\t",*pcPos)) pcPos++;
		  pcLPar=strchr(pcPos,'(');
		  if (pcLPar && (pcLPar-pcPos)<MAX_NAMELEN)
		  {
		   char cCallSubName[MAX_NAMELEN],cRegSubName[4*MAX_NAMELEN];
		   char cRawVal[MAX_EXPAND];
           DWORD dwValType;
		   DWORD dwNameLen=sizeof(cRegSubName);
		   DWORD dwValLen=sizeof(cRawVal);
		   BOOL  bSubFound=FALSE;
		   int   iSubNameLen=pcLPar-pcPos;
		
	       strnset(cCallSubName,'\0',sizeof(cCallSubName));
		   strnset(cRawVal,'\0',sizeof(cRawVal));
		   strncpy(cCallSubName,pcPos,iSubNameLen);
//-------- Search subroutine name in current script text --------
           char *pcSubStart=pcPos; 
           do		 
           {
            if (pcSubStart=FindMatch(ECD.pcScriptStart,ECD.pcScriptEnd,ECD,"SUBROUTINE","RETURN","SUBROUTINE"))
            {
             char *pcRegSubName=cRegSubName;
             pcSubStart=pcSubStart+ECD.dwLenCS+10;
             while (strchr(" \n\r\t",*pcSubStart)) pcSubStart++;
             while (IsValidVarNameChar(*pcSubStart)) *pcRegSubName++=*pcSubStart++;
             int  iPar=0; 
             char cQuote='\0';
             do
             {
              *pcRegSubName++=*pcSubStart;
              *pcRegSubName='\0';
              switch (*pcSubStart)
              {
               case '(':if (cQuote=='\0') iPar++;break;
               case ')':if (cQuote=='\0') iPar--;break;
               case '"':if (cQuote)
                        {
                         if (cQuote=='"') cQuote='\0';
                        }
                        else cQuote='"';
                        break;
               case '\047':if (cQuote)
                        {
                         if (cQuote=='\047') cQuote='\0';
                        }
                        else cQuote='\047';
                        break;
              }; 
              pcSubStart++;
             } while (*pcSubStart && (iPar>0)); 
	         if (bSubFound=(strnicmp(cRegSubName,cCallSubName,iSubNameLen)==0))
		     {
              if (pcSubStart=SkipCmdEnd(pcSubStart,ECD,"")) strncpy(cRawVal,pcSubStart,sizeof(cRawVal));
              else                                          bSubFound=FALSE;  
             };        
            };
           } while (!bSubFound && pcSubStart); 
//-------- Search subroutine name in registry --------------------
	       if (hKeySubroutines && !bSubFound)
		   { 
            int   iX=0;
            while ((RegEnumValue(hKeySubroutines,iX,cRegSubName,&dwNameLen,NULL,&dwValType,(BYTE*) cRawVal,&dwValLen)==ERROR_SUCCESS) && !bSubFound)
            {
			 switch (dwValType)
			 {
			  case REG_EXPAND_SZ: // Filename containing the subroutine body
			  case REG_SZ:
			       if (bSubFound=(strnicmp(cRegSubName,cCallSubName,iSubNameLen)==0))
				   {
				    DWORD    dwFileRW;
//				    OFSTRUCT OFS;
//                  HFILE SubHandle=OpenFile(cRawVal,&OFS,OF_READ | OF_SHARE_DENY_WRITE);
                    HANDLE SubHandle=CreateFile(cRawVal,GENERIC_READ,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
                    if (SubHandle != INVALID_HANDLE_VALUE)
                    {
                     bSubFound=ReadFile(SubHandle,cRawVal,sizeof(cRawVal),&dwFileRW,NULL);
                     CloseHandle(SubHandle);
					}
					else bSubFound=FALSE;
				   };
			       break;
			  case REG_MULTI_SZ:  // Actual subroutine body
			       bSubFound=(strnicmp(cRegSubName,cCallSubName,iSubNameLen)==0);
			       break;
			 };
			 dwNameLen=sizeof(cRegSubName);
			 dwValLen=sizeof(cRawVal);
 	         iX++;
			};
		   }; // if (hKeySubroutines)
		   if (bSubFound)
		   {
//--------- Variable list in subroutine definitions--------			 
			int iSubVarCnt=0;
			char cSubVarName[32][MAX_NAMELEN];
			BOOL bOK=TRUE;
			char *pcSubVar=strchr(cRegSubName,'(');

			if (pcSubVar)
			{
			 pcSubVar++; // skip the (
			 while (*pcSubVar && (*pcSubVar != ')') && bOK && (iSubVarCnt<32))
			 {
			  while (strchr(" \n\t\r",*pcSubVar)) pcSubVar++;
			  if (isalpha(*pcSubVar) || (*pcSubVar=='_'))
			  {
			   char *pcStart=pcSubVar;
			   while (IsValidVarNameChar(*pcSubVar)) pcSubVar++;
			   strnset(cSubVarName[++iSubVarCnt],'\0',MAX_NAMELEN);
			   strncpy(cSubVarName[iSubVarCnt],pcStart,pcSubVar-pcStart);
			   while (strchr(" \n\t\r",*pcSubVar)) pcSubVar++;
			   if (*pcSubVar==',') pcSubVar++;
			  }
		      else bOK=FALSE;
			 };
			} else bOK=FALSE;
			if (bOK)
			{
			 int         i,iCallParCnt=0;
		     struct      stVarBlock SubVar; // Each subroutine has separate variables!
			 STACKENTRY  Parameter;
			 PSTVARIABLE pVar;
			
             SubVar.iLast=-1;              // none for starters...
	         GetRidOfNullChars(cRawVal,'\n');
//		      ExpandEnvironmentStrings(cRawVal,cSubBody,sizeof(cSubBody));
//-----------Transfer all global vars---------------
			 for(i=0;i<=(pVBlock->iLast);i++) if (pVBlock->Var[i]->cXName[0]) AddVBlockEntry(NULL,pVBlock->Var[i]->cName,pVBlock->Var[i]->V,&SubVar,cRegSubName,pcERR,iCnt);
//-----------Transfer call parameters---------------
		     pcPos=pcLPar+1; // Skip ( 
		     while (*pcPos && (*pcPos !=')') && bOK)
			 {
			  pcPos=EvalEx(pcPos,pcE,ECD,&Parameter,pVBlock,TRUE,pcHTM,pcERR,iCnt);
			  iCallParCnt++;
			  if (iCallParCnt<=iSubVarCnt)
			  {
			   switch (Parameter.Type)
			   {
			    case ST_EMPTY:
			         bOK=FALSE;
				     break;
				case ST_VARPTR:
			         pVar=(PSTVARIABLE) Parameter.dwVal;
			         AddVBlockEntry(Parameter.cVal,cSubVarName[iCallParCnt],pVar->V,&SubVar,cRegSubName,pcERR,iCnt);
			         break;
			    default:
			         AddVBlockEntry(NULL,cSubVarName[iCallParCnt],Parameter,&SubVar,cRegSubName,pcERR,iCnt);
			   };
			  } else bOK=FALSE;
			  while (strchr(" \n\t\r",*pcPos)) pcPos++;
			  if (*pcPos==',') pcPos++;
			 };
			 if (bOK)
			 {
//-----------Call the subroutine ... ---------------
			  HTMLExecute(cRawVal,cRawVal+strlen(cRawVal),ECD,pMBlock,&SubVar,LOGHandle,pbCanContinue,cRegSubName,pcERR,iCnt,FALSE,pASH);
//-----------Return variables passed by name--------
			  for(i=0;i<=(SubVar.iLast);i++) if (SubVar.Var[i]->cXName[0]) AddVBlockEntry(NULL,SubVar.Var[i]->cXName,SubVar.Var[i]->V,pVBlock,cRegSubName,pcERR,iCnt);
			 } 
             else Report(EVENTLOG_ERROR_TYPE,294,"CALL failed - Subroutine '%s' called with invalid or missing parameters",cCallSubName,pcHTM,pcERR,iCnt); 
			}
			else Report(EVENTLOG_ERROR_TYPE,293,"CALL failed - Subroutine '%s' defined with invalid or missing variables",cCallSubName,pcHTM,pcERR,iCnt);
		   }
		   else Report(EVENTLOG_ERROR_TYPE,292,"CALL failed - Subroutine '%s' not defined",cCallSubName,pcHTM,pcERR,iCnt);
		  }
		  else Report(EVENTLOG_ERROR_TYPE,291,"%s","CALL failed - Subroutine name invalid or too long",pcHTM,pcERR,iCnt);
 		  pcPos=SkipCmdEnd(pcPos,ECD,"");
		  break;
	 case KW_IF:
	      char *pcElse,*pcEndIf;
	  	  pcPos=EvalEx(pcPos+2,pcE,ECD,&Result,pVBlock,FALSE,pcHTM,pcERR,iCnt);
 		  pcPos=SkipCmdEnd(pcPos,ECD,"");
          if (pcEndIf=FindMatch(pcPos,pcE,ECD,"ENDIF","IF ","ENDIF"))
          {
		 	 				                    // -------------------------------------
		   if (IsTrue(Result,pcHTM,pcERR,iCnt)) // TRUE, do between IF-ELSE, or IF-ENDIF
		   {				                    // -------------------------------------
		    if (pcElse=FindMatch(pcPos,pcEndIf,ECD,"ELSE","IF ","ENDIF")) 
		    {
		     HTMLExecute(pcPos,pcElse-1,ECD,pMBlock,pVBlock,LOGHandle,pbCanContinue,pcHTM,pcERR,iCnt,bMain,pASH);
            }
            else 
            {
		     HTMLExecute(pcPos,pcEndIf-1,ECD,pMBlock,pVBlock,LOGHandle,pbCanContinue,pcHTM,pcERR,iCnt,bMain,pASH);
            }; 
           }
		                                        // ----------------------------------------------------------
		   else                                 // FALSE, do everything between ELSE-ENDIF or skip pass ENDIF
		   {	                                // ----------------------------------------------------------
		    if (pcElse=FindMatch(pcPos,pcEndIf,ECD,"ELSE","IF ","ENDIF")) 
		    {
		     pcPos=SkipCmdEnd(pcElse,ECD,"ELSE");
		     HTMLExecute(pcPos,pcEndIf-1,ECD,pMBlock,pVBlock,LOGHandle,pbCanContinue,pcHTM,pcERR,iCnt,bMain,pASH);
            };
		   };
	       pcPos=SkipCmdEnd(pcEndIf,ECD,"ENDIF");
          }
	      else // No ENDIF!
		  {
           Report(EVENTLOG_ERROR_TYPE,251,"%s","Syntax: ENDIF missing",pcHTM,pcERR,iCnt);
		   bSeekCmd=FALSE;
		  };
	      break; 
  	 case KW_ELSE:
          Report(EVENTLOG_ERROR_TYPE,251,"%s","Syntax: ELSE before IF",pcHTM,pcERR,iCnt);
	 	  pcPos=SkipCmdEnd(pcPos,ECD,"ELSE");
	      break; 
	case  KW_ENDIF:
          Report(EVENTLOG_ERROR_TYPE,251,"%s","Syntax: ENDIF before IF",pcHTM,pcERR,iCnt);
		  pcPos=SkipCmdEnd(pcPos,ECD,"ENDIF");
	      break; 
	case  KW_REPEAT:
	      pcRepeat=pcCmd; 
		  char *pcUntil;
		  pcPos=SkipCmdEnd(pcPos,ECD,"REPEAT");
		  if (pcUntil=FindMatch(pcPos,pcE,ECD,"UNTIL","REPEAT","UNTIL"))
		  {
           HTMLExecute(pcPos,(char*) pcUntil-1,ECD,pMBlock,pVBlock,LOGHandle,pbCanContinue,pcHTM,pcERR,iCnt,bMain,pASH);
		   pcPos=EvalEx(pcUntil+ECD.dwLenCS+5,pcE,ECD,&Result,pVBlock,FALSE,pcHTM,pcERR,iCnt);
		   pcPos=SkipCmdEnd(pcPos,ECD,"");
		   if (!IsTrue(Result,pcHTM,pcERR,iCnt))
		   {
		    dwRepeat++;
		    if (dwRepeat>ECD.dwMaxLoop) Report(EVENTLOG_WARNING_TYPE,151,"REPEAT-UNTIL loop terminated after %s iterations",ECD.cMaxLoop,pcHTM,pcERR,iCnt);
		    else pcPos=pcRepeat; // *** LOOP ***
		   }; 
		  }
		  else
		  {
           Report(EVENTLOG_ERROR_TYPE,251,"%s","Syntax: UNTIL missing",pcHTM,pcERR,iCnt);
           HTMLExecute(pcPos,pcE,ECD,pMBlock,pVBlock,LOGHandle,pbCanContinue,pcHTM,pcERR,iCnt,bMain,pASH);
		   bSeekCmd=FALSE;
		  };
	      break; 
	 case KW_UNTIL:
          Report(EVENTLOG_ERROR_TYPE,251,"%s","Syntax: UNTIL before REPEAT",pcHTM,pcERR,iCnt);
		  pcPos=SkipCmdEnd(pcPos,ECD,"UNTIL"); // Bug: does not ignore question marks!
	      break; 
	case  KW_WHILE: 
	      pcWhile=pcCmd;
		  char *pcEndWhile;
		  pcPos=EvalEx(pcPos+5,pcE,ECD,&Result,pVBlock,FALSE,pcHTM,pcERR,iCnt);
		  pcPos=SkipCmdEnd(pcPos,ECD,"");
		  if (IsTrue(Result,pcHTM,pcERR,iCnt))
		  {
		   if (pcEndWhile=FindMatch(pcPos,pcE,ECD,"ENDWHILE","WHILE","ENDWHILE"))
		   {
		    pcPos=HTMLExecute(pcPos,(char*) pcEndWhile-1,ECD,pMBlock,pVBlock,LOGHandle,pbCanContinue,pcHTM,pcERR,iCnt,bMain,pASH);
		    dwWhile++;
		    if (dwWhile>ECD.dwMaxLoop) 
		    {
		     Report(EVENTLOG_WARNING_TYPE,151,"WHILE-ENDWHILE loop terminated after %s iterations",ECD.cMaxLoop,pcHTM,pcERR,iCnt);
			 pcPos=SkipCmdEnd(pcEndWhile,ECD,"ENDWHILE");
		    }
		    else pcPos=pcWhile; // *** LOOP ***
		   }
		   else // No ENDWHILE
		   {
            Report(EVENTLOG_ERROR_TYPE,251,"%s","Syntax: ENDWHILE missing",pcHTM,pcERR,iCnt);
		    HTMLExecute(pcPos,pcE,ECD,pMBlock,pVBlock,LOGHandle,pbCanContinue,pcHTM,pcERR,iCnt,bMain,pASH);
		    bSeekCmd=FALSE;
		   };
		  }
		  else // FALSE, skip pass ENDWHILE
		  {
		   if (pcEndWhile=FindMatch(pcPos,pcE,ECD,"ENDWHILE","WHILE","ENDWHILE")) pcPos=SkipCmdEnd(pcEndWhile,ECD,"ENDWHILE");
		   else // No ENDWHILE
		   {
            Report(EVENTLOG_ERROR_TYPE,251,"%s","Syntax: ENDWHILE missing",pcHTM,pcERR,iCnt);
		    bSeekCmd=FALSE;
		   };
		  };
	      break; 
	 case KW_ENDWHILE:
          Report(EVENTLOG_ERROR_TYPE,251,"%s","Syntax: ENDWHILE before WHILE",pcHTM,pcERR,iCnt);
	 	  pcPos=SkipCmdEnd(pcPos,ECD,"ENDWHILE");
	      break; 
	 case KW_SUB: // Means we finished the main part...
		  pcPos=SkipCmdEnd(pcPos,ECD,""); 
          if (bMain)
             *pbCanContinue=FALSE;        // THIS WILL ABORT HTMLExecute()!!!
          else
             bSeekCmd=FALSE;              // RETURN FROM SUBROUTINE!    
		  break;
     case KW_RET:
          pcPos+=2; 
     case KW_EXIT:
          pcPos+=4;
          while (strchr(" \n\r\t",*pcPos)) pcPos++;
          if (strncmp(pcPos,ECD.cCmdEnd,ECD.dwLenCE)) // Looks like a parameter?
          { 
		   pcPos=EvalEx(pcPos,pcE,ECD,&Result,pVBlock,FALSE,pcHTM,pcERR,iCnt);
           if (Result.Type==ST_STRING) Report(EVENTLOG_WARNING_TYPE,153,"%s",Result.cVal,pcHTM,pcERR,iCnt);
          };
		  pcPos=SkipCmdEnd(pcPos,ECD,""); // Just being polite, not needed
          if (bMain)
             *pbCanContinue=FALSE;        // THIS WILL ABORT HTMLExecute()!!!
          else
             bSeekCmd=FALSE;              // RETURN FROM SUBROUTINE!    
		  break;
	 case KW_WRITE:
	      pcPos+=5;
		  cWriteBuff[0]='\0';
          do
		  {
		   if (*pcPos==',') pcPos++;
		   pcPos=EvalEx(pcPos,pcE,ECD,&Result,pVBlock,FALSE,pcHTM,pcERR,iCnt);
		   switch (Result.Type)
		   {
		    case ST_INT:
		         sprintf(Result.cVal,"%i",Result.dwVal);
				 strcat(cWriteBuff,Result.cVal);
		         break;
		    case ST_REAL:
		         sprintf(Result.cVal,"%g",Result.fpVal);   
				 strcat(cWriteBuff,Result.cVal);
		         break;
		    case ST_STRING:
				 strcat(cWriteBuff,Result.cVal);
		         break;
		   };
		  } while (bWriteNext && (*pcPos==','));
          *pbCanContinue=AddMBlockEntry(cWriteBuff,cWriteBuff+strlen(cWriteBuff),pMBlock,pcHTM,pcERR,iCnt);
		  pcPos=SkipCmdEnd(pcPos,ECD,"");
		  break;
	 case KW_FILE:
		  pcPos=EvalEx(pcPos+4,pcE,ECD,&Result,pVBlock,FALSE,pcHTM,pcERR,iCnt);
		  pcPos=SkipCmdEnd(pcPos,ECD,"");
		  if (AssertType(Result,ST_STRING,"FILE command param must be string",pcHTM,pcERR,iCnt))
		  {
		   BY_HANDLE_FILE_INFORMATION WFInfo;
//		   OFSTRUCT	WFS;
//         HFILE    WFHandle=OpenFile(Result.cVal,&WFS,OF_READ | OF_SHARE_DENY_NONE);
           HANDLE WFHandle=CreateFile(Result.cVal,GENERIC_READ,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
           if (WFHandle==INVALID_HANDLE_VALUE)
           {
#ifdef WINET
            if (hInstWinetDLL)
            {
            HINTERNET hiConn=(*fpInternetOpenA)(ECD.bIsFilter ? pcHTMLExDescF : pcHTMLExDescE,INTERNET_OPEN_TYPE_PRECONFIG,NULL,NULL,0);
            if (hiConn)
            {
             HINTERNET hiURL=(*fpInternetOpenUrlA)(hiConn,Result.cVal,NULL,0,INTERNET_FLAG_RELOAD,0);
             if (hiURL)
             {
			  char *pcTempBuff=(char*) GlobalAlloc(GPTR,MAX_WINET_FILE);
              if (pcTempBuff)
              {
               if ((*fpInternetReadFile)(hiURL,(void*) pcTempBuff,MAX_WINET_FILE,&dwFileRW))
               {
                if (dwFileRW) // Read something?
                {
                 int i=pMBlock->iLast+1;
                 if (pMBlock->MB[i]=(char*) GlobalAlloc(GPTR,dwFileRW+1))
                 {
                  strcpy((char*) pMBlock->MB[i],pcTempBuff);
                  pMBlock->iLast=i;
                 }
                 else
                 {
                  ltoa(dwFileRW+1,cBuff,10);
                  Report(EVENTLOG_ERROR_TYPE,254,"GlobalAlloc(GPTR,%s) failed in FILE command",cBuff,pcHTM,pcERR,iCnt);
                 };
                } 
                else Report(EVENTLOG_ERROR_TYPE,306,"InternetReadFile() in FILE command read 0 bytes",NULL,pcHTM,pcERR,iCnt);
               };
               GlobalFree((HGLOBAL) pcTempBuff); 
              };    // if (pcBuff)
              (*fpInternetCloseHandle)(hiURL);
             }		// if (hiURL)
             else Report(EVENTLOG_ERROR_TYPE,305,"InternetOpenUrl(,%s,,,,) failed in FILE command",Result.cVal,pcHTM,pcERR,iCnt);
             (*fpInternetCloseHandle)(hiConn);
            }
            else Report(EVENTLOG_ERROR_TYPE,304,"InternetOpen() failed in FILE command",NULL,pcHTM,pcERR,iCnt);
            };      // if (hInstWinetDLL)
#else
            Report(EVENTLOG_ERROR_TYPE,253,"Failed opening '%s' in FILE command",Result.cVal,pcHTM,pcERR,iCnt);
#endif
           } 
		   else
		   {
            GetFileInformationByHandle(WFHandle,&WFInfo);
		    if (WFInfo.nFileSizeHigh==0) // ABSURD!
		    {
             int i=pMBlock->iLast+1;
             if (pMBlock->MB[i]=(char*) GlobalAlloc(GPTR,WFInfo.nFileSizeLow+1))
             {
              ReadFile(WFHandle,(void*) pMBlock->MB[i],WFInfo.nFileSizeLow,&dwFileRW,NULL);
              pMBlock->iLast=i;
             }
             else
             {
              ltoa(WFInfo.nFileSizeLow,cBuff,10);
              Report(EVENTLOG_ERROR_TYPE,254,"GlobalAlloc(GPTR,%s) failed in FILE command",cBuff,pcHTM,pcERR,iCnt);
             };
		    };
		   };
		   CloseHandle(WFHandle);
          };
		  break;
	 case KW_LOG: 
	 	  pcPos=EvalEx(pcPos+3,pcE,ECD,&Par1,pVBlock,FALSE,pcHTM,pcERR,iCnt);
		  if (AssertType(Par1,ST_STRING,"LOG param #1 must be string",pcHTM,pcERR,iCnt))
		  {
		   while (*pcPos==',') pcPos++;
		   pcPos=EvalEx(pcPos,pcE,ECD,&Par2,pVBlock,FALSE,pcHTM,pcERR,iCnt);
		   if (AssertType(Par2,ST_STRING,"LOG param #2 must be string",pcHTM,pcERR,iCnt)) 
           {
            if (LOGHandle != INVALID_HANDLE_VALUE) WriteFile(LOGHandle,cBuff,sprintf(cBuff,"%s=%s\r\n",Par1.cVal,Par2.cVal),&dwFileRW,NULL);
            else                                   Report(EVENTLOG_WARNING_TYPE,159,"LOG command failed - logging disabled for '%s'",pcHTM,pcHTM,pcERR,iCnt);
           }
		  };  // Assert Par1
		  pcPos=SkipCmdEnd(pcPos,ECD,"");
  	      break; 
     case KW_SETHEADER:
     case KW_ADDHEADER:
          pcPos+=3;
	 case KW_HEADER: 
          pcKeywordHeader=pcPos; // Points to 'H' in the word 'HEADER'!!!
	 	  pcPos=EvalEx(pcPos+6,pcE,ECD,&Par1,pVBlock,FALSE,pcHTM,pcERR,iCnt);
		  if (AssertType(Par1,ST_STRING,"HEADER name (param #1) must be string",pcHTM,pcERR,iCnt))
		  {
           if (Par1.cVal[0])
           {
		    while (*pcPos==',') pcPos++;
		    pcPos=EvalEx(pcPos,pcE,ECD,&Par2,pVBlock,FALSE,pcHTM,pcERR,iCnt);
		    if (AssertType(Par2,ST_STRING,"HEADER value (param #2) must be string",pcHTM,pcERR,iCnt)) 
            {
             if (Par2.cVal[0])
             { 
              int iLast=pASH->iLast;
              BOOL bAddMode=TRUE; // Suppose the header will be added...
              if ((pcKeywordHeader-3) >= pcS)
              {
               if (toupper(*(pcKeywordHeader-3))=='S') bAddMode=FALSE;
              }; 
              if (iLast>=MAX_HBLOCK) Report(EVENTLOG_ERROR_TYPE,287,"HEADER command failed -  cannot add '%s'",Par1.cVal,pcHTM,pcERR,iCnt);
              else // Add to header block...
              {
               iLast++;
               if (Par1.cVal[strlen(Par1.cVal)-1] != ':') strcat(Par1.cVal,":");
               strcpy(pASH->cHName [iLast],Par1.cVal); 
               strcpy(pASH->cHValue[iLast],Par2.cVal); 
			   pASH->bAdd[iLast]=bAddMode;
               pASH->iLast=iLast;
              }
             } // Par2 not empty
            }  // Assert Par2
           }   // Par1 not empty
		  };   // Assert Par1
		  pcPos=SkipCmdEnd(pcPos,ECD,"");
  	      break; 
	 case KW_EVENT:
		  pcPos=EvalEx(pcPos+5,pcE,ECD,&Par1,pVBlock,FALSE,pcHTM,pcERR,iCnt);
		  if (AssertType(Par1,ST_INT,"EVENT param #1 must be integer",pcHTM,pcERR,iCnt))
		  {
		   while (*pcPos==',') pcPos++;
		   pcPos=EvalEx(pcPos,pcE,ECD,&Par2,pVBlock,FALSE,pcHTM,pcERR,iCnt);
		   if (AssertType(Par2,ST_INT,"EVENT param #2 must be integer",pcHTM,pcERR,iCnt))
		   {
		    while (*pcPos==',') pcPos++;
		    pcPos=EvalEx(pcPos,pcE,ECD,&Par3,pVBlock,FALSE,pcHTM,pcERR,iCnt);
		    if (AssertType(Par3,ST_STRING,"EVENT param #3 must be string",pcHTM,pcERR,iCnt))
		    {
		     switch (Par1.dwVal % 3)
		     {
		      case 0:
                   Report(EVENTLOG_INFORMATION_TYPE,Par2.dwVal,"%s",Par3.cVal,pcHTM,pcERR,iCnt);
				   break;
		      case 1:
                   Report(EVENTLOG_WARNING_TYPE,Par2.dwVal,"%s",Par3.cVal,pcHTM,pcERR,iCnt);
				   break;
		      case 2:
                   Report(EVENTLOG_ERROR_TYPE,Par2.dwVal,"%s",Par3.cVal,pcHTM,pcERR,iCnt);
				   break;
		     };
		    };    // Assert Par3
		   }	  // Assert Par2
		  };	  // Assert Par1
		  pcPos=SkipCmdEnd(pcPos,ECD,"");
	      break; 
	 default: // THIS MUST BE A FUNCTION CALL OR VARIABLE DEFINITION!
          pcPosBuff=pcPos;
          pcFunName=cFunName;
          *pcFunName='\0';
          while (IsValidVarNameChar(*pcPosBuff)) 
          {
           *pcFunName++=*pcPosBuff++;
           *pcFunName='\0';
 		  };
          if (FindFun(cFunName,&FunCmd)) // FUNCTION
          {
		   int iParCnt=0;
		   BOOL bParOK=TRUE;
		   STACKENTRY Parameter;
           DATASTACK DStack;
           DStack.iDSP=0;
 		   pcPos=pcPosBuff;
		   while (strchr(" \t\n\r",*pcPos)) pcPos++;
		   if (*pcPos != '(') Report(EVENTLOG_ERROR_TYPE,276,"( expected after function name",NULL,pcHTM,pcERR,iCnt);
		   do
		   {
		    pcPos++; // Skip ( and ,
		    iParCnt++;
			pcPos=EvalEx(pcPos,pcE,ECD,&Parameter,pVBlock,TRUE,pcHTM,pcERR,iCnt);
			bParOK=(Parameter.Type == ST_EMPTY ? FALSE : PushData(Parameter.Type,Parameter.dwVal,Parameter.fpVal,Parameter.cVal,&DStack,pcHTM,pcERR,iCnt));
		   } while (bParOK && (*pcPos == ','));
		   pcPos++;
		   if (!bParOK || ((FunCmd.iPar != iParCnt) && (FunCmd.iPar>0)))
           {
            sprintf(cBuff,"'%s()' requires %i parameter(s), not %i (or param #%i is invalid)",cFunName,FunCmd.iPar,iParCnt,iParCnt);     
            Report(EVENTLOG_ERROR_TYPE,279,cBuff,NULL,pcHTM,pcERR,iCnt);
           }
		   else (FunCmd.fpF)(&DStack,iParCnt,pcHTM,pcERR,iCnt); // Evaluate the function if all params are OK!
          }
          else	// VARIABLE ASSIGNMENT
          {
           iVarDef=0;
		   do
		   {
		    if (*pcPos==',') pcPos++;
		    pcPos=GetVarName(pcPos,pcE,cVarName[0],ECD,pVBlock,pcHTM,pcERR,iCnt);
		    if (cVarName[0][0]) strcpy(cVarName[++iVarDef],cVarName[0]);
		   } while(cVarName[0][0] && (iVarDef<16) && (*pcPos==','));
		   while (strchr(" =\t\n\r",*pcPos)) pcPos++;
		   pcPos=EvalEx(pcPos,pcE,ECD,&Result,pVBlock,FALSE,pcHTM,pcERR,iCnt);
           for (int i=1;i<=iVarDef;i++) AddVBlockEntry(NULL,cVarName[i],Result,pVBlock,pcHTM,pcERR,iCnt);
          };
	      pcPos=SkipCmdEnd(pcPos,ECD,"");
	      break; 
    };  // switch
   }
  }	  
  else // pcCmd==NULL
  {
   AddMBlockEntry(pcPos,pcPos+strlen(pcPos),pMBlock,pcHTM,pcERR,iCnt);
   bSeekCmd=FALSE;
  };  // if (pcCmd)
 }    // while (...)
 return (pcPos);
};

/////////////////////////////////////////////////////////////////
//	             TAG EXPANSION ROUTINES
/////////////////////////////////////////////////////////////////
void TagConst(char *pcName,char *pcValue,char *pcTagName,const char *pcTagValue)
{
 strcpy(pcName,pcTagName);
 strcpy(pcValue,pcTagValue);
};

void TagServVar(char *pcName,char *pcValue,char *pcTagName,EXTENSION_CONTROL_BLOCK *pECB,HTTP_FILTER_CONTEXT *pfc,char *pcServVarName,char* pcDefault)
{
 DWORD dwValueLen=MAX_TAGVALUE;
 TagConst(pcName,pcValue,pcTagName,pcDefault);
 if (pECB)
  (pECB->GetServerVariable)(pECB->ConnID,pcServVarName,pcValue,&dwValueLen);
 else
  (pfc->GetServerVariable)(pfc,pcServVarName,pcValue,&dwValueLen);
};
		
BOOL GetValueFromTagKeyInINI(char* pcININame,char* pcTagN,char* pcTagV)
{
 GetPrivateProfileString(KEY_TAGS,pcTagN,pcTagN,pcTagV,MAX_TAGVALUE,pcININame);
 return (strcmp(pcTagN,pcTagV));
};

BOOL GetValueFromTagKeyInReg(HKEY hKeyActual,char* pcTagN,char* pcTagV)
{
 DWORD dwValLen,dwType;
 char cUnExpanded[MAX_TAGVALUE];

 if (hKeyActual)
 {
  dwValLen=MAX_TAGVALUE;
  if(RegQueryValueEx(hKeyActual,pcTagN,NULL,&dwType,(LPBYTE) cUnExpanded,&dwValLen) == ERROR_SUCCESS)
  {
   if (dwType==REG_MULTI_SZ) GetRidOfNullChars(cUnExpanded,'\n');
   ExpandEnvironmentStrings(cUnExpanded,pcTagV,MAX_TAGVALUE);
   return (TRUE);
  };
 };
 return (FALSE);
};

BOOL PickValueFromRandomKeyInINI(char* pcININame,char* pcTagN,char* pcTagV)
{
 char cRandomLine['Z'-'A'+1][MAX_TAGVALUE];
 int  iRandomCnt=0;
 char cRandomEntryName[8],cRandomEntryDefault[8];

 for(char c='A';c<='Z';c++)
 {
  sprintf(cRandomEntryName,"c%c",c);
  sprintf(cRandomEntryDefault,"c%c",c);
  GetPrivateProfileString(pcTagN,cRandomEntryName,cRandomEntryDefault,cRandomLine[iRandomCnt],MAX_TAGVALUE,pcININame);
  if (strcmp(cRandomEntryDefault,cRandomLine[iRandomCnt])) iRandomCnt++;
 };
 if (iRandomCnt)
 {
  strcpy(pcTagV,cRandomLine[rand() % iRandomCnt]);
  return (TRUE);
 };
 return (FALSE);
};

BOOL PickValueFromRandomKeyInReg(HKEY hKeyParent,char* pcTagN,char* pcTagV)
{
 char cRandomLine['Z'-'A'+1][MAX_TAGVALUE];
 char cUnExpanded[MAX_TAGVALUE];
 int  iRandomCnt=0;
 char cRandomEntryName[8];
 HKEY hKeyRandom;
 DWORD dwValLen,dwType;

 if(hKeyParent)
 {
  if (RegOpenKeyEx(hKeyParent,pcTagN,0,KEY_READ,&hKeyRandom)==ERROR_SUCCESS) 
  {
   for(char c='A';c<='Z';c++)
   {
    sprintf(cRandomEntryName,"%c",c); // Note: not cA..cZ, just A...Z
    dwValLen=sizeof(cRandomLine[0]);
    if (RegQueryValueEx(hKeyRandom,cRandomEntryName,NULL,&dwType,(LPBYTE) cUnExpanded,&dwValLen) == ERROR_SUCCESS)
    {
	 if (dwType==REG_MULTI_SZ) GetRidOfNullChars(cUnExpanded,'\n');
	 ExpandEnvironmentStrings(cUnExpanded,cRandomLine[iRandomCnt],MAX_TAGVALUE);
	 iRandomCnt++;
	};
   };
   RegCloseKey(hKeyRandom);
   if (iRandomCnt)
   {
    strcpy(pcTagV,cRandomLine[rand() % iRandomCnt]);
    return (TRUE);
   };
  };
 };
 return (FALSE);
};

void ErrorPage(HTTP_FILTER_CONTEXT *pfc,int iErr,char *pcErr)
{
 char cErrPage[2048];

 sprintf(cErrPage,"<html><head><title>HTMLEx Error #%i</title></head><body><hr><h2>%s</h2><hr><h4>%s</h4></body></html>",iErr,pcErr,pcHTMLExVer);
 if (pfc->pFilterContext=pfc->AllocMem(pfc,strlen(cErrPage+1),0)) strcpy((char*) pfc->pFilterContext,cErrPage);
};

void HTMLExpand(EXTENSION_CONTROL_BLOCK *pECB,HTTP_FILTER_CONTEXT *pfc,char* pcHTMName,char* pcININame,char* pcLOGName,char *pcERRName)
{

 HGLOBAL         hExpanded;
 BY_HANDLE_FILE_INFORMATION HTMInfo;
 FILETIME        FTLocal;
 SYSTEMTIME      STNow,STFileLocal,STGMT,STAnHourAgo;
 tm              TMNow,TMFileLocal,TMGMT,TMAnHourAgo;
// OFSTRUCT        OFS;
 DWORD           dwFileRW; 
 DWORD           dwHTMExSize;
 BOOL            bNotErrorReply=TRUE; // ... serving the client requested file, not the error substitute
 char            cBuff[256];
 char            cTagN[MAX_TAGVALUE],cTagV[MAX_TAGVALUE];
 char            *pcHTM=NULL,*pcS,*pcF1,*pcF2,*pcD;
 int             iCat,i,j;
 char            cTagName[MAX_TAG][MAX_TAGNAME],cTagValue[MAX_TAG][MAX_TAGVALUE];
 char            cRawTag[MAX_TAGNAME];
 BOOL            bTagFound;

 char            cTodaysDate[64],cTextCnt[32];
// [LastAccess]  section, will be updated!
 char            cDate[128],cTime[128];
 char            cIP[128],cRemoteIP[128],cAsteriskedIP[128];
 const char      *pcAsteriskTrail[]={"*.*.*.*",".*.*.*",".*.*",".*"};
 char            cHost[128],cLastHost[128];
 char            cBrowser[256];
 char            cReferer[256];
 char            cCookie[MAX_TAGVALUE];
 int             iCnt;              // Default: 0 
// [Log]         section, read only, except cStartDate!
 char            cIgnoreAddress[128]; // Default: cRegIgnoreAddress 
 char            cStartDate[128];     // Default: <cTodaysDate>
 char            cDateFormat[128];    // Default: cRegDateFormat
 char            cTimeFormat[128];    // Default: cRegTimeFormat
 int             iDigitCnt;		      // Default: dwRegDigitCnt, Valid: 2..9
// [Config]      section, read only!
 char            cTagStart[16],cTagEnd[16]; // Default: cRegTag...
 int             iGenerateErrFiles;         // Default: dwRegGenerateErrFiles
 char*           pcERR;
 DWORD           dwLenTS,dwLenTE;
// [Exec]        section, read only!
 struct stExecConf ECD;
// [Digits]      section, read only!
 char            cDigitImg[10][128];   
// Control this page via <!HTMLEx [noexpand | noexpand] [execute | noexecute] [nolog] [justcount] [noctags] [nortags] [ignorecache]>
 char            *pcHTMLEx;
 DWORD           dwLogThisOne=1;    // Log by default
 BOOL            bCreateINI=TRUE;   // (Almost) always mantain .INIs
 DWORD           dwJustCount=0;	    // Do everything, don't just count
 BOOL            bExpandCTags=TRUE; // Expand constant tags
 BOOL            bExpandRTags=TRUE; // Expand random tags
// Logging...
 HANDLE          LOGHandle=INVALID_HANDLE_VALUE;
 BY_HANDLE_FILE_INFORMATION LOGInfo;
 BOOL            bZapLOGFile;
 int             iMaxLogFileSize;   // Default: dwRegMaxLogFileSize
// Count this hit ...
 BOOL            bCountThisHit=TRUE;// YES, OF COURSE!
//
// SCRIPT LANGUAGE VARIABLES
//
 BOOL   bExecute=TRUE;
 struct stMemBlock MBlock;
 struct stVarBlock VBlock;
 const char *pcTwoZeroes="\0\0";
 const char *GSTVarName[]={"_Year","_Month","_DayOfWeek","_Day","_Hour","_Minute","_Second","_Milliseconds","_TickCount","_MemoryLoad"};
 char   cVarName[MAX_NAMELEN];
 DWORD  dwNameLen=sizeof(cVarName);
 char   cVarValue[MAX_STACKSTRING];
 DWORD  dwValueLen=sizeof(cVarValue);
 char   cINIVarBuff[2048];
 HBLOCK SrvReplyHead;          // Will accumulate additional HTTP headers here... 

 SrvReplyHead.iLast=0;
 ECD.bIsFilter=(pECB == NULL); // Identity problem: am I a filter or an extension?
 if (!FileExists(pcHTMName))
 {
  char cRequestedHTMName[256];
  strcpy(cRequestedHTMName,pcHTMName);
  strcpy(strrchr(pcHTMName,'\\')+1,cRegD404Reply);
  if (!FileExists(pcHTMName))
  {
   strcpy(pcHTMName,cRegS404Reply);
   if (!FileExists(pcHTMName))
   {
    if (dwRegLog404Errors) Report(EVENTLOG_ERROR_TYPE,202,"File not found - server error message sent instead",NULL,cRequestedHTMName,NULL,0);
    return;
   };
  };
  bNotErrorReply=FALSE; // will block .INI and .LOG generation for non-existent file!
  if (dwRegLog404Errors) Report(EVENTLOG_ERROR_TYPE,202,"File not found - '%s' sent instead",pcHTMName,cRequestedHTMName,NULL,0);
 }
// ----------------------------
// OPEN THE REQUESTED HTM FILE!
// ----------------------------
 HANDLE HTMHandle=CreateFile(pcHTMName,GENERIC_READ | GENERIC_WRITE,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
 if (HTMHandle==INVALID_HANDLE_VALUE)
 {
  Report(EVENTLOG_ERROR_TYPE,203,"Failed opening '%s' in HTMLExpand()",pcHTMName,pcHTMName,NULL,0);
  return;										//And here neither? 
 };
 GetFileInformationByHandle(HTMHandle,&HTMInfo);
 if (HTMInfo.nFileSizeHigh != 0)  //This is beyond any reason...
 {
  CloseHandle((HANDLE) HTMHandle);
  Report(EVENTLOG_WARNING_TYPE,204,"File %s is too large.",pcHTMName,pcHTMName,NULL,0);
  return;                                    
 };
 dwHTMExSize=HTMInfo.nFileSizeLow+MAX_EXPAND;
 pcHTM=(char*)GlobalAlloc(GPTR,dwHTMExSize);
 if (pcHTM == NULL)
 {
  CloseHandle((HANDLE) HTMHandle);
  ltoa(dwHTMExSize,cBuff,10);
  Report(EVENTLOG_ERROR_TYPE,205,"GlobalAlloc(,%s) failed.",cBuff,pcHTMName,NULL,0);
  return;
 };
 if (!ReadFile((HANDLE) HTMHandle,pcHTM,HTMInfo.nFileSizeLow,&dwFileRW,NULL))
 {
  CloseHandle((HANDLE) HTMHandle);
  GlobalFree(pcHTM);
  Report(EVENTLOG_ERROR_TYPE,206,"%s - ReadFile() failed.",pcHTMName,pcHTMName,NULL,0); 
  return;
 };
 CloseHandle((HANDLE) HTMHandle);
//
// Mark this exact moment (and initialize the pseudo-random generator pseudo-randomly!)
//
 GetLocalTime(&STNow);
 SystemTime2TM(STNow,&TMNow);
 srand((unsigned) STNow.wMilliseconds);
 pcHTMLEx=strstr(pcHTM,"<!---HTMLEx");	// case sensitive, comment by HTML conventions!
 if (pcHTMLEx=(pcHTMLEx==NULL) ? strstr(pcHTM,"<!HTMLEx") : pcHTMLEx) // case sensitive!
 {
  char cControlTag[256];
  char *pcControlTag=cControlTag;
  while ((*pcHTMLEx) && (*pcHTMLEx != '>') && ((pcControlTag-cControlTag)<sizeof(cControlTag)))
  {
   *pcControlTag=*pcHTMLEx;
   pcControlTag++;*pcControlTag='\0';
   *pcHTMLEx=' ';pcHTMLEx++;
  };
  if (*pcHTMLEx == '>') *pcHTMLEx=' '; 
  if (ECD.bIsFilter) // Filter, check for <!HTMLEx expand execute ...>
  {
   if (dwRegExpandByDefault ? (strstr(cControlTag," noexpand") != NULL) : (strstr(cControlTag," expand") == NULL))
   {
    GlobalFree(pcHTM);
    return;  
   };
   bExecute=(dwRegExecuteByDefault ? (strstr(cControlTag," noexecute")==NULL) : (strstr(cControlTag," execute")!=NULL));
  }
  else 
  {
   bExecute=TRUE;                                      // Extension, expand and execute always
   if (strstr(cControlTag," noini")) bCreateINI=FALSE; // Disable INIs only for .ZTMs!!!
  };
  if (strstr(cControlTag," nolog")) dwLogThisOne=0;       // case sensitive!
  if (strstr(cControlTag," justcount")) dwJustCount=1;    // case sensitive!
  if (strstr(cControlTag," noctags")) bExpandCTags=FALSE; // case sensitive!
  if (strstr(cControlTag," nortags")) bExpandRTags=FALSE; // case sensitive!
  if (strstr(cControlTag," ignorecache"))
  {
////////////////////////////////////////////////////////////////
// PORTABILITY WARNING: SHOULD BE REWRITTEN FOR BIG-ENDIAN CPUs?
////////////////////////////////////////////////////////////////
// 64 bit integers are added here, where the time to which
//  is added is stored as two 32 bit words in a (FILETIME)
//                       structure.
//////////////////////////////////////////////////////////////// 
   GetSystemTime(&STGMT);
   SystemTime2TM(STGMT,&TMGMT);
   strcpy(SrvReplyHead.cHName[1],"Last-Modified:");
   strftime(SrvReplyHead.cHValue[1],sizeof(SrvReplyHead.cHValue[1]),"%a, %d %b %Y %H:%M:%S GMT",&TMGMT);
   SrvReplyHead.bAdd[1]=FALSE;

   __int64  FTGMT,FTAnHourAgo,FTOff=(24*60*60);
   SystemTimeToFileTime(&STGMT,(PFILETIME) &FTGMT);
   FTOff*=10000000; // 100ns intervals in one little hour!
   FTAnHourAgo=FTGMT-FTOff; 
   FileTimeToSystemTime((PFILETIME) &FTAnHourAgo,&STAnHourAgo);
   SystemTime2TM(STAnHourAgo,&TMAnHourAgo);
   strcpy(SrvReplyHead.cHName[2],"Expires:");
   strftime(SrvReplyHead.cHValue[2],sizeof(SrvReplyHead.cHValue[1]),"%a, %d %b %Y %H:%M:%S GMT",&TMAnHourAgo);
   SrvReplyHead.bAdd[2]=FALSE;

   SrvReplyHead.iLast=2;
  };
 }
 else  // <!HTMLEx not found!
 {
  if (!dwRegExpandByDefault && ECD.bIsFilter)
  {
   GlobalFree(pcHTM);
   return;  
  }
 };
 i=1;
 TagConst(cTagName[i],cTagValue[i],"%%","%");i++;
 TagConst(cTagName[i],cTagValue[i],"%HTMLExAuthor%",pcHTMLExAuthor);i++;
 TagConst(cTagName[i],cTagValue[i],"%HTMLExVer%",pcHTMLExVer);i++;
 TagConst(cTagName[i],cTagValue[i],"%HTMLExLogo%",pcHTMLExLogo);i++;
 TagConst(cTagName[i],cTagValue[i],"%AdminEmail%",cRegAdminEmail);i++;
 TagConst(cTagName[i],cTagValue[i],"%AdminName%" ,cRegAdminName);i++;
 TagConst(cTagName[i],cTagValue[i],"%HOSTNAME%"  ,cHostname);i++;
 TagServVar(cTagName[i],cTagValue[i],"%REMOTE_ADDR%",pECB,pfc,"REMOTE_ADDR","<Unknown IP address>");strcpy(cRemoteIP,cTagValue[i]);i++;
 TagServVar(cTagName[i],cTagValue[i],"%HTTP_REFERER%",pECB,pfc,"HTTP_REFERER","<Unknown referer>");strcpy(cReferer,cTagValue[i]);i++;
 TagServVar(cTagName[i],cTagValue[i],"%HTTP_COOKIE%",pECB,pfc,"HTTP_COOKIE","");strcpy(cCookie,cTagValue[i]);i++;
//
// INI File - [Config] Section
//
 GetPrivateProfileString("Config","cTagStart",cRegTagStart,cTagStart,sizeof(cTagStart),pcININame);dwLenTS=strlen(cTagStart);
 GetPrivateProfileString("Config","cTagEnd"  ,cRegTagEnd  ,cTagEnd  ,sizeof(cTagEnd)  ,pcININame);dwLenTE=strlen(cTagEnd);
 iGenerateErrFiles=GetPrivateProfileInt("Config","iGenerateErrFiles",(int) dwRegGenerateErrFiles,pcININame);
 pcERR=(iGenerateErrFiles ? pcERRName : NULL);
//
// INI File - [Log] Section
//
 iMaxLogFileSize=GetPrivateProfileInt("Log","iMaxLogFileSize",(int) dwRegMaxLogFileSize,pcININame);
 if (iMaxLogFileSize<0) iMaxLogFileSize=0; 
 GetPrivateProfileString("Log","cDateFormat",cRegDateFormat,cDateFormat,sizeof(cDateFormat),pcININame);
 GetPrivateProfileString("Log","cTimeFormat",cRegTimeFormat,cTimeFormat,sizeof(cTimeFormat),pcININame);
 GetPrivateProfileString("Log","cIgnoreAddress",cRegIgnoreAddress,cIgnoreAddress,sizeof(cIgnoreAddress),pcININame);
 strcpy(cAsteriskedIP,cRemoteIP);
 if (stristr(cIgnoreAddress,"*.*.*.*")) bCountThisHit=FALSE; // *.*.*.* disables counting for all IP addresses!
 for(int iAsterisk=3;bCountThisHit && (iAsterisk>0);iAsterisk--)
 {
  char *pcDotPos=cAsteriskedIP;
  for(int iDP=1;iDP<=iAsterisk;iDP++) pcDotPos=strchr(pcDotPos+1,'.');
  strcpy(pcDotPos,pcAsteriskTrail[iAsterisk]);
  if (strstr(cIgnoreAddress,cAsteriskedIP)) bCountThisHit=FALSE;
 };
 strftime(cTodaysDate,sizeof(cTodaysDate),cDateFormat,&TMNow);
 GetPrivateProfileString("Log","cStartDate",cTodaysDate,cStartDate,sizeof(cStartDate),pcININame);
 TagConst(cTagName[i],cTagValue[i],"%LogStartDate%",cStartDate);i++;
 if (bCountThisHit && bNotErrorReply && bCreateINI) WritePrivateProfileString("Log","cStartDate",cStartDate,pcININame);
//
// Do everything needed for counting 
//
 iDigitCnt=GetPrivateProfileInt("Log","iDigitCnt",(int) dwRegDigitCnt,pcININame);
 strcpy(cBuff,"cX");           // Keys:     c0=... , c1=... , ... , c9=...
 for(j=0;j<10;j++)
 {
  cBuff[1]=(char) 48+j;
  GetPrivateProfileString(KEY_DIGITS,cBuff,cRegDigitImg[j],cDigitImg[j],sizeof(cDigitImg[0]),pcININame);
  if (strlen(cDigitImg[j])==0) strcpy(cDigitImg[j],pcDefaultDigitImg[j]);
 };
 sprintf(cTextCnt,"%i",iCnt=GetPrivateProfileInt("LastAccess","iCnt",0,pcININame));
 TagConst(cTagName[i],cTagValue[i],"%LastCnt%",cTextCnt);i++;
 TagConst(cTagName[i],cTagValue[i],"%LastGraphCnt%"," ");GraphCnt(iCnt,iDigitCnt,&cDigitImg[0][0],cTagValue[i]);i++;
 if (bCountThisHit) iCnt++;
 sprintf(cTextCnt,"%i",iCnt);TagConst(cTagName[i],cTagValue[i],"%CurrentCnt%",cTextCnt);i++;
 TagConst(cTagName[i],cTagValue[i],"%CurrentGraphCnt%"," ");GraphCnt(iCnt,iDigitCnt,&cDigitImg[0][0],cTagValue[i]);i++;
//
// A feature, not a bug: Random tags and IP address resolving still operational!
//
 TagServVar(cTagName[i],cTagValue[i],"%REMOTE_HOST%",pECB,pfc,"REMOTE_HOST","<Unknown host>");i++;
 strcpy(cHost,cTagValue[i-1]);

//
// And everthing else if not specifically forbidden
//
 if (!dwJustCount)
 {
  TagServVar(cTagName[i],cTagValue[i],"%SERVER_SOFTWARE%",pECB,pfc,"SERVER_SOFTWARE","<Unknown server software>");i++;
  TagServVar(cTagName[i],cTagValue[i],"%CONTENT_LENGTH%",pECB,pfc,"CONTENT_LENGTH"," ");i++;
  TagServVar(cTagName[i],cTagValue[i],"%CONTENT_TYPE%",pECB,pfc,"CONTENT_TYPE"," ");i++;
  TagServVar(cTagName[i],cTagValue[i],"%GATEWAY_INTERFACE%",pECB,pfc,"GATEWAY_INTERFACE"," ");i++;
  TagServVar(cTagName[i],cTagValue[i],"%PATH_INFO%",pECB,pfc,"PATH_INFO"," ");i++;
  TagServVar(cTagName[i],cTagValue[i],"%PATH_TRANSLATED%",pECB,pfc,"PATH_TRANSLATED"," ");i++;
  TagServVar(cTagName[i],cTagValue[i],"%QUERY_STRING%",pECB,pfc,"QUERY_STRING"," ");i++;
  TagServVar(cTagName[i],cTagValue[i],"%REMOTE_USER%",pECB,pfc,"REMOTE_USER","<Anonymous>");i++;
  TagServVar(cTagName[i],cTagValue[i],"%REQUEST_METHOD%",pECB,pfc,"REQUEST_METHOD"," ");i++;
  TagServVar(cTagName[i],cTagValue[i],"%SCRIPT_NAME%",pECB,pfc,"SCRIPT_NAME"," ");i++;
  TagServVar(cTagName[i],cTagValue[i],"%SERVER_NAME%",pECB,pfc,"SERVER_NAME"," ");i++;
  TagServVar(cTagName[i],cTagValue[i],"%SERVER_PORT%",pECB,pfc,"SERVER_PORT"," ");i++;
  TagServVar(cTagName[i],cTagValue[i],"%SERVER_PROTOCOL%",pECB,pfc,"SERVER_PROTOCOL"," ");i++;
  TagServVar(cTagName[i],cTagValue[i],"%HTTP_ACCEPT%",pECB,pfc,"HTTP_ACCEPT"," ");i++;
  GetPrivateProfileString("LastAccess","cDate","<Unknown date>",cDate,sizeof(cDate),pcININame);TagConst(cTagName[i],cTagValue[i],"%LastAccessDate%",cDate);i++;
  GetPrivateProfileString("LastAccess","cTime","<Unknown time>",cTime,sizeof(cTime),pcININame);TagConst(cTagName[i],cTagValue[i],"%LastAccessTime%",cTime);i++;
  GetPrivateProfileString("LastAccess","cIP","<Unknown IP address>",cIP,sizeof(cIP),pcININame);TagConst(cTagName[i],cTagValue[i],"%LastClientIPAddress%",cIP);i++;
  GetPrivateProfileString("LastAccess","cHost","<Unknown host>",cLastHost,sizeof(cLastHost),pcININame);TagConst(cTagName[i],cTagValue[i],"%LastClientHost%",cLastHost);i++;
  GetPrivateProfileString("LastAccess","cBrowser","<Unknown browser>",cBrowser,sizeof(cBrowser),pcININame);TagConst(cTagName[i],cTagValue[i],"%LastClientBrowser%",cBrowser);i++;
//
// %Current~% Tags
//  
  FileTimeToLocalFileTime(&HTMInfo.ftLastWriteTime,&FTLocal);
  FileTimeToSystemTime(&FTLocal,&STFileLocal);
  SystemTime2TM(STFileLocal,&TMFileLocal);
  strftime(cDate,sizeof(cDate),cDateFormat,&TMFileLocal);TagConst(cTagName[i],cTagValue[i],"%LastFileUpdateDate%",cDate);i++;
  strftime(cTime,sizeof(cTime),cTimeFormat,&TMFileLocal);TagConst(cTagName[i],cTagValue[i],"%LastFileUpdateTime%",cTime);i++;
  strftime(cDate,sizeof(cDate),cDateFormat,&TMNow);TagConst(cTagName[i],cTagValue[i],"%CurrentAccessDate%",cDate);i++;
  strftime(cTime,sizeof(cTime),cTimeFormat,&TMNow);TagConst(cTagName[i],cTagValue[i],"%CurrentAccessTime%",cTime);i++;
  strcpy(cIP,cRemoteIP);TagConst(cTagName[i],cTagValue[i],"%CurrentClientIPAddress%",cIP);i++;
  TagServVar(cTagName[i],cTagValue[i],"%HTTP_USER_AGENT%",pECB,pfc,"HTTP_USER_AGENT","<Unknown browser>");i++;
  strcpy(cBrowser,cTagValue[i-1]);TagConst(cTagName[i],cTagValue[i],"%CurrentClientBrowser%",cBrowser);i++;
 } // if(!dwJustCount)
// pfc->pFilterContext=bExecute ? (GlobalAlloc(GMEM_PTR,dwHTMExSize)) : (pfc->AllocMem)(pfc,dwHTMExSize,0);
 hExpanded=bExecute ? (GlobalAlloc(GPTR,dwHTMExSize)) : (HGLOBAL)(pfc->AllocMem)(pfc,dwHTMExSize,0);
 if (hExpanded == NULL)
 {
  GlobalFree(pcHTM);
  ltoa(dwHTMExSize,cBuff,10);
  Report(EVENTLOG_ERROR_TYPE,207,bExecute ? "GlobalAlloc(,%s,) failed." : "AllocMem(,%s,) failed.",cBuff,pcHTMName,pcERR,iCnt);
  return;
 };
//
// Now we can finaly do it!
//
 pcS=pcHTM;
 pcD=(char*) hExpanded;
 BOOL bCont=TRUE;
// *pcD='\0';
 if (!bExecute) memset(pcD,'\0',dwHTMExSize);
 while (bCont)
 {
  pcF1=strstr(pcS,cTagStart);
  if (pcF1)
  {
   iCat=pcF1-pcS;
   strncat(pcD,pcS,iCat);
   pcF2=strstr(pcF1+dwLenTS,cTagEnd);
   if (pcF2)
   {
   	if ((pcF2-pcF1)>=(int)(MAX_TAGNAME+dwLenTS)) // Too far, don't even try...
	{
	 strncat(pcD,pcF1,pcF2-pcF1/*+dwLenTE*/);
	 pcS=pcF2;        // Maybe it's the start of a new one... 
//	 pcS=pcF2+dwLenTE;
	 bCont=TRUE;
	}
	else
	{
     int iRawTagLen=pcF2-(pcF1+dwLenTS);
	 strncpy(cRawTag,pcF1+dwLenTS,iRawTagLen);
	 cRawTag[iRawTagLen]='\0';
	 memset(cTagN,'\0',sizeof(cTagN));
	 sprintf(cTagN,"%%%s%%",cRawTag);
  	 bTagFound=FALSE;
 	 for(j=i-1;!bTagFound && (j>0);j--)
	 {
	  if (strcmp(cTagName[j],cTagN)==0)
	  {
	   strcat(pcD,cTagValue[j]);
	   bTagFound=TRUE;
	  };
	 }; // for(j...)
     if (!bTagFound && (strnicmp(cTagN,"%HTTP_",6)==0))
     {
      TagServVar(cTagName[i],cTagValue[i],cTagN,pECB,pfc,cRawTag," ");i++;
      strcat(pcD,cTagValue[j]);
      bTagFound=TRUE;
     };
	 if (!bTagFound && (strcmp(cTagN,"%CurrentClientHost%")==0)) 
     {
      ResolveIPAddress(cHost,pcHTMName,pcERRName,iCnt);
	  TagConst(cTagName[i],cTagValue[i],"%CurrentClientHost%",cHost);i++;
	  strcat(pcD,cHost);
	  bTagFound=TRUE;
     };
	 if(!bTagFound && bExpandCTags && (i<MAX_TAG)) // not found and constant tags enabled
	 {
	  bTagFound=GetValueFromTagKeyInINI(pcININame,cTagN,cTagV) ? TRUE : GetValueFromTagKeyInReg(hKeyTags,cTagN,cTagV);
	  if (bTagFound)
	  {
	   TagConst(cTagName[i],cTagValue[i],cTagN,cTagV);i++;
   	   strcat(pcD,cTagV);
	  };
	 };
	 if(!bTagFound && bExpandRTags) // not found and random tags enabled
	 {
	  bTagFound=PickValueFromRandomKeyInINI(pcININame,cTagN,cTagV) ? TRUE : PickValueFromRandomKeyInReg(hKeyHTMLEx,cTagN,cTagV);
	  if (bTagFound) strcat(pcD,cTagV);
	 };
	 if (!bTagFound)		// Could not find it anywhere!
	 {
	  sprintf(cTagV,"%s%s%s",cTagStart,cRawTag,cTagEnd);
 	  sprintf(cBuff,"Tag %s is unknown",cTagV);
	  strcat(pcD,cTagV); 
      Report(EVENTLOG_WARNING_TYPE,106,"%s",cBuff,pcHTMName,pcERR,iCnt);
	 };
     bCont=TRUE;
	 pcS=pcF2+dwLenTE;
    }
   }
   else	 // *pcF2==NULL
   {
    bCont=FALSE;
	iCat=strlen(pcF1);
    strncat(pcD,pcF1,iCat);
   }
  }
  else   // *pcF1==NULL
  {
   bCont=FALSE;
   iCat=strlen(pcS);
   strncat(pcD,pcS,iCat);
  };
 }; // while(bCont)

 GlobalFree(pcHTM);

 if (bCountThisHit && bNotErrorReply && bCreateINI)
 { 
//
// Update the INI File ([LastAccess] section)
//
  WritePrivateProfileString("LastAccess","cDate",cDate,pcININame);
  WritePrivateProfileString("LastAccess","cTime",cTime,pcININame);
  WritePrivateProfileString("LastAccess","cIP",cIP,pcININame);
  WritePrivateProfileString("LastAccess","cHost",cHost,pcININame);
  WritePrivateProfileString("LastAccess","cBrowser",cBrowser,pcININame);
  WritePrivateProfileString("LastAccess","cReferer",cReferer,pcININame);
  WritePrivateProfileString("LastAccess","iCnt",cTextCnt,pcININame);
 };  
//
// Write an entry into LOG File. WriteProfileString cannot handle >64k, so we have to emulate it here
//
 if (/*bExecute | */(dwRegGenerateLogs && dwLogThisOne && bNotErrorReply))
 {
//  LOGHandle=OpenFile(pcLOGName,&OFS,OF_WRITE | OF_SHARE_DENY_NONE);
  LOGHandle=CreateFile(pcLOGName,GENERIC_WRITE,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
  if (LOGHandle==INVALID_HANDLE_VALUE) Report(EVENTLOG_ERROR_TYPE,208,"Unable to open or create '%s' log file",pcLOGName,pcHTMName,pcERR,iCnt);
  else
  {
   if (iMaxLogFileSize)
   {
    GetFileInformationByHandle(LOGHandle,&LOGInfo);
    if (bZapLOGFile=((LOGInfo.nFileSizeHigh>0) || (LOGInfo.nFileSizeLow>(DWORD) 1024*iMaxLogFileSize)))
    {
     sprintf(cBuff,"The Log file %s has grown over the allowed size of %i kilobyte(s), and has been zapped",pcLOGName,iMaxLogFileSize);
     Report(EVENTLOG_INFORMATION_TYPE,9,"%s",cBuff,pcHTMName,pcERR,iCnt);
    }
   }
   else bZapLOGFile=FALSE; // ==0, grow ad infinitum!
   SetFilePointer(LOGHandle,0,NULL,bZapLOGFile ? FILE_BEGIN : FILE_END);
   if (bCountThisHit)
   {
    WriteFile(LOGHandle,cBuff,sprintf(cBuff,"\r\n[%i]\r\n",iCnt),&dwFileRW,NULL);
    WriteFile(LOGHandle,cBuff,sprintf(cBuff,"cDate=%s\r\n",cDate),&dwFileRW,NULL);
    WriteFile(LOGHandle,cBuff,sprintf(cBuff,"cTime=%s\r\n",cTime),&dwFileRW,NULL);
    WriteFile(LOGHandle,cBuff,sprintf(cBuff,"cIP=%s\r\n",cIP),&dwFileRW,NULL);
    WriteFile(LOGHandle,cBuff,sprintf(cBuff,"cHost=%s\r\n",cHost),&dwFileRW,NULL);
    WriteFile(LOGHandle,cBuff,sprintf(cBuff,"cBrowser=%s\r\n",cBrowser),&dwFileRW,NULL);
    WriteFile(LOGHandle,cBuff,sprintf(cBuff,"cReferer=%s\r\n",cReferer),&dwFileRW,NULL);
    if (cCookie[0]) WriteFile(LOGHandle,cBuff,sprintf(cBuff,"cCookie=%s\r\n",cCookie),&dwFileRW,NULL);
   };
  };
 }
 else LOGHandle=INVALID_HANDLE_VALUE; // if (dwRegGenerateLogs etc...)
 if (bExecute)
 {
  BOOL bThereIsMemory=TRUE; 
  STACKENTRY ExtVar;
  GetPrivateProfileString("Exec","cCmdStart",cRegCmdStart,ECD.cCmdStart,sizeof(ECD.cCmdStart),pcININame);ECD.dwLenCS=strlen(ECD.cCmdStart);
  GetPrivateProfileString("Exec","cCmdEnd"  ,cRegCmdEnd  ,ECD.cCmdEnd  ,sizeof(ECD.cCmdEnd)  ,pcININame);ECD.dwLenCE=strlen(ECD.cCmdEnd);
  ECD.dwMaxLoop=GetPrivateProfileInt("Exec","iMaxLoop",(int) dwRegMaxLoop,pcININame);
  ECD.pcScriptStart=(char*) hExpanded;
  ECD.pcScriptEnd=ECD.pcScriptStart+strlen(ECD.pcScriptStart);
  sprintf(ECD.cSubStart,"%sSUBROUTINE",ECD.cCmdStart);
  sprintf(ECD.cMaxLoop,"%i",ECD.dwMaxLoop);
  DWORD dwFinalSize=0;
  MBlock.iLast=0;
  VBlock.iLast=-1;
  AddVBlockEntry("<DEF>","NULL",vNothing,&VBlock,pcHTM,pcERR,iCnt); // Located at index 0
//  Read-only variables, will invoke GetSystemTime() API function!
  for (i=0;i<=9;i++) AddVBlockEntry("<SYS>",(char*) GSTVarName[i],v0,&VBlock,pcHTM,pcERR,iCnt);        
//
  ExtVar.Type=ST_STRING;
  strcpy(ExtVar.cVal,cHostname);
  ExtVar.dwVal=0;
  ExtVar.fpVal=0;
  AddVBlockEntry("<DEF>","HOSTNAME",ExtVar,&VBlock,pcHTM,pcERR,iCnt);
  AddVBlockEntry("<DEF>","FALSE",v0,&VBlock,pcHTM,pcERR,iCnt);      // Zero
  AddVBlockEntry("<DEF>","TRUE" ,v1,&VBlock,pcHTM,pcERR,iCnt);		// One
  AddVBlockEntry("<DEF>","INFORMATION",v0,&VBlock,pcHTM,pcERR,iCnt);//
  AddVBlockEntry("<DEF>","WARNING"    ,v1,&VBlock,pcHTM,pcERR,iCnt);// Convenient for use as WRITELOG param #1
  AddVBlockEntry("<DEF>","ERROR"      ,v2,&VBlock,pcHTM,pcERR,iCnt);//
  if (!ECD.bIsFilter) // Define GET and POST variables only if it was called as an extension!
  {
   ExtVar.Type=ST_STRING;
   ExtVar.dwVal=0;
   ExtVar.fpVal=0;
   if (pECB->lpszQueryString) strcpy(ExtVar.cVal,pECB->lpszQueryString); else strnset(ExtVar.cVal,'\0',sizeof(ExtVar.cVal));
   AddVBlockEntry("<DEF>","GET" ,ExtVar,&VBlock,pcHTM,pcERR,iCnt);// Data retrieved from client using the GET method
   if (pECB->cbTotalBytes)
   {
	strncpy(ExtVar.cVal,(char*) pECB->lpbData,pECB->cbAvailable);
	ExtVar.cVal[pECB->cbAvailable]='\0';
   }
   else ExtVar.cVal[0]='\0';
   AddVBlockEntry("<DEF>","POST",ExtVar,&VBlock,pcHTM,pcERR,iCnt);// Data retrieved from client using the POST method
  };
  if (hKeyVariables)						                // Pick-up predefined variables from registry
  {
   int   iX=0;
   DWORD dwRegVarType;
   while (RegEnumValue(hKeyVariables,iX,cVarName,&dwNameLen,NULL,&dwRegVarType,(BYTE*) cVarValue,&dwValueLen)==ERROR_SUCCESS)
   {
    ExtVar=vNothing;
    switch (dwRegVarType)
	{
	 case REG_DWORD:
	      ExtVar.Type=ST_INT; 
		  memcpy(&ExtVar.dwVal,cVarValue,4);
	      break;
	 case REG_EXPAND_SZ:
	      ExtVar.Type=ST_STRING;
		  ExpandEnvironmentStrings(cVarValue,ExtVar.cVal,sizeof(cVarValue));
	      break;
	 case REG_MULTI_SZ:
	      GetRidOfNullChars(cVarValue,'\n');
	 case REG_SZ:
	      ExtVar.Type=ST_STRING;
		  strcpy(ExtVar.cVal,cVarValue);
	      break;
	};
	if (ExtVar.Type != ST_EMPTY) AddVBlockEntry("<REG>",cVarName,ExtVar,&VBlock,pcHTMName,pcERR,iCnt);
    dwNameLen=sizeof(cVarName);
    dwValueLen=sizeof(cVarValue);
	iX++;
   };
  };
  GetPrivateProfileString(KEY_VARS,NULL,pcTwoZeroes,cINIVarBuff,sizeof(cINIVarBuff),pcININame);
  if (cINIVarBuff[0])
  {
   char* pcVarBuff=cINIVarBuff;
   while (*pcVarBuff)
   {
    strcpy(cVarName,pcVarBuff);
    GetPrivateProfileString(KEY_VARS,cVarName,pcTwoZeroes,cVarValue,sizeof(cVarValue),pcININame);
	if (cVarValue[0])
	{
     ExtVar=vNothing;
	 switch (toupper(cVarName[0]))
	 {
      case 'I':
 	       ExtVar.Type=ST_INT;
		   ExtVar.dwVal=strtoul(cVarValue,NULL,0);
	       break;
      case 'F':
 	       ExtVar.Type=ST_REAL;
		   ExtVar.fpVal=atof(cVarValue);
	       break;
	  default:
 	       ExtVar.Type=ST_STRING;
		   strcpy(ExtVar.cVal,cVarValue);
	       break;
	 };
	};
	if (ExtVar.Type != ST_EMPTY) AddVBlockEntry("<INI>",cVarName,ExtVar,&VBlock,pcHTMName,pcERR,iCnt);
	pcVarBuff=strchr(pcVarBuff,'\0')+1;
   };  // while
  };
  HTMLExecute(ECD.pcScriptStart,ECD.pcScriptEnd,ECD,&MBlock,&VBlock,LOGHandle,&bThereIsMemory,pcHTMName,pcERR,iCnt,TRUE,&SrvReplyHead);
  if (SrvReplyHead.iLast) // Any headers added?
  {						// Yup, put them into temporary file...
   char cTempFileName[128],cEntryName[8],cHeaderNameAndValue[MAX_HNAME+MAX_HVALUE];
   
   sprintf(cTempFileName,"%s\\%08X.TMP",cRegTempDir,ECD.bIsFilter ? (DWORD) pfc : (DWORD) pECB);
   strcpy(cEntryName,"cA");  
   for (int i=1;i<=SrvReplyHead.iLast;i++)
   {
    cEntryName[1]=(char) 64+i; // cA, cB, cC, etc...
    sprintf(cHeaderNameAndValue,"%s %s",SrvReplyHead.cHName[i],SrvReplyHead.cHValue[i]);
    WritePrivateProfileString(SrvReplyHead.bAdd[i] ? "AddHeader" : "SetHeader",cEntryName,cHeaderNameAndValue,cTempFileName);
   };
  }
  for (i=1;i<=VBlock.iLast;i++) GlobalFree(VBlock.Var[i]);
  GlobalFree(hExpanded);
  if (MBlock.iLast) // At least one MBlock
  {
   for (i=1;i<=MBlock.iLast;i++) dwFinalSize+=GlobalSize(MBlock.MB[i]);
   if (pfc->pFilterContext=(pfc->AllocMem)(pfc,dwFinalSize+1,0))
   {
    strcpy((char*) pfc->pFilterContext,(char*) MBlock.MB[1]);
    for (i=2;i<=MBlock.iLast;i++) strcat((char*) pfc->pFilterContext,(char*) MBlock.MB[i]);
   }
   else
   {
    ltoa(dwFinalSize,cBuff,10);
    Report(EVENTLOG_ERROR_TYPE,298,"AllocMem(pfc,%s,0) failed - HTMLExecute output lost",cBuff,pcHTMName,pcERR,iCnt);
    ErrorPage(pfc,298,"Out of memory - HTMLExecute output lost");
   };
   for (i=1;i<=MBlock.iLast;i++) GlobalFree(MBlock.MB[i]);
  }
  else			  // No MBlock at all - impossible?!
  {
   Report(EVENTLOG_ERROR_TYPE,299,"HTMLExecute returned no memory block",NULL,pcHTMName,pcERR,iCnt);
   ErrorPage(pfc,299,"HTMLExecute returned no memory block");
  };
 }
 else pfc->pFilterContext=(void*) hExpanded;
 if (LOGHandle != INVALID_HANDLE_VALUE) 
 {
  if (bZapLOGFile) SetEndOfFile(LOGHandle);
  CloseHandle(LOGHandle);
 };
// Report(EVENTLOG_INFORMATION_TYPE,8,"%s",(char*) pfc->pFilterContext);
};

//////////////////////////////////////////////////////
//                DLL INITIALIZATION
//////////////////////////////////////////////////////
BOOL WINAPI DllMain(HINSTANCE hinstDll,DWORD dwReason,LPVOID lpvContext)
{
 HKEY  hKeyW3SVC,hKeyParams,hKeyDigits;
 DWORD dwValLen,dwDisposition;
 char  cRegSkipdir[MAX_SKIPDIR*256];
 int j;
 const char *pcDefaultDigitVal[]={"0","1","2","3","4","5","6","7","8","9"};

 if (dwReason==DLL_PROCESS_DETACH)
 {
#ifdef THUMBNAIL
  if (hInstImgDLL) FreeLibrary(hInstImgDLL);
#endif
#ifdef WINET
  if (hInstWinetDLL) FreeLibrary(hInstWinetDLL);
#endif
 };

 if (dwReason==DLL_PROCESS_ATTACH)
 {
  hEvent=RegisterEventSource(NULL,"HTMLEx");
#ifdef WINET
  hInstWinetDLL=LoadLibraryEx(WINETDLLNAME,NULL,0);
  if (hInstWinetDLL)
  {
   DWORD dwRW;
   fpInternetQueryOption=(int (__stdcall *)(void *,unsigned long,void *,unsigned long *))GetProcAddress(hInstWinetDLL,"InternetQueryOption");
   (*fpInternetQueryOption)(NULL,INTERNET_OPTION_VERSION,(void*) &WinetVersion,&dwRW);
   fpInternetOpenA=(void *(__stdcall *)(const char *,unsigned long,const char *,const char *,unsigned long))GetProcAddress(hInstWinetDLL,"InternetOpenA");
   fpInternetOpenUrlA=(void *(__stdcall *)(void *,const char *,const char *,unsigned long,unsigned long,unsigned long)) GetProcAddress(hInstWinetDLL,"InternetOpenUrlA");
   fpInternetReadFile=(int (__stdcall *)(void *,void *,unsigned long,unsigned long *)) GetProcAddress(hInstWinetDLL,"InternetReadFile");
   fpInternetCloseHandle=(int (__stdcall *)(void *)) GetProcAddress(hInstWinetDLL,"InternetCloseHandle");
  };
#endif 
// Set default values
  if (gethostname(cHostname,sizeof(cHostname))) strcpy(cHostname,"localhost");
  strcpy(cRegS404Reply,pcEmpty);
  strcpy(cRegD404Reply,pcEmpty);
#ifdef THUMBNAIL
  strcpy(cRegThumbnailPath,pcEmpty);
#endif
  iSkipdirCount=0;
  strcpy(cRegTagStart,TAG_START);
  strcpy(cRegTagEnd,TAG_END);
  strcpy(cRegDefault_Load_File,DEFAULTLOADFILE);
  strcpy(cRegIgnoreAddress,IGNOREADDRESS);
  strcpy(cRegExpandableExtensions,EXPANDABLEEXTENSIONS);
  strcpy(cRegUnexpandableExtensions,UNEXPANDABLEEXTENSIONS);
  strcpy(cRegDateFormat,DATEFORMAT);
  strcpy(cRegTimeFormat,TIMEFORMAT);
  for(j=0;j<10;j++) strcpy(cRegDigitImg[j],pcDefaultDigitImg[j]);
  dwRegDigitCnt=DIGITCNT;
  dwRegMaxLogFileSize=MAXLOGFILESIZE;
  dwRegExpandByDefault=EXPANDBYDEFAULT;
  dwRegExpandDefaultPages=EXPANDDEFAULTPAGES;
  dwRegExpandOtherPages=EXPANDOTHERPAGES;
  dwRegGenerateLogs=GENERATELOGS;
  dwRegGenerateErrFiles=GENERATEERRFILES;
  dwRegLog404Errors=LOG404ERRORS;
//
  strcpy(cRegCmdStart,CMD_START);
  strcpy(cRegCmdEnd,CMD_END);
  dwRegExecuteByDefault=EXECUTEBYDEFAULT;
  dwRegMaxLoop=MAXLOOP;
// Override with registry
  hKeyHTMLEx=NULL;
  hKeyTags=NULL;
  hKeyVariables=NULL;
  hKeySubroutines=NULL;
  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,KEY_W3SVC,0,KEY_READ,&hKeyW3SVC)==ERROR_SUCCESS)
  {
   if (RegOpenKeyEx(hKeyW3SVC,KEY_PARAMS,0,KEY_READ,&hKeyParams)==ERROR_SUCCESS) 
   {
    dwValLen=sizeof(cRegDefault_Load_File);if (RegQueryValueEx(hKeyParams,"Default Load File",NULL,NULL,(LPBYTE) cRegDefault_Load_File,&dwValLen) != ERROR_SUCCESS) strcpy(cRegDefault_Load_File,DEFAULTLOADFILE);
    dwValLen=sizeof(cRegAdminEmail);if (RegQueryValueEx(hKeyParams,"AdminEmail",NULL,NULL,(LPBYTE) cRegAdminEmail,&dwValLen) != ERROR_SUCCESS) strcpy(cRegAdminEmail,"Admin@corp.com");
    dwValLen=sizeof(cRegAdminName); if (RegQueryValueEx(hKeyParams,"AdminName" ,NULL,NULL,(LPBYTE) cRegAdminName, &dwValLen) != ERROR_SUCCESS) strcpy(cRegAdminName ,"Administrator");
#ifdef THUMBNAIL
    HKEY hKeyVRoots;
    hInstImgDLL=NULL;
    if (RegOpenKeyEx(hKeyParams,KEY_VROOTS,0,KEY_READ,&hKeyVRoots)==ERROR_SUCCESS) 
    {
     char cTNailValueName[64];
     sprintf(cTNailValueName,"%s,",THUMBNAIL_VROOT); // Who knows what IIS puts after that comma!?
     dwValLen=sizeof(cRegThumbnailPath);
     if (RegQueryValueEx(hKeyVRoots,cTNailValueName,NULL,NULL,(LPBYTE) cRegThumbnailPath,&dwValLen) != ERROR_SUCCESS)
     {
      strcpy(cRegThumbnailPath,pcEmpty);
      Report(EVENTLOG_ERROR_TYPE,289,"Directory alias '%s' not defined - THUMBNAIL() will return empty string",THUMBNAIL_VROOT,NULL,NULL,0);
     }
     else
     {
      char *pcFirstComma=strchr(cRegThumbnailPath,',');
      if (pcFirstComma) *pcFirstComma='\0'; // just the path, no other params
//
// TODO: load the JPEG/GIF DLL routines!
//
      hInstImgDLL=LoadLibraryEx(IMGDLLNAME,NULL,0);
      if (/*(hInstImgDLL != HINSTANCE_ERROR) &&*/ (hInstImgDLL))
      {
      }
      else
      {
       Report(EVENTLOG_ERROR_TYPE,290,"'%s' not found or invalid - THUMBNAIL() will return empty string",IMGDLLNAME,NULL,NULL,0);
       hInstImgDLL=NULL;
      };
     };
     RegCloseKey(hKeyVRoots);
    }
    else Report(EVENTLOG_ERROR_TYPE,200,"RegOpenKeyEx(W3SVC\\Parameters\\Virtual Roots) failed - check your IIS installation",NULL,NULL,NULL,0);
#endif
	RegCloseKey(hKeyParams);
   }
   else Report(EVENTLOG_ERROR_TYPE,200,"RegOpenKeyEx(W3SVC\\Parameters) failed - check your IIS installation",NULL,NULL,NULL,0);

   if (RegCreateKeyEx(hKeyW3SVC,KEY_HTMLEX,0,NULL,REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&hKeyHTMLEx,&dwDisposition)==ERROR_SUCCESS)
   {
    if (RegCreateKeyEx(hKeyHTMLEx,KEY_SUBS,0,NULL,REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&hKeySubroutines,&dwDisposition) != ERROR_SUCCESS)
    {
//   Report(EVENTLOG_WARNING_TYPE,100,"RegCreateKeyEx(W3SVC\\HTMLEx\\Subroutines) failed - no subroutines in registry",NULL,NULL,NULL,0);
     hKeySubroutines=NULL;
    };
    if (RegCreateKeyEx(hKeyHTMLEx,KEY_VARS,0,NULL,REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&hKeyVariables,&dwDisposition) != ERROR_SUCCESS)
    {
//   Report(EVENTLOG_WARNING_TYPE,100,"RegCreateKeyEx(W3SVC\\HTMLEx\\Variables) failed - no variables in registry",NULL,NULL,NULL,0);
     hKeyVariables=NULL;
    };
    if (RegCreateKeyEx(hKeyHTMLEx,KEY_TAGS,0,NULL,REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&hKeyTags,&dwDisposition) != ERROR_SUCCESS)
    {
//   Report(EVENTLOG_WARNING_TYPE,100,"RegCreateKeyEx(W3SVC\\HTMLEx\\Tags) failed - no constant tags in registry",NULL,NULL,NULL,0);
     hKeyTags=NULL;
    };
    dwValLen=sizeof(cRegSkipdir);
    if (RegQueryValueEx(hKeyHTMLEx,"SkipDirectories",NULL,NULL,(LPBYTE) cRegSkipdir,&dwValLen) == ERROR_SUCCESS)
    {
     char *pcS=strupr(cRegSkipdir),*pcE;
	 do
	 {
	  if (iSkipdirCount<MAX_SKIPDIR)
	  {
	   strnset(cSkipdir[++iSkipdirCount],'\0',sizeof(cSkipdir[0]));
	   pcE=strchr(pcS,',');
	   if (pcE) 
	   {
	    strncpy(cSkipdir[iSkipdirCount],pcS,pcE-pcS);
	    pcS=pcE+1;
	   }
	   else strcpy(cSkipdir[iSkipdirCount],pcS);
//     Report(EVENTLOG_WARNING_TYPE,999,"All URLs from '%s' will be skipped",cSkipdir[iSkipdirCount]);
	  };
	 } while (pcE);
    };
    dwValLen=sizeof(cRegS404Reply);             if (RegQueryValueEx(hKeyHTMLEx,"S404Reply",NULL,NULL,(LPBYTE) cRegS404Reply,&dwValLen) != ERROR_SUCCESS) strcpy(cRegS404Reply,pcEmpty);
    dwValLen=sizeof(cRegD404Reply);             if (RegQueryValueEx(hKeyHTMLEx,"D404Reply",NULL,NULL,(LPBYTE) cRegD404Reply,&dwValLen) != ERROR_SUCCESS) strcpy(cRegD404Reply,pcEmpty);
    dwValLen=sizeof(cRegTempDir);               if (RegQueryValueEx(hKeyHTMLEx,"TempDir",NULL,NULL,(LPBYTE) cRegTempDir,&dwValLen) != ERROR_SUCCESS) GetTempPath(sizeof(cRegTempDir),cRegTempDir);
    dwValLen=sizeof(cRegTagStart);              if (RegQueryValueEx(hKeyHTMLEx,"TagStart",NULL,NULL,(LPBYTE) cRegTagStart,&dwValLen) != ERROR_SUCCESS) strcpy(cRegTagStart,TAG_START);
    dwValLen=sizeof(cRegTagEnd);                if (RegQueryValueEx(hKeyHTMLEx,"TagEnd",NULL,NULL,(LPBYTE) cRegTagEnd,&dwValLen) != ERROR_SUCCESS) strcpy(cRegTagStart,TAG_END);
    dwValLen=sizeof(cRegIgnoreAddress);         if (RegQueryValueEx(hKeyHTMLEx,"IgnoreAddress",NULL,NULL,(LPBYTE) cRegIgnoreAddress,&dwValLen) != ERROR_SUCCESS) strcpy(cRegIgnoreAddress,IGNOREADDRESS);
    dwValLen=sizeof(cRegExpandableExtensions);  if (RegQueryValueEx(hKeyHTMLEx,"ExpandableExtensions",NULL,NULL,(LPBYTE) cRegExpandableExtensions,&dwValLen) != ERROR_SUCCESS) strcpy(cRegExpandableExtensions,EXPANDABLEEXTENSIONS);
    dwValLen=sizeof(cRegUnexpandableExtensions);if (RegQueryValueEx(hKeyHTMLEx,"UnexpandableExtensions",NULL,NULL,(LPBYTE) cRegUnexpandableExtensions,&dwValLen) != ERROR_SUCCESS) strcpy(cRegUnexpandableExtensions,UNEXPANDABLEEXTENSIONS);
    dwValLen=sizeof(cRegDateFormat);            if (RegQueryValueEx(hKeyHTMLEx,"DateFormat",NULL,NULL,(LPBYTE) cRegDateFormat,&dwValLen) != ERROR_SUCCESS) strcpy(cRegDateFormat,DATEFORMAT);
    dwValLen=sizeof(cRegTimeFormat);            if (RegQueryValueEx(hKeyHTMLEx,"TimeFormat",NULL,NULL,(LPBYTE) cRegTimeFormat,&dwValLen) != ERROR_SUCCESS) strcpy(cRegTimeFormat,TIMEFORMAT);
    dwValLen=sizeof(dwRegDigitCnt);             if (RegQueryValueEx(hKeyHTMLEx,"DigitCnt",NULL,NULL,(LPBYTE) &dwRegDigitCnt,&dwValLen) != ERROR_SUCCESS) dwRegDigitCnt=DIGITCNT;
    if (RegCreateKeyEx(hKeyHTMLEx,KEY_DIGITS,0,NULL,REG_OPTION_NON_VOLATILE,KEY_READ,NULL,&hKeyDigits,&dwDisposition)==ERROR_SUCCESS)
    {
     for(j=0;j<10;j++)
     {
      dwValLen=sizeof(cRegDigitImg[0]);if (RegQueryValueEx(hKeyDigits,(char*) pcDefaultDigitVal[j],NULL,NULL,(LPBYTE) cRegDigitImg[j],&dwValLen) != ERROR_SUCCESS) strcpy(cRegDigitImg[j],pcDefaultDigitImg[j]);
     };
	 RegCloseKey(hKeyDigits);
    }
    else
    {
//   Report(EVENTLOG_WARNING_TYPE,101,"RegCreateKeyEx(W3SVC\\HTMLEx\\Digits) failed - using defaults",NULL,NULL,NULL,0);
    };
    dwValLen=sizeof(dwRegMaxLogFileSize);    if (RegQueryValueEx(hKeyHTMLEx,"MaxLogFileSize",NULL,NULL,(LPBYTE) &dwRegMaxLogFileSize,&dwValLen) != ERROR_SUCCESS) dwRegMaxLogFileSize=MAXLOGFILESIZE;
    dwValLen=sizeof(dwRegExpandByDefault);   if (RegQueryValueEx(hKeyHTMLEx,"ExpandByDefault",NULL,NULL,(LPBYTE) &dwRegExpandByDefault,&dwValLen) != ERROR_SUCCESS) dwRegExpandByDefault=EXPANDBYDEFAULT;
    dwValLen=sizeof(dwRegExpandDefaultPages);if (RegQueryValueEx(hKeyHTMLEx,"ExpandDefaultPages",NULL,NULL,(LPBYTE) &dwRegExpandDefaultPages,&dwValLen) != ERROR_SUCCESS) dwRegExpandDefaultPages=EXPANDDEFAULTPAGES;
    dwValLen=sizeof(dwRegExpandOtherPages);  if (RegQueryValueEx(hKeyHTMLEx,"ExpandOtherPages",NULL,NULL,(LPBYTE) &dwRegExpandOtherPages,&dwValLen) != ERROR_SUCCESS) dwRegExpandOtherPages=EXPANDOTHERPAGES;
    dwValLen=sizeof(dwRegGenerateLogs);      if (RegQueryValueEx(hKeyHTMLEx,"GenerateLogs",NULL,NULL,(LPBYTE) &dwRegGenerateLogs,&dwValLen) != ERROR_SUCCESS) dwRegGenerateLogs=GENERATELOGS;
    dwValLen=sizeof(dwRegGenerateErrFiles);  if (RegQueryValueEx(hKeyHTMLEx,"GenerateErrFiles",NULL,NULL,(LPBYTE) &dwRegGenerateErrFiles,&dwValLen) != ERROR_SUCCESS) dwRegGenerateErrFiles=GENERATEERRFILES;
    dwValLen=sizeof(dwRegLog404Errors);      if (RegQueryValueEx(hKeyHTMLEx,"Log404Errors",NULL,NULL,(LPBYTE) &dwRegLog404Errors,&dwValLen) != ERROR_SUCCESS) dwRegLog404Errors=GENERATEERRFILES;
//
    dwValLen=sizeof(cRegCmdStart);if (RegQueryValueEx(hKeyHTMLEx,"CmdStart",NULL,NULL,(LPBYTE) cRegCmdStart,&dwValLen) != ERROR_SUCCESS) strcpy(cRegCmdStart,CMD_START);
    dwValLen=sizeof(cRegCmdEnd)  ;if (RegQueryValueEx(hKeyHTMLEx,"CmdEnd"  ,NULL,NULL,(LPBYTE) cRegCmdEnd  ,&dwValLen) != ERROR_SUCCESS) strcpy(cRegCmdEnd  ,CMD_END);
    dwValLen=sizeof(dwRegExecuteByDefault);if (RegQueryValueEx(hKeyHTMLEx,"ExecuteByDefault",NULL,NULL,(LPBYTE) &dwRegExecuteByDefault,&dwValLen) != ERROR_SUCCESS) dwRegExecuteByDefault=EXECUTEBYDEFAULT;
    dwValLen=sizeof(dwRegMaxLoop)         ;if (RegQueryValueEx(hKeyHTMLEx,"MaxLoop",NULL,NULL,(LPBYTE) &dwRegMaxLoop,&dwValLen) != ERROR_SUCCESS) dwRegMaxLoop=MAXLOOP;
//  RegCloseKey(hKeyHTMLEx); DO NOT CLOSE IT - WILL READ RANDOM KEYS FROM HERE!!!
   }
   else
   {
    Report(EVENTLOG_WARNING_TYPE,102,"RegCreateKeyEx(W3SVC\\HTMLEx) failed - using defaults",NULL,NULL,NULL,0);
    hKeyHTMLEx=NULL;
   };
   RegCloseKey(hKeyW3SVC);
  }
  else Report(EVENTLOG_ERROR_TYPE,201,"RegOpenKeyEx(W3SVC) failed",NULL,NULL,NULL,0);
  strupr(cRegExpandableExtensions);   // Needed: comparison is case sensitive!
  strupr(cRegUnexpandableExtensions); // Needed: comparison is case sensitive!
  strupr(cRegDefault_Load_File);      // Needed: comparison is case sensitive!
 };
 return (TRUE);
};

//////////////////////////////////////////////////////
//	            IIS / HTMLXD ROUTINES
//////////////////////////////////////////////////////
BOOL WINAPI SetDebugMode(HANDLE hSE)
{
 bDebugMode=TRUE;
 hStdErr=hSE;
 return (TRUE);
};

BOOL WINAPI GetFilterVersion(HTTP_FILTER_VERSION *pVer)
{
 pVer->dwFlags = (SF_NOTIFY_NONSECURE_PORT     |
                  SF_NOTIFY_SECURE_PORT        | //ATTENTION: NEVER TRIED IT!
      		      SF_NOTIFY_URL_MAP            |
		          SF_NOTIFY_SEND_RAW_DATA      |
  	     	      SF_NOTIFY_ORDER_DEFAULT);
 pVer->dwFilterVersion = HTTP_FILTER_REVISION;
 strcpy(pVer->lpszFilterDesc,pcHTMLExDescF);
 Report(EVENTLOG_INFORMATION_TYPE,0,"GetFilterVersion: %s loaded succesfully",pVer->lpszFilterDesc,NULL,NULL,0);
 return TRUE;
}

DWORD WINAPI HttpFilterProc(HTTP_FILTER_CONTEXT *pfc,DWORD dwNotificationType,VOID *pvData)
{
#define MAX_SRVHDR 32
 char cTempFileName[128];

 switch (dwNotificationType)
 {
  case SF_NOTIFY_URL_MAP:
       {
	    char cHTMName[256],cININame[256],cLOGName[256],cERRName[256];

 	    pfc->pFilterContext = NULL; //Default is no messing around
 	    HTTP_FILTER_URL_MAP* pURLMap = (PHTTP_FILTER_URL_MAP) pvData;
	    char* pcPhysPath = pURLMap->pszPhysicalPath; 
		char* pcLastBackslashPos=strrchr(pcPhysPath,'\\');
		if (pcLastBackslashPos)
		{
		 strupr(pcPhysPath); // Needed: comparison is case sensitive!
	     char* pcLastDotPos=strrchr(pcLastBackslashPos,'.');
	     if (pcLastDotPos)
	     {		   
		  if(strstr(cRegExpandableExtensions,pcLastDotPos))
	      {
	       char* pcFirstDotPos=strchr(pcLastBackslashPos,'.');
		   if (pcFirstDotPos)
		   {
		    char cFirstExtension[16];
		    char* pcFirstExtension=strnset(cFirstExtension,'\0',16);
		    *pcFirstExtension++=*pcFirstDotPos++; // copy the dot!								 
		    while(isalnum(*pcFirstDotPos)) *pcFirstExtension++=*pcFirstDotPos++;
		    if(strstr(cRegUnexpandableExtensions,cFirstExtension))
			{
		     return SF_STATUS_REQ_NEXT_NOTIFICATION;;
			};
		   };
		   if (iSkipdirCount)
		   {
		    char cHTMDir[256];
			strncpy(cHTMDir,pcPhysPath,pcLastBackslashPos-pcPhysPath+1);
			cHTMDir[pcLastBackslashPos-pcPhysPath+1]='\0';
		    for(int i=iSkipdirCount;i>0;i--)
		    if(strstr(cHTMDir,cSkipdir[i]))
			{
		     return SF_STATUS_REQ_NEXT_NOTIFICATION;
			};
		   };
		   strcpy(cHTMName,pcPhysPath);
		   if (strstr(strupr(cHTMName),cRegDefault_Load_File)) // Is this a default page?
		   {  ///// DEFAULT FILE
		    if (dwRegExpandDefaultPages)
		    {
 		     strcpy(cININame,pcPhysPath);strcpy(strrchr(cININame,'.'),".ini");
		     strcpy(cLOGName,pcPhysPath);strcpy(strrchr(cLOGName,'.'),".log");
		     strcpy(cERRName,pcPhysPath);strcpy(strrchr(cERRName,'.'),".err");
		     HTMLExpand(NULL,pfc,cHTMName,cININame,cLOGName,cERRName);
             strcpy(pURLMap->pszPhysicalPath,cHTMName); // Let's fool the IIS...
		    }
		   }
		   else
		   {	 ///// ANY OTHER FILE
		    if (dwRegExpandOtherPages)
		    {
		     strcpy(cININame,pcPhysPath);strcpy(strrchr(cININame,'.'),".ini");
		     strcpy(cLOGName,pcPhysPath);strcpy(strrchr(cLOGName,'.'),".log");
		     strcpy(cERRName,pcPhysPath);strcpy(strrchr(cERRName,'.'),".err");
		     HTMLExpand(NULL,pfc,cHTMName,cININame,cLOGName,cERRName);
             strcpy(pURLMap->pszPhysicalPath,cHTMName);
		    }
		   };  // if (default/not default)
		  };   // if (expandable)
	     };    // if (pcDotPos)
		};     // if (pcLastBacklashPos)
	   };	   // SF_NOTIFY_URL_MAP
	   break;
  case SF_NOTIFY_SEND_RAW_DATA:
       {
        HTTP_FILTER_RAW_DATA* pRawData = (PHTTP_FILTER_RAW_DATA) pvData;
	    if (pfc->pFilterContext)                            //HYPERTEXT? 
	    {
	     DWORD dwExpandedLen=strlen((char*) pfc->pFilterContext);
         char *pcAllHdr=(char*) pRawData->pvInData;    
		 if (strstr(pcAllHdr,"HTTP/1.")) //HEADER
		 {
          int  iLastSrvHdr=0;
          char cSrvHdr[MAX_SRVHDR][256];
          char cLast='\n',*pcDest;

          while (*pcAllHdr && (iLastSrvHdr<MAX_SRVHDR))
          {
           if ((cLast=='\n') && (*pcAllHdr>=' '))
           {
            iLastSrvHdr++;
            pcDest=cSrvHdr[iLastSrvHdr];
		   };
           cLast=*pcAllHdr;
		   *pcDest++=*pcAllHdr++;
		   *pcDest='\0';
		  }; // while()
          if (TempFileExists(cTempFileName,(DWORD) pfc))
          {
           char cEntryName[8],cHeaderNameAndValue[MAX_HNAME+MAX_HVALUE];
           strcpy(cEntryName,"cA");  
           for (int i=1;i<=MAX_HBLOCK;i++)
           {
            cEntryName[1]=(char) 64+i; // cA, cB, cC, etc...
            GetPrivateProfileString("AddHeader",cEntryName,pcEmpty,cHeaderNameAndValue,sizeof(cHeaderNameAndValue),cTempFileName);
            if (cHeaderNameAndValue[0] && (iLastSrvHdr<MAX_SRVHDR))
            {
             iLastSrvHdr++;
             sprintf(cSrvHdr[iLastSrvHdr],"%s\r\n",cHeaderNameAndValue);
            }
            else
            {
             GetPrivateProfileString("SetHeader",cEntryName,pcEmpty,cHeaderNameAndValue,sizeof(cHeaderNameAndValue),cTempFileName);
             if (cHeaderNameAndValue[0] && (iLastSrvHdr<MAX_SRVHDR))
             {
              char cHeaderName[256];       
              BOOL bSet=FALSE;     
              char *pcColon=strchr(strcpy(cHeaderName,cHeaderNameAndValue),':');
              if (pcColon) *(pcColon+1)='\0'; 
              for (int j=1;(j<=iLastSrvHdr) && !bSet;j++)
              {
               if (strstr(cSrvHdr[j],cHeaderName))
               {
                sprintf(cSrvHdr[j],"%s\r\n",cHeaderNameAndValue);
				bSet=TRUE;
               };
 			  };
              if (!bSet && (iLastSrvHdr<MAX_SRVHDR))
              { 
               iLastSrvHdr++;
               sprintf(cSrvHdr[iLastSrvHdr],"%s\r\n",cHeaderNameAndValue);
              };
             };
            };
           }; // for()
           if (!bDebugMode) DeleteFile(cTempFileName);
		  }; // if (TempFile())
          for (int j=1;j<=iLastSrvHdr;j++)
          {
  		   if (strstr(cSrvHdr[j],"Content-Length:")) sprintf(cSrvHdr[j],"Content-Length: %i\r\n",dwExpandedLen);
           if (j==1) strcpy((char*) pRawData->pvInData,cSrvHdr[j]);
           else      strcat((char*) pRawData->pvInData,cSrvHdr[j]);
		  };
          strcat((char*) pRawData->pvInData,"\r\n");
		  pRawData->cbInData=(DWORD) strlen((char*) pRawData->pvInData);
		 }
		 else												 //BODY
		 {
//		  char cDebug[256];
//		  sprintf(cDebug,"%lx %li %li %li",pfc->pFilterContext,dwExpandedLen,pRawData->cbInData,pRawData->cbInBuffer);
//        Report(EVENTLOG_INFORMATION_TYPE,1,"Body - %s",cDebug);
		  pRawData->cbInBuffer=MAX_WRITEBUFF;
		  pRawData->pvInData=pfc->pFilterContext;
          if (dwExpandedLen<=MAX_WRITEBUFF)
		  {
	       pRawData->cbInData=dwExpandedLen;
		   pfc->pFilterContext=NULL;                                      //LAST CHUNK
		  }
		  else
		  {
	       pRawData->cbInData=MAX_WRITEBUFF;
		   pfc->pFilterContext=(char*) pfc->pFilterContext+MAX_WRITEBUFF; //NEXT CHUNK
		  }; // if()
		 };	 // if(HEADER)
		}    // if(pfc...) ie. HTMLEx processed file!
//		else Report(EVENTLOG_INFORMATION_TYPE,2,"pfc->pFilterContext=NULL\n%s",(char*) pRawData->pvInData);
       };
	   break;
  };
  return SF_STATUS_REQ_NEXT_NOTIFICATION;
}
/////////////////////////////////////////////
//           ISAPI EXTENSION CODE
/////////////////////////////////////////////
void CreateDebugPage(char *pcPage,EXTENSION_CONTROL_BLOCK *pECB,char *pcPageTitle,char *pcH2)
{
 char cTableRow[1024];

 sprintf(pcPage,"<html><head><title>%s</title></head>",pcPageTitle);	 
 strcat(pcPage,pcH2);
 strcat(pcPage,"<hr><table border=1>");
 sprintf(cTableRow,"<tr><th>dwVersion<td>%08x",pECB->dwVersion);strcat(pcPage,cTableRow);
 sprintf(cTableRow,"<tr><th>lpszMethod<td>%s",pECB->lpszMethod);strcat(pcPage,cTableRow);
 sprintf(cTableRow,"<tr><th>lpszQueryString<td>%s",pECB->lpszQueryString);strcat(pcPage,cTableRow);
 sprintf(cTableRow,"<tr><th>lpszPathInfo<td>%s",pECB->lpszPathInfo);strcat(pcPage,cTableRow);
 sprintf(cTableRow,"<tr><th>lpszPathTranslated<td>%s",pECB->lpszPathTranslated);strcat(pcPage,cTableRow);
 sprintf(cTableRow,"<tr><th>cbTotalBytes<td>%i",pECB->cbTotalBytes);strcat(pcPage,cTableRow);
 sprintf(cTableRow,"<tr><th>cbAvailable<td>%i",pECB->cbAvailable);strcat(pcPage,cTableRow);
 sprintf(cTableRow,"<tr><th>lpbData<td>%s",pECB->lpbData);strcat(pcPage,cTableRow);
 sprintf(cTableRow,"<tr><th>lpszContentType<td>%s",pECB->lpszContentType);strcat(pcPage,cTableRow);
 strcat(pcPage,"</table><hr>");
 strcat(pcPage,pcHTMLExVer);
 strcat(pcPage,"</body></html>");
};

//
// Emulate the AllocMem function in HTTP_FILTER_CONTEXT
//
void* WINAPI AllocMem(PHTTP_FILTER_CONTEXT pfc,DWORD cbSize,DWORD dwReserved)
{
 return (LPVOID) GlobalAlloc(GMEM_FIXED,cbSize);
};

BOOL WINAPI GetExtensionVersion(HSE_VERSION_INFO *pVer)
{
 pVer->dwExtensionVersion = MAKELONG(HSE_VERSION_MINOR,HSE_VERSION_MAJOR );
 lstrcpyn(pVer->lpszExtensionDesc,pcHTMLExDescE,HSE_MAX_EXT_DLL_NAME_LEN);
 return TRUE;
};

DWORD WINAPI HttpExtensionProc(EXTENSION_CONTROL_BLOCK *pECB)
{
 DWORD dwHeaderLen,dwContentLen;
 char cHTTPHeader[8192];
 char cZTMErrPage[4096];
 char cZTMName[256],cININame[256],cLOGName[256],cERRName[256],cTempFileName[128];
 HTTP_FILTER_CONTEXT ExtFC; // Nasty trick: emulate ISAPI filter from ISAPI extension!

 strcpy(cZTMName,pECB->lpszPathTranslated);
 strcpy(cININame,cZTMName);strcpy(strrchr(cININame,'.'),".ini");
 strcpy(cLOGName,cZTMName);strcpy(strrchr(cLOGName,'.'),".log");
 strcpy(cERRName,cZTMName);strcpy(strrchr(cERRName,'.'),".err");
 ExtFC.cbSize=sizeof(HTTP_FILTER_CONTEXT);
 ExtFC.Revision=HTTP_FILTER_REVISION;
 ExtFC.fIsSecurePort=FALSE;            
 ExtFC.pFilterContext=NULL;
 ExtFC.GetServerVariable=NULL; // Not needed, we will use the one from pECB
 ExtFC.AllocMem=AllocMem;	   // Points to GlobalAlloc()
 HTMLExpand(pECB,&ExtFC,cZTMName,cININame,cLOGName,cERRName);
 BOOL bTempFileExists=TempFileExists(cTempFileName,(DWORD) pECB);
 dwHeaderLen=sizeof(cHTTPHeader);
 if (ExtFC.pFilterContext)
 {
  dwContentLen=strlen((char*) ExtFC.pFilterContext);
  sprintf(cHTTPHeader,"Content-type: text/html\r\nContent-Length: %i\r\n",dwContentLen);
  if (bTempFileExists)
  {
   char cEntryName[8],cHeaderNameAndValue[MAX_HNAME+MAX_HVALUE];
   strcpy(cEntryName,"cA");  
   for (int i=1;i<=MAX_HBLOCK;i++)
   {
    cEntryName[1]=(char) 64+i; // cA, cB, cC, etc...
    GetPrivateProfileString("AddHeader",cEntryName,pcEmpty,cHeaderNameAndValue,sizeof(cHeaderNameAndValue),cTempFileName);
    if (cHeaderNameAndValue[0])
    {
     strcat(cHTTPHeader,cHeaderNameAndValue); // append new header
     strcat(cHTTPHeader,"\r\n");			  // and \r\n to close it properly
    }
    else break;
   };
  }; // TempFileExists!
  pECB->ServerSupportFunction(pECB->ConnID,HSE_REQ_SEND_RESPONSE_HEADER,NULL,&dwHeaderLen,(LPDWORD) cHTTPHeader);
  pECB->WriteClient(pECB->ConnID,(LPVOID) ExtFC.pFilterContext,&dwContentLen,0);
  GlobalFree((HGLOBAL) ExtFC.pFilterContext);		 
 }
 else
 {
  Report(EVENTLOG_ERROR_TYPE,300,"Script '%s' generated no output",cZTMName,NULL,NULL,0);
  CreateDebugPage(cZTMErrPage,pECB,"HTMLEx Error #300","<h3>Script generated no output.</h3>");
  dwContentLen=strlen(cZTMErrPage);
  sprintf(cHTTPHeader,"Content-type: text/html\r\nContent-Length: %i\r\n\r\n",dwContentLen);
  pECB->ServerSupportFunction(pECB->ConnID,HSE_REQ_SEND_RESPONSE_HEADER,NULL,&dwHeaderLen,(LPDWORD) cHTTPHeader);
  pECB->WriteClient(pECB->ConnID,(LPVOID) cZTMErrPage,&dwContentLen,0);
 };
 if (bTempFileExists && !bDebugMode) DeleteFile(cTempFileName);
 return HSE_STATUS_SUCCESS;
};
/////////////////////////////////////////////
//			   THAT'S ALL FOLKS!
/////////////////////////////////////////////
